/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.common.context;

import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Synchro client plugin description.
 * The description is build with the request User-Agent information.
 * OpenMIS synchro client has a specific User-Agent encoding to get
 * Client terminal informations. This class implements the User-Agent decoding.
 * Use to get information on the client configuration.
 * If the client isn't an OpenMIS client the user agent is not decoded and only the getClientPlateformString()
 * method will return the user agent. Other method will return null.
 * Use isOPenMISClient() to see if it's an OpenMIS client.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class UserTerminal  {

  public static final int SYNCHRO_TYPE_INDIRECT = 0;
  public static final int SYNCHRO_TYPE_DIRECT = 1;

  private boolean plugIns;
  private String clientOS;
  private String clientOSVersion;
  private String clientPlugInsVersion;
  private int synchroType;
  private String language;
  private String useragent;
  private String clientPlugInName;
  
  public static String getUserAgentFromRequest(HttpServletRequest request)  {
    String ua = (String) request.getHeader("User-Agent");
    if (ua == null) {
      ua = (String) request.getHeader("user-agent");
    }
    return ua;
  }


  public UserTerminal(String browserUserAgent) throws Exception {
    // user agent
    //OpenMSPPlug-<OpenMobileISMajorVersion>.<OpenMobileISMinorVersion>-<OS>[<OSVersion>]-<user language>-<synchro type if present>
    this.useragent = browserUserAgent;
    
		if (browserUserAgent.indexOf("OpenMSPPlug")!= -1) {
			plugIns = true;		
      clientPlugInName = "OpenMSPPlug";
    } 
    if (plugIns)  {

      StringTokenizer strToken = new StringTokenizer(browserUserAgent, "-");

      String token = null;
      if (strToken.hasMoreTokens())  { //OpenMSPPlug
         token = strToken.nextToken();
      }
      if (strToken.hasMoreTokens())  { //OpenMobile IS Version
        token = strToken.nextToken();
        this.clientPlugInsVersion=token;
       }
       if (strToken.hasMoreTokens())  { //OS and OSVersion
          token = strToken.nextToken();
          int index  = token.indexOf('[');
          this.clientOS = token.substring(0, index);
          this.clientOSVersion = token.substring(index+1, token.length()-1);
       }
       if (strToken.hasMoreTokens())  { //language
         token = strToken.nextToken();
         this.language=token;
        }
      // manage synchro type
      if (strToken.hasMoreTokens())  {
        token = strToken.nextToken();
        if (token.equals("PC")) {
          synchroType = UserTerminal.SYNCHRO_TYPE_INDIRECT;
        } else  {
          synchroType = UserTerminal.SYNCHRO_TYPE_DIRECT;
        }
      }
		}
  }

  /**
   * return the general plateform type of the client PALMOS or PPC for example,
   */
  public String getClientPlateformType() {
    return this.clientOS;
  }

  /**
   * return the synchro type. Client.SYNCHRO_TYPE_INDIRECT or Client.SYNCHRO_TYPE_DIRECT
   */
  public int getSynchroType() {
    return this.synchroType;
  }

  /**
   * return true if the client is an OpenMIS client
   */
  public boolean isOPenMISClient() {
    return plugIns;
  }

  public String getClientLanguage() {
    return this.language;
  }

  /**
   * return the client synchro plug-ins version
   */
  public String getClientPlugInsVersion() {
    return clientPlugInsVersion;
  }

  /**
   * return the User-Agent
   */
  public String getUserAgent() {
    return this.useragent;
  }

   public String getClientOS() {
    return this.clientOS;
  }

  public boolean equals(Object o) {
    if (o instanceof UserTerminal)  {
      return (((UserTerminal)o).getUserAgent().equals(useragent));
    }
    return false;
  }

  public int hashCode() {
    return useragent.hashCode();
  }


  public String getClientOSVersion() {
    return clientOSVersion;
  }


  public String getClientPlugInName() {
    return clientPlugInName;
  }


}