/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.common.user;

import java.util.Hashtable;

import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.util.exception.ServiceException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: generic user service. Use UserManagerDB to get user data
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class UserManager {
  private static UserManager manager;
//  private UserManagerFactory factory;
  private Hashtable factoryList;

  private UserManager() {
    factoryList = new Hashtable(10);
  }

  public static UserManager getManager() {
    if (manager == null)  {
      synchronized (UserManager.class) {
        if (manager == null)  {
          manager = new UserManager();
          ApplicationContextManager.getManager().addManager(manager);
        }
      }
    }
    return manager;
  }
  
  public void registerUserManagerFactoryForGroup(String group, UserManagerFactory factory) {
    factoryList.put(group, factory);
  }
  
  /**
   * Authenticate the login/pass for group.
   * return the authenticated userId if ok.
   * @param group
   * @param login
   * @param pass
   * @return
   * @throws ServiceException
   * @throws UserNotFoundException
   */
  public String authenticateUser(String group, String login, String pass) throws ServiceException, UserNotFoundException {
    UserManagerFactory factory = (UserManagerFactory) factoryList.get(group);
    if (factory != null)  {
      return factory.authenticateUser(login, pass);
    } else  {
      throw new UserNotFoundException("No UserManagerFactory registered for group :"+group);
    }
  }

  public void storeUser(String group, User user) throws ServiceException, UserAlreadyExistException {
    UserManagerFactory factory = (UserManagerFactory) factoryList.get(group);
    if (factory != null)  {
      factory.storeUser(user);
    } else  {
      throw new ServiceException("No UserManagerFactory registered for group :"+group);
    }
  }

  public User getUser(String group, String userID) throws ServiceException, UserNotFoundException {
    UserManagerFactory factory = (UserManagerFactory) factoryList.get(group);
    if (factory != null)  {
      return factory.getUser(userID);
    } else  {
      throw new UserNotFoundException("No UserManagerFactory registered for group :"+group);
    }
  }
  
  public void deleteUser(String group, String userID) throws ServiceException, UserNotFoundException {
    UserManagerFactory factory = (UserManagerFactory) factoryList.get(group);
    if (factory != null)  {
      factory.deleteUser(userID);
    } else  {
      throw new UserNotFoundException("No UserManagerFactory registered for group :"+group);
    }
  }
}