/*
 *        OpenMobileIS - a free Java Framework for mobile applications
 *
 *   Copyright (C) 2002  Philippe Delrieu.
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Philippe Delrieu kept the rigth to distribute all code Copyrighted by philippe Delrieu
 *  under other licence term even commercial one.
 *  
 *  Modifications :
 *  2002 Creation P.Delrieu
 * 
 */

package org.openmobileis.common.user.profil.impl;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

import java.util.*;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.openmobileis.common.user.profil.Profil;
import org.openmobileis.common.user.profil.ProfilDataFactory;
import org.openmobileis.common.user.profil.ProfilProperty;
import org.openmobileis.common.user.profil.ProfilRubric;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.log.*;
import org.openmobileis.common.util.file.FileUtilities;
import org.openmobileis.common.util.exception.DefaultException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Manage profil data in an xml file.

 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class XmlFileProfilDataFactory implements ProfilDataFactory {
  private HashMap profilList;

  class ProfilListParser  extends DefaultHandler  {
    private String currentProfilName;
    private HashMap rubricList;
    private ArrayList propertyList;

    ProfilListParser()  {
      rubricList = new HashMap(10);
      propertyList = new ArrayList(10);
    }

    /** Start document. */
    public void startDocument() {
    } // startDocument()

    /** Start element. */
    public void startElement(java.lang.String uri,
                         java.lang.String qname,
                         java.lang.String name,
                         Attributes attrs)
                  throws SAXException {
      if (name.equals("PROFILNAME")) {
        rubricList.clear();
        propertyList.clear();
        currentProfilName = attrs.getValue("name");
        if (currentProfilName == null)  {
          throw new SAXException("TAG PROFILNAME with no attribut name");
        }
      } else if (name.equals("ENTRY"))  {
        String channel = attrs.getValue("channelName");
        if (channel == null)  {
          throw new SAXException("TAG ENTRY with no attribut channelName");
        }
        String rubric = attrs.getValue("rubricName");
        if (rubric == null)  {
          throw new SAXException("TAG ENTRY with no attribut rubricName");
        }
        ProfilRubric rub = new ProfilRubric(channel, rubric, null, new ProfilProperty[0]);
        String description = attrs.getValue("description");
        if (description != null)  {
          rub.description = description;
        } else  {
          rub.description = "";
        }
        rubricList.put(channel+"/"+rubric, rub);
     } else if (name.equals("PROPERTY")) {
        String key = attrs.getValue("key");
        if (key == null)  {
          throw new SAXException("TAG PROPERTY with no attribut key for property");
        }
        String value = attrs.getValue("value");
        if (value == null)  {
          throw new SAXException("TAG PROPERTY with no attribut value for property");
        }
        ProfilProperty prop = new ProfilProperty(key, value);
        String channelName = attrs.getValue("channel");
        if (channelName != null)  {
          String rubricName = attrs.getValue("rubric");
          if (rubricName == null) {
           throw new SAXException("TAG PROPERTY with no attribut rubric for property");
          }
          String channelRub = channelName+"/"+rubricName;
          ProfilRubric profRub = (ProfilRubric)rubricList.get(channelRub);
          if (profRub == null)  {
            throw new SAXException("TAG PROPERTY no Rubric define for a rubric property "+channelRub);
          }
          ProfilProperty[] oldList = profRub.propertiesList;
          ProfilProperty[] newList = new ProfilProperty[oldList.length+1];
          System.arraycopy(oldList, 0, newList, 0, oldList.length);
          newList[oldList.length] = prop;
          profRub.propertiesList = newList;

        } else  {
          propertyList.add(prop);
        }
      }
    } // startElement(String,AttributeList)

    public void endElement (String uri, String qname, String localName) throws SAXException {
      if (localName.equals("PROFILNAME")) {
        ProfilRubric[] rubs = new ProfilRubric[rubricList.size()];
        rubricList.values().toArray(rubs);
        ProfilProperty[] props = new ProfilProperty[propertyList.size()];
        propertyList.toArray(props);
        Profil newidlprofil = new Profil("0", props, rubs);
        profilList.put(currentProfilName, newidlprofil);
      }
    }


    /** Characters. */
    public void characters(char ch[], int start, int length) {
    }
  }

  public XmlFileProfilDataFactory() {
    profilList = new HashMap(10);
    try {
      String file =  PropertiesManager.getManager().getProperty("org.openmobileis.common.user.profil.xmlprofilfile.path");
      if (file == null) {
        LogManager.traceError(LogServices.DEFAULTSERVICE, "No profil list XML file defined. Profil management is not initialized correctly");
        return;
      }
      file = FileUtilities.convertFileNameToSystem(file);
      ProfilListParser handler = new ProfilListParser();
      SAXParserFactory factory = SAXParserFactory.newInstance();
      // Parse the input 
      SAXParser saxParser = factory.newSAXParser();
      saxParser.parse(new InputSource(new java.io.FileReader(file)), handler);
    } catch (Exception e) {
      LogManager.trace(new DefaultException ("Problem occurs during loading profil list file", e));
    }
  }

  /**
   * return the profil for specified profil name.
   * null if not found
   */
  public Profil getProfil(String profilName)  {
    return (Profil) profilList.get(profilName);
  }
}