/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.common.util.collection;

import java.util.Vector;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  sort algorythms
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class Sort {

	// no one should instantiate this class

	private Sort() {}

	/**
	 * Sort a Vector of Object references according to the ordering
	 * specified by the Orderable object instance method compareTo().
	 *
	 * @param		vec the vector of Objects
	 * @param		op a reference to an Orderable object instance
	 * @exception	IllegalArgumentException for null arguments
	 * @see			Orderable
	 */

	public synchronized static void shellsort(Vector vec, Orderable op)
	{
		if (vec == null || op == null)
			throw new IllegalArgumentException("null argument");

		int n = vec.size();
		if (n < 2)
			return;

		// copy out Vector slots to speed things up
    int i = 0;
		Object new_vec[] = new Object[n];
		for (i = 0; i < n; i++)
			new_vec[i] = vec.elementAt(i);

    shellsort(new_vec, op);

		// copy back to Vector

		for (i = 0; i < n; i++)
			vec.setElementAt(new_vec[i], i);
	}


	/**
	 * Sort a CyberArray of Object references according to the ordering
	 * specified by the Orderable object instance method compareTo().
  * use shellSort
	 *
	 * @param		vec the vector of Objects
	 * @param		op a reference to an Orderable object instance
	 * @exception	IllegalArgumentException for null arguments
	 * @see			Orderable
	 */

	public synchronized static void shellsort(Array array, Orderable op)	{
		if (array == null || op == null)
			throw new IllegalArgumentException("null argument");

		int n = array.size();
		if (n < 2)
			return;

		// copy out Vector slots to speed things up
		Object new_vec[] = new Object[n];
		System.arraycopy(array.getArrayElements(),0, new_vec, 0, n);

    shellsort(new_vec, op);

		// copy back to Vector
    array.setArrayElements(new_vec);
	}

  public static void shellsort(Object[] data, Orderable op)  {
		// sort using Shellsort
		int h = 1;
		int i = 0;
		int n = data.length;


		while (h <= n)
			h = h * 3 + 1;
		while (h > 1) {
			h /= 3;
			for (i = h + 1; i <= n; i++) {
				Object v = data[i - 1];
				int j = i;
				while (op.compareTo(data[j - h - 1], v) > 0) {
					data[j - 1] = data[j - h - 1];
					j -= h;
					if (j <= h)
						break;
				}
				data[j - 1] = v;
			}
		}
  }

}
