/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.common.util.file;

import java.io.*;

/**
 * contains various utilities method related to file use.
 */
public class FileUtilities {
  private static final String emptyString = "";

  /**
    replace all nom system file separator by system file separator
    use for windows system to convert zip file name to windews file name
  */
  public static String convertFileNameToSystem(String zipFile) {
    String ret =null;
    switch (File.separatorChar) {
      case '/' :
        ret = zipFile.replace('\\', '/');
        break;
      case '\\' :
        ret = zipFile.replace('/', '\\');
        break;
      default : 
        ret = zipFile;
        break;
    }

    return ret;
  }

  /**
   * return the path extracted from the specified complete file name. The completeFileName must be conforme to the
   * system file name (path+separator+fileName).
   * Path end with the System separator.
   * If not path is specified return empty string.
   */
  public static String getPathFromCompleteFileName(String completeName)   {
    int index = completeName.lastIndexOf(File.separatorChar);
    if (index != -1)  {
      return completeName.substring(0, index+1);
    } else {
      return emptyString;
    }
  }

  /**
   * return the name of the file extracted from the specified complete file name. The completeFileName must be conforme to the
   * system file name (path+SystemSeparator+fileName).
   * if the file has not path return the CompleteFileName
   * if the CompleteFileName has no fileName return empty string.
   */
  public static String getNameFromCompleteFileName(String completeName)   {
    int index = completeName.lastIndexOf(File.separatorChar);
    if (index == -1)  {
      return completeName;
    } else {
      if (index != completeName.length()) {
        return completeName.substring(index+1, completeName.length());
      }
    }
    return emptyString;
  }

  /**
   * move file from old name to new name. If new fileExist delete it before move
   */
  public static void moveFile(String oldFileName, String newFileName)  {
    File newFile = new File(newFileName);
		 
    if (newFile.exists()) {
    	newFile.delete();
    }
    File oldFile = new File(oldFileName);
    oldFile.renameTo(newFile);
    if (oldFile.exists()) {
      oldFile.delete();
    }
  }

  public static String concertToUnixFomat(String filePath)  {
    return filePath.replace('\\', '/');
  }

	public static byte[] readFile(String completeFileName) throws FileNotFoundException, IOException {
		FileInputStream fileStream = new FileInputStream(completeFileName);
		try {
			//read channel XML
			byte[] buffer = new byte[1024];
			ByteArrayOutputStream stream = new ByteArrayOutputStream(buffer.length);
			int bytes_read;
			try {
				while(( bytes_read = fileStream.read(buffer)) != -1)  // Copy bytes to zipfile
					stream.write(buffer, 0, bytes_read);
			} finally {
				stream.flush();
				stream.close();
			}
			return stream.toByteArray();
		} finally {
			fileStream.close();
		}
	}

	public static byte[] readFile(File file) throws FileNotFoundException, IOException {
		FileInputStream fileStream = new FileInputStream(file);
		try {
			//read channel XML
			byte[] buffer = new byte[1024];
			ByteArrayOutputStream stream = new ByteArrayOutputStream(buffer.length);
			int bytes_read;
			try {
				while(( bytes_read = fileStream.read(buffer)) != -1)  // Copy bytes to zipfile
					stream.write(buffer, 0, bytes_read);
			} finally {
				stream.flush();
				stream.close();
			}
			return stream.toByteArray();
		} finally {
			fileStream.close();
		}
	}

  /**
   * remove all file with the specified ext starting at file directory.
   * If recursif is true remove inside sub directory.
   * if ext is null, remove all file.
   */
  public static final void removeAllFile(File file, String ext, boolean recurcif) {
    if ((recurcif) &&(file.isDirectory())) {
      File[] fileList = file.listFiles();
      for (int i=0; i<fileList.length; i++) {
        removeAllFile(fileList[i], ext, recurcif);
      }
    }
    if (ext != null)  {
      if (file.getName().indexOf(ext) != -1)  {
        file.delete();
      }
    } else  {
      file.delete();
    }
  }

/*  public static final void moveFile(String sourcePath, String destPath)  throws FileNotFoundException, IOException {
    InputStream sourceFile = new BufferedInputStream(new FileInputStream(sourcePath));
    try {
      OutputStream destFile = new  BufferedOutputStream(new FileOutputStream(destPath));
      try {
        byte[] buffer = new byte[1024];
        int bytes_read;
        while(( bytes_read = sourceFile.read(buffer)) != -1)  // Copy bytes to zipfile
          destFile.write(buffer, 0, bytes_read);
      } finally {
        destFile.flush();
        destFile.close();
      }
    } finally {
      sourceFile.close();
    }
    File toRemove = new File(sourcePath);
  }*/

  public static final void copyFile(String sourcePath, String destPath)  throws FileNotFoundException, IOException {
    // create destination directory if not exist
    int index = destPath.lastIndexOf(File.separatorChar);
    if (index != -1)  {
      String dirPath = destPath.substring(0, index);
      if (dirPath.length() !=0) {
        File dirfile = new File(dirPath);
        if (!dirfile.exists()) {
          dirfile.mkdirs();
        }
      }
    }

    InputStream sourceFile = new BufferedInputStream(new FileInputStream(sourcePath));
    try {
      OutputStream destFile = new  BufferedOutputStream(new FileOutputStream(destPath));
      try {
        byte[] buffer = new byte[1024];
        int bytes_read;
        while(( bytes_read = sourceFile.read(buffer)) != -1)  // Copy bytes to zipfile
          destFile.write(buffer, 0, bytes_read);
      } finally {
        destFile.flush();
        destFile.close();
      }
    } finally {
      sourceFile.close();
    }
  }

}