/*
 *        OpenMobileIS - a free Java Framework for mobile applications
 *
 *   Copyright (C) 2002  Philippe Delrieu.
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Philippe Delrieu kept the rigth to distribute all code Copyrighted by philippe Delrieu
 *  under other licence term even commercial one.
 *  
 *  Modifications :
 *  2002 Creation P.Delrieu
 * 
 */

package org.openmobileis.common.util.file;

import java.io.*;
import java.util.zip.*;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class ZipUtilities {

  /**
   * unzip the input data in the specified directory. Directory must be terminated by the end file separator.
   */
  public static void unZipData(InputStream inputstream, String unZipDirectory) throws IOException  {
    ZipInputStream zipFile = new ZipInputStream(inputstream);
    ZipEntry entry;
    while ( (entry = zipFile.getNextEntry()) != null) {
      String completeName = entry.getName();
      // get file directory and test if exist.
      String plateformFileName = FileUtilities.convertFileNameToSystem(completeName);
      if (entry.isDirectory()) {
    	  File dirToCreate = new File(unZipDirectory+plateformFileName);
    	  if (!dirToCreate.exists()) {
    		  dirToCreate.mkdirs();
    	  }
    	  zipFile.closeEntry();
    	  continue;
      }
      int index = completeName.lastIndexOf("/");
      if (index != -1)  {
        File repToCreate = new File(unZipDirectory+plateformFileName.substring(0, index));
        if (!repToCreate.exists())  {
          repToCreate.mkdirs();
        }
      }
      FileOutputStream stream = new FileOutputStream(unZipDirectory+plateformFileName);
      try {
        int read_bytes = 0;
        byte[] buffer = new byte[2048];
        while ( (read_bytes = zipFile.read(buffer)) != -1) {
          stream.write(buffer,0, read_bytes);
        }
      } finally {
        stream.close();
      }
      zipFile.closeEntry();
    }
    zipFile.close();
  }
  
  public static void unzipEntryFromDataToDestFile(InputStream inputstream, String entryName, String destFile) throws IOException {
      ZipInputStream zipFile = new ZipInputStream(inputstream);
      ZipEntry entry;
      while ( (entry = zipFile.getNextEntry()) != null) {
        String completeName = entry.getName();
        if (completeName.equals(entryName))  {
            FileOutputStream stream = new FileOutputStream(destFile);
            try {
              int read_bytes = 0;
              byte[] buffer = new byte[2048];
              while ( (read_bytes = zipFile.read(buffer)) != -1) {
                stream.write(buffer,0, read_bytes);
              }
            } finally {
              stream.close();
            }
        }
        zipFile.closeEntry();
      }
      zipFile.close();
  }


  public static byte[] getZipEntryFromStreal(InputStream inputstream, String entryName) throws IOException  {
    ZipInputStream zipFile = new ZipInputStream(inputstream);
    ZipEntry entry;
    byte[] outData = null;
    while ( (entry = zipFile.getNextEntry()) != null) {
      String completeName = entry.getName();
      if (completeName.equals(entryName))  {
        ByteArrayOutputStream stream = new ByteArrayOutputStream();
        try {
          int read_bytes = 0;
          byte[] buffer = new byte[2048];
          while ( (read_bytes = zipFile.read(buffer)) != -1) {
            stream.write(buffer,0, read_bytes);
          }
        } finally {
          stream.close();
        }
        outData = stream.toByteArray();
        zipFile.closeEntry();
        break;
      }
      zipFile.closeEntry();
    }
    zipFile.close();
    return outData;
  }

}