/*
 *        OpenMobileIS - a free Java Framework for mobile applications
 *
 *   Copyright (C) 2002  Philippe Delrieu.
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Philippe Delrieu kept the rigth to distribute all code Copyrighted by philippe Delrieu
 *  under other licence term even commercial one.
 *  
 *  Modifications :
 *  2002 Creation P.Delrieu
 * 
 */

package org.openmobileis.common.util.log;

import java.util.Properties;
import java.io.*;

/**
 * Title:        OpenMobileIS project source <BR>
 * Implement a LogTracer that Log in a File. The File is define by specifing the property LOGFILE.
 * Implements an archiving system. Define the property LOGDIRARCHIVE to define the directory where log are archived.
 * @author P.Delrieu
 * @version 1.0
 * @since JDK1.1
 */
public class FileOpenCloseLogTracer extends DefaultLogTracer {

  protected FileOutputStream fileStream;
  protected String logFile;

  public FileOpenCloseLogTracer() {
  }
  /**
   * Two property are defined :
   *  LOGFILE : file where log are written.
   *  LOGDIRARCHIVE : directory where old log are archived.
   */

  public void setLogTracerProperties(Properties props)  {
    try {
      logFile = props.getProperty("org.openmobileis.common.log.file");
      if (logFile == null)  {
        System.out.println("LOGFILE property is not defined");
        System.out.println("LOG to console");
        outStream = System.out;
        return;
      }
      File logfile = new File(logFile);
      if (logfile.exists()) {
        String archivedir = props.getProperty("org.openmobileis.common.log.archivefile");
        if (archivedir == null) {
          System.out.println("LOGDIRARCHIVE property is not defined. Old log file is not archived");
        } else {
          int index = logFile.lastIndexOf(File.separator);
          String fileName = logFile;
          if (index !=-1) {
            fileName = logFile.substring(index+1);
          }
          java.text.SimpleDateFormat formatter
            = new java.text.SimpleDateFormat ("dd-MM-yy#hh-mm-ss", java.util.Locale.FRANCE);
          String dateString = formatter.format(new java.util.Date());
          String logArchiveFile = archivedir+dateString+"-"+fileName;
          FileInputStream input  = new FileInputStream(logFile);
          FileOutputStream output = new FileOutputStream(logArchiveFile);
          try {
            byte[] buf = new byte[4096];
            int len = 0;
            while ( ( len = input.read( buf ) ) != -1 )
              output.write( buf, 0, len );
            output.flush();
          } finally {
            input.close();
            output.close();
          }
        }
      }
      outStream = System.out;
//      System.setOut(outStream);
//      System.setErr(outStream);
    } catch (Exception e) {
      System.out.println("Error in FileTraceManager Init" +e);
    }
  }

	/**
	 * Trace the message to the stream defined showing the specified priority and service.
	 */
	public synchronized void trace(int service, int priority, String message)  {
		try	{
			fileStream = new FileOutputStream(logFile, true);
			try	{
				outStream = new PrintStream(fileStream, true);
				super.trace(service, priority, message);
			} finally	{
				outStream = System.out;
				fileStream.flush();
				fileStream.close();
			}
		} catch (Throwable ex)	{
			ex.printStackTrace();
		}

	}
  
  public File getLogFile()	{
    return  new File(logFile);
  }

}