/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */


package org.openmobileis.common.util.log;


/**
 * Define priority code use by LogService to dispatch log message.
 * @author P.Delrieu
 * @version 1.0
 * @since JDK1.1
 */
public class LogPriorities {
  /**
   * Message urgent. Le systme est initilisable ou risque de le devenir  trs court terme.
   */
  public final static int EMERGENCY=-3;

  /**
   * Message alertant l'administrateur systme qu'une action de sa part est requise.
   */
  public final static int ALERT=-2;

  /**
   * Message critique. Une action rapide doit tre ralise.
   */
  public final static int CRITIQUE=-1;

  /**
   * Message d'erreur. Une erreur est survenu ne gnant pas le fonctionement global du systme.
   */
  public final static int ERROR=0;

  /**
   * Message d'avertissement.
   */
  public final static int WARNING=1;

  /**
   * Message de fonctionement normal, sans gravit particulire.
   */
  public final static int NOTICE=2;

  /**
   * Message  titre informatif.
   */
  public final static int INFO=3;

  /**
   * Message de dbogage.
   */
  public final static int DEBUG=4;

  /**
   * log spriority use to log user synchro data
   */
  public static int USERSYNCHRO = 5;

  public final static String toString(int service)  {
    String ret = "";
    switch (service)  {
      case EMERGENCY :
        ret = "EMERGENCY";
        break;
      case ALERT :
        ret = "ALERT";
        break;
      case CRITIQUE :
        ret = "CRITIQUE";
        break;
      case ERROR :
        ret = "ERROR";
        break;
      case WARNING :
        ret = "WARNING";
        break;
      case NOTICE :
        ret = "NOTICE";
        break;
      case DEBUG :
        ret = "DEBUG";
        break;
      default :
        ret = "INFO";
        break;
    }
    return ret;
  }

}