/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb;

/**
 * Use to describe collection index properties.
 * FODB create index by using the FODBIndexDescriptor provided.
 * 
 * Name : define the collection to attach the index
 * memberName : define the field or method of the collection object that has to be indexed 
 * type : define the index type : UNIQUE or MULTIPLE
 * order : define the order of the Btree index. By default use 15 for collection of Thousand object.
 *   For greater collection increase the order. 
 */
public abstract class FODBIndexDescriptor implements java.io.Serializable	{
	static final long serialVersionUID = 5521257935120563452L;
	public static final int UNIQUE = 1;
	public static final int MULTIPLE = 2;
	private String name;
	private String memberName;
	private int type;
	private int order;
	
	public FODBIndexDescriptor(String newName, int indexType, String newMemberName, int btreeorder) {
		//correct Btree algo pb. Order must be pair
		if (btreeorder%2 != 0)	{
			btreeorder = btreeorder+1;
		}
		this.order = btreeorder;
		name = newName;
		this.type = indexType;
		memberName = newMemberName;
	}
	
	public String getName() {
		return name;
	}
	
	public String getMemberName() {
		return memberName;
	}
	
	public int getType() {
		return type;
	}
	
	public boolean isUnique() {
		return (type == UNIQUE) ? true : false;
	}
	
	public boolean isMultiple() {
		return (type == MULTIPLE) ? true : false;
	}

	public int getOrder() {
		return order;
	}
}
