/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.index;

import java.io.*;

/**
 * Header of Btree Page stored in file.
 * @author	Philippe Delrieu
 * @modifications : Romain Beaugrand
 * @since	JDK 1.1
 * @version 1.0.
 */
public class FODBIndexHeader   implements Externalizable
{
    static final long serialVersionUID = 3138616672520914061L;

    //---------------------
    // fields
    //---------------------
		private String name;
    private String memberName;
    public int type;
    public int  order;
    public int  keyLen;
    public int incTabSize;
    public long pos;
    public long rootPtr;
    
    public FODBIndexHeader() {
    }
    
    public FODBIndexHeader(String iName, String mName) {
    	name = iName;
    	memberName = mName;
    }
    
    public String getName() {
    	return name;
    }
    
    public String getMemberName() {
    	return memberName;
    }
    
	public void writeExternal(ObjectOutput out) throws IOException {
		int i;
		
		for (i = 0; i < name.length(); i++) {
			out.writeChar(name.charAt(i));
		}
		out.writeChar('\0');
		
		for (i = 0; i < memberName.length(); i++) {
			out.writeChar(memberName.charAt(i));
		}
		out.writeChar('\0');
		
		out.writeInt(type);
		out.writeInt(order);
		out.writeInt(keyLen);
		out.writeInt(incTabSize);
		out.writeLong(pos);
		out.writeLong(rootPtr);
	}
	
	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
		char parseChar;
		
		StringBuffer tmpName = new StringBuffer();
		while ((parseChar = in.readChar()) != '\0') {
			tmpName.append(parseChar);
		}
		name = tmpName.toString();
		
		StringBuffer tmpMemberName = new StringBuffer();
		while ((parseChar = in.readChar()) != '\0') {
			tmpMemberName.append(parseChar);
		}
		memberName = tmpMemberName.toString();
		
		type = in.readInt();
		order = in.readInt();
		keyLen = in.readInt();
		incTabSize = in.readInt();
		pos = in.readLong();
		rootPtr = in.readLong();
	}
}
