/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.index;

import java.io.IOException;
import java.lang.reflect.*;

import org.openmobileis.common.util.collection.LongArray;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.fastobjectdb.FODBIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FODBIntIndexDescriptor;
import org.openmobileis.database.fastobjectdb.db.exception.FODBException;
import org.openmobileis.database.fastobjectdb.db.exception.FODBQueryException;
import org.openmobileis.database.fastobjectdb.db.index.node.IntNode;
import org.openmobileis.database.fastobjectdb.db.index.node.Node;
import org.openmobileis.database.fastobjectdb.db.query.soda.SodaIndexComparator;
import org.openmobileis.database.fastobjectdb.db.query.soda.SodaIntIndexComparator;
import org.openmobileis.database.fastobjectdb.db.store.FODBCollectionIndexFile;

/**
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public abstract class FODBIntIndex extends FODBIndex {

  public FODBIntIndex(FODBIndexHeader newHeader, FODBCollectionIndexFile cFile, AccessibleObject accObj) throws FODBException {
    super(newHeader, cFile, accObj);
  }

  public FODBIntIndex(FODBIntIndexDescriptor descriptor, FODBCollectionIndexFile cFile, AccessibleObject accObj) throws FODBException {
    super(descriptor, cFile, accObj);
  }

  protected void specificDescriptorVerifications(FODBIndexDescriptor descriptor) throws FODBException {
    if (!(descriptor instanceof FODBIntIndexDescriptor)) {
      throw new FODBException("Internal Error");
    }

    FODBIntIndexDescriptor desc = (FODBIntIndexDescriptor) descriptor;

    // verify parameters
    if (desc.getOrder() < 3)
      throw new FODBException("ERROR Bad BTree order. BTree order must be more than 3");
  }

  protected Node initRoot(FODBIndexDescriptor descriptor) throws FODBException {
    FODBIntIndexDescriptor desc = (FODBIntIndexDescriptor) descriptor;
    return new IntNode(desc.getOrder());
  }

  protected void specificHeaderInit(FODBIndexDescriptor descriptor)  throws FODBException {
    FODBIntIndexDescriptor desc = (FODBIntIndexDescriptor) descriptor;
    header.order = desc.getOrder();
  }

  public Object getKey(Object obj) throws FODBException {
    int key = 0;

    if (accessObj instanceof Method) {
      try {
        key = ((Integer) (((Method) accessObj).invoke(obj, new Object[0]))).intValue();
      } catch (InvocationTargetException ex) { 
        throw new FODBException("Unable to execute index's method", ex);
      } catch (IllegalAccessException ex) { 
        throw new FODBException("Unable to execute index's method", ex);
      }
    } else {
      try {
        key = (((Field) accessObj).getInt(obj));
      } catch (IllegalAccessException ex) {
        throw new FODBException("Unable to access index's field", ex);
      }
    }

    return new Integer(key);
  }

  protected void insertKey(Object obj, long ptr) throws IOException, ClassNotFoundException, FODBException {
    int key = ((Integer)obj).intValue();

    // search for the key
    IntNode root = (IntNode) this.readRoot();
    SearchResult searchResult = new SearchResult();
    searchResult = this.search(root, key, searchResult);
  //  long found = Node.NO_NODE;

    if (searchResult.found) {
      // As we called 'add', if we are in a UNIQUE index, we must
      // return an exception.
      if (getType() == UNIQUE) {
        throw new FODBException("Entry already present in Unique Index");
      }

      // store new data record pointer
      //		  long result = writeKeyPtr(ins.page, ins.pos, pos, oldPos);
      // if we're in a MULTIPLE index, result is :
      //	-> the new pos of the Pointer List if it grew up.
      //	-> NULL_PTR if it didn't.
      //		  long modified = this.writeKeyPtr(ins.page, ins.pos, pos, oldPos);
      this.writeKeyPtr(searchResult.node, searchResult.pos, ptr);
      return;
    }
    if (getType() != UNIQUE) {
      ptr = ((FODBMultipleIntIndex) this).createPtrArray(ptr);
    }
    IntNode btreenode = (IntNode) searchResult.node;
    boolean flag;
    for (flag = btreenode.pushInLeaf(key, searchResult.pos, ptr);
      flag && btreenode.parentPtr != Node.NO_NODE;
      btreenode = (IntNode) colFile.readNode(btreenode.parentPtr)) {
      int i1 = this.minKey;
      IntNode btreenode1 = new IntNode(header.order);
      btreenode1.parentPtr = btreenode.parentPtr;
      int i;
      for (i = i1 + 1; i < btreenode1.getMaxNbKey(); i++) {
        btreenode1.setKeyPtrAtPos(btreenode.getKeyAtPos(i), btreenode.getNodePtrAtPos(i), i - i1 - 1);
        btreenode1.branchs[i - i1 - 1] = btreenode.branchs[i];
      }

      btreenode1.branchs[i - i1 - 1] = btreenode.branchs[i];
      btreenode1.nbKey = btreenode1.getMaxNbKey() - i1 - 1;
      colFile.writeNode(btreenode1, header);
      if (btreenode1.branchs[0] != Node.NO_NODE) {
        for (int j = 0; j <= btreenode1.nbKey; j++) {
          IntNode tempNode = (IntNode) colFile.readNode(btreenode1.branchs[j]);
          tempNode.parentPtr = btreenode1.filePtr;
          colFile.writeNode(tempNode, header);
        }

      }

      btreenode.nbKey = this.minKey;
      colFile.writeNode(btreenode, header);
      int newkey = btreenode.getKeyAtPos(i1);
      long keyptr = btreenode.getNodePtrAtPos(i1);

      IntNode parentNode = (IntNode) colFile.readNode(btreenode.parentPtr);
      SearchResult searchResult1 = parentNode.searchNode(newkey, searchResult);
      flag = parentNode.promote(newkey, keyptr, searchResult1.pos, btreenode1.filePtr);
      colFile.writeNode(parentNode, header);

    }
    colFile.writeNode(btreenode, header);

    if (!flag)
      return;

    // work on root
    root = (IntNode) this.readRoot();
    IntNode parent = new IntNode(header.order);
    colFile.writeNode(parent, header);
    int j1 = this.minKey;
    IntNode btreenode2 = new IntNode(header.order);
    btreenode2.parentPtr = parent.filePtr;
    int k;
    for (k = j1 + 1; k < btreenode2.getMaxNbKey(); k++) {
      btreenode2.setKeyPtrAtPos(root.getKeyAtPos(k), root.getNodePtrAtPos(k), k - j1 - 1);
      btreenode2.branchs[k - j1 - 1] = root.branchs[k];
    }

    btreenode2.branchs[k - j1 - 1] = root.branchs[k];
    btreenode2.nbKey = btreenode2.getMaxNbKey() - j1 - 1;
    colFile.writeNode(btreenode2, header);
    if (btreenode2.branchs[0] != Node.NO_NODE) {
      for (int l = 0; l <= btreenode2.nbKey; l++) {
        IntNode tempNode = (IntNode) colFile.readNode(btreenode2.branchs[l]);
        tempNode.parentPtr = btreenode2.filePtr;
        colFile.writeNode(tempNode, header);
      }
    }
    root.nbKey = this.minKey;
    root.parentPtr = parent.filePtr;
    colFile.writeNode(root, header);
    parent.nbKey = 1;
    parent.setKeyPtrAtPos(root.getKeyAtPos(j1), root.getNodePtrAtPos(j1), 0);
    parent.branchs[0] = root.filePtr;
    parent.branchs[1] = btreenode2.filePtr;
    colFile.writeNode(parent, header);
    return;
  }

  //-------------------------
  // private utility methods
  //-------------------------

  // find a node
  private SearchResult search(IntNode node, int key, SearchResult searchResult) throws IOException, ClassNotFoundException {
    int pos = 0;

    if (node.nbKey == 0) {
      searchResult.found = false;
      searchResult.node = node;
      searchResult.pos = pos;
      return searchResult;
    }
    searchResult = node.searchNode(key, searchResult);
    if (searchResult.found)
      return searchResult;
    if (searchResult.node.branchs[0] == Node.NO_NODE) {
      return searchResult;
    } else {
      IntNode newNode = (IntNode) colFile.readNode(node.branchs[searchResult.pos]);
      //verify parent ptr of the node
      if (newNode.parentPtr != node.filePtr) {
        LogManager.traceError(0, "ERROR BAD NODE PARENT");
      }
      searchResult = this.search(newNode, key, searchResult);
      return searchResult;
    }
  }
	
	public SearchResult getNodeForKey(Object key) throws FODBException	{
		try	{
			IntNode root = (IntNode) this.readRoot();
			SearchResult searchResult = new SearchResult();
			searchResult = this.search(root, ((Integer)key).intValue(), searchResult);
			return searchResult;		
		} catch (Throwable ex)	{
			throw new FODBException(ex);
		}		
	}

	public long[] query(SodaIndexComparator comparator) throws FODBQueryException {
		try	{
			// find record
			LongArray array = new LongArray(15);
			IntNode root = (IntNode) this.readRoot();
			switch (comparator.getSearchAlgo())	{
				case SodaIndexComparator.INF_EQUALS_TRAVERSAL :
					this.searchLikeInfEquals(root, array, (SodaIntIndexComparator)comparator);
					break;
				case SodaIndexComparator.SUP_EQUALS_TRAVERSAL :
					this.searchLikeSupEquals(root, array, (SodaIntIndexComparator)comparator);
					break;
				case SodaIndexComparator.FULL_TRAVERSAL :
					this.searchLikeFull(root, array, (SodaIntIndexComparator)comparator);
					break;
				default :
					break;
			}
			
	
			int size = array.size();
	
			if (size == 0) {
				return new long[0];
			}
	
			return array.toArray();
		} catch (Throwable ex)	{
			throw new FODBQueryException(ex);
		}
	}

	// find a node
	private void searchLikeInfEquals(IntNode node, LongArray array, SodaIntIndexComparator comparator) throws IOException, ClassNotFoundException {
		int pos = 0;

		if (node.nbKey == 0) {
			return;
		}

		while (true) {
			if (pos == node.nbKey)
				break;
			if (comparator.isSelected(node.getKeyAtPos(pos))) {
				addSearchResult(node, pos, array);
				if (node.branchs[pos] != Node.NO_NODE) {
					searchLikeInfEquals((IntNode)colFile.readNode(node.branchs[pos]), array, comparator);
				}
				++pos;
			} else {
				int comp = comparator.compareTo(node.getKeyAtPos(pos));
				if (comp < 0)
					++pos;
				else
					break;
			}
		}

		if (node.branchs[pos] == Node.NO_NODE) {
			return;
		} else {
			searchLikeInfEquals((IntNode)colFile.readNode(node.branchs[pos]), array, comparator);
		}
	}

	// find a node
	private void searchLikeSupEquals(IntNode node, LongArray array, SodaIntIndexComparator comparator) throws IOException, ClassNotFoundException {
		int pos = node.nbKey;  //begin at last key

		if (node.nbKey == 0) {
			return;
		}

		while (true) {
			if (pos == 0)
				break;
			if (comparator.isSelected(node.getKeyAtPos(pos-1))) {
				addSearchResult(node, pos-1, array);
				if (node.branchs[pos] != Node.NO_NODE) {
					searchLikeSupEquals((IntNode)colFile.readNode(node.branchs[pos]), array, comparator);
				}
				--pos;
			} else {
				int comp = comparator.compareTo(node.getKeyAtPos(pos-1));
				if (comp < 0)
					--pos;
				else
					break;
			}
		}

		if (node.branchs[pos] == Node.NO_NODE) {
			return;
		} else {
			searchLikeSupEquals((IntNode)colFile.readNode(node.branchs[pos]), array, comparator);
		}
	}

	private void searchLikeFull(IntNode pg, LongArray array, SodaIntIndexComparator comparator) throws IOException, ClassNotFoundException {
		int pos = 0;

		if (pg.nbKey == 0) {
			return;
		}

		while (true) {
			if (pos == pg.nbKey) {
				break;
			}
			if (comparator.isSelected(pg.getKeyAtPos(pos))) {
				addSearchResult(pg, pos, array);
			}

			if (pg.branchs[pos] != Node.NO_NODE) {
				searchLikeFull((IntNode) colFile.readNode(pg.branchs[pos]), array, comparator);
			}

			++pos;
		}

		if (pg.branchs[pos] == Node.NO_NODE) {
			return;
		} else {
			searchLikeFull((IntNode) colFile.readNode(pg.branchs[pos]), array, comparator);
		}
	}

  protected boolean deleteKey(Object obj, long dbptr) throws IOException, ClassNotFoundException, FODBException {
    int key = ((Integer)obj).intValue();
//    System.out.println("" + header.name + " delete : " + key);

    IntNode root = (IntNode) this.readRoot();
    SearchResult searchResult = new SearchResult();
    searchResult = this.search(root, key, searchResult);
    if (!searchResult.found) {
      return false;
    }
    boolean empty = this.removeKeyPtr(searchResult.node, searchResult.pos, dbptr);
    if (!empty) {
      return true;
    }
//    long found = searchResult.node.getNodePtrAtPos(searchResult.pos);

    IntNode btreenode = (IntNode) searchResult.node;
    if (btreenode.branchs[0] != Node.NO_NODE) { //swap key until we find a leaf.
      btreenode = (IntNode) colFile.readNode(btreenode.branchs[searchResult.pos + 1]);
      while (btreenode.branchs[0] != Node.NO_NODE) {
        btreenode = (IntNode) colFile.readNode(btreenode.branchs[0]);
      }
      IntNode resultNode = (IntNode) searchResult.node;
      resultNode.setKeyPtrAtPos(btreenode.getKeyAtPos(0), btreenode.getNodePtrAtPos(0), searchResult.pos);
      colFile.writeNode(resultNode, header);
      searchResult.node = btreenode; // work on the leaf.
      searchResult.pos = 0;
    }
    btreenode.removeKeyAtPpos(searchResult.pos);
    colFile.writeNode(btreenode, header);
    for (boolean flag = btreenode.nbKey < this.minKey; flag && (btreenode.parentPtr != Node.NO_NODE); flag = btreenode.nbKey < this.minKey) {
      //get the node branch index in parent.
      int i = 0;
      IntNode parentNode = (IntNode) colFile.readNode(btreenode.parentPtr);
      for (i = 0; parentNode.branchs[i] != btreenode.filePtr; i++);
      this.restore(parentNode, i);
      btreenode = parentNode;
    }

    if (btreenode.parentPtr != Node.NO_NODE)
      return true;
    if (btreenode.nbKey > 0)
      return true;

    if (btreenode.branchs[0] != Node.NO_NODE) {
      IntNode firstChild = (IntNode) colFile.readNode(root.branchs[0]);
      if (firstChild.nbKey > 0) {
        firstChild.parentPtr = Node.NO_NODE;
        colFile.writeNode(firstChild, header);
        colFile.deleteNode(btreenode);
        return true;
      }
    }
    return true;
  }

  // BTREE Manipulation method

  private void moveLeft(IntNode node, int i) throws IOException, ClassNotFoundException {
    IntNode btreenode = (IntNode) colFile.readNode(node.branchs[i]);
    btreenode.nbKey++;
    btreenode.setKeyPtrAtPos(node.getKeyAtPos(i), node.getNodePtrAtPos(i), btreenode.nbKey - 1);
    IntNode branch = (IntNode) colFile.readNode(node.branchs[i + 1]);
    btreenode.branchs[btreenode.nbKey] = branch.branchs[0];
    colFile.writeNode(btreenode, header);
    if (btreenode.branchs[btreenode.nbKey] != Node.NO_NODE) {
      IntNode tempNode = (IntNode) colFile.readNode(btreenode.branchs[btreenode.nbKey]); //non pass
      tempNode.parentPtr = btreenode.filePtr;
      colFile.writeNode(tempNode, header);
    }
    btreenode = branch;
    node.setKeyPtrAtPos(btreenode.getKeyAtPos(0), btreenode.getNodePtrAtPos(0), i);
    colFile.writeNode(node, header);

    for (int j = 0; j < btreenode.nbKey - 1; j++) {
      btreenode.setKeyPtrAtPos(btreenode.getKeyAtPos(j + 1), btreenode.getNodePtrAtPos(j + 1), j);
      btreenode.branchs[j] = btreenode.branchs[j + 1];
    }

    btreenode.branchs[btreenode.nbKey - 1] = btreenode.branchs[btreenode.nbKey];
    btreenode.nbKey--;
    colFile.writeNode(btreenode, header);
  }

  private void moveRight(IntNode node, int pos) throws IOException, ClassNotFoundException {
    IntNode rightBranch = (IntNode) colFile.readNode(node.branchs[pos + 1]);
    rightBranch.branchs[rightBranch.nbKey + 1] = rightBranch.branchs[rightBranch.nbKey];
    for (int j = rightBranch.nbKey - 1; j >= 0; j--) {
      rightBranch.setKeyPtrAtPos(rightBranch.getKeyAtPos(j), rightBranch.getNodePtrAtPos(j), j + 1);
      rightBranch.branchs[j + 1] = rightBranch.branchs[j];
    }

    rightBranch.nbKey++;
    rightBranch.setKeyPtrAtPos(node.getKeyAtPos(pos), node.getNodePtrAtPos(pos), 0);
    IntNode leftBranch = (IntNode) colFile.readNode(node.branchs[pos]);
    node.setKeyPtrAtPos(leftBranch.getKeyAtPos(leftBranch.nbKey - 1), leftBranch.getNodePtrAtPos(leftBranch.nbKey - 1), pos);
    rightBranch.branchs[0] = leftBranch.branchs[leftBranch.nbKey]; //change child parent ptr.
    colFile.writeNode(rightBranch, header);
    if (rightBranch.branchs[0] != Node.NO_NODE) {
      IntNode tempNode = (IntNode) colFile.readNode(rightBranch.branchs[0]);
      tempNode.parentPtr = rightBranch.filePtr;
      colFile.writeNode(tempNode, header);
    }

    leftBranch.nbKey--;
    colFile.writeNode(leftBranch, header);
    colFile.writeNode(node, header);
  }

  private void combine(IntNode node, int pos) throws IOException, ClassNotFoundException {
    IntNode rightBranch = (IntNode) colFile.readNode(node.branchs[pos + 1]);
    IntNode leftBranch = (IntNode) colFile.readNode(node.branchs[pos]);
    leftBranch.nbKey++;
    leftBranch.setKeyPtrAtPos(node.getKeyAtPos(pos), node.getNodePtrAtPos(pos), leftBranch.nbKey - 1);
    leftBranch.branchs[leftBranch.nbKey] = rightBranch.branchs[0];
    for (int j = 0; j < rightBranch.nbKey; j++) {
      leftBranch.nbKey++;
      leftBranch.setKeyPtrAtPos(rightBranch.getKeyAtPos(j), rightBranch.getNodePtrAtPos(j), leftBranch.nbKey - 1);
      leftBranch.branchs[leftBranch.nbKey] = rightBranch.branchs[j + 1];
    }

    colFile.writeNode(leftBranch, header);
    if (leftBranch.branchs[0] != Node.NO_NODE) {
      for (int k = 0; k <= leftBranch.nbKey; k++) {
        IntNode tempNode = (IntNode) colFile.readNode(leftBranch.branchs[k]); //non pass
        tempNode.parentPtr = leftBranch.filePtr;
        colFile.writeNode(tempNode, header);
      }
    }

    node.removeKeyAtPpos(pos);
    colFile.writeNode(node, header);
    colFile.deleteNode(rightBranch);
  }

  private void restore(IntNode node, int pos) throws IOException, ClassNotFoundException {
    if (pos == 0) {
      IntNode child = (IntNode) colFile.readNode(node.branchs[1]);
      if (child.nbKey > this.minKey) {
        this.moveLeft(node, 0);
      } else {
        this.combine(node, 0);
      }
    } else if (pos == node.nbKey) {
      IntNode child = (IntNode) colFile.readNode(node.branchs[pos - 1]);
      if (child.nbKey > this.minKey) {
        moveRight(node, pos - 1);
      } else {
        combine(node, pos - 1);
      }
    } else {
      IntNode childleft = (IntNode) colFile.readNode(node.branchs[pos - 1]);
      IntNode childrigth = (IntNode) colFile.readNode(node.branchs[pos + 1]);
      if (childleft.nbKey > this.minKey) {
        moveRight(node, pos - 1);
      } else if (childrigth.nbKey > this.minKey) {
        moveLeft(node, pos);
      } else {
        combine(node, pos);
      }
    }
  }

  public org.openmobileis.common.util.collection.Array getArrayKey(Object obj) throws FODBException{
  	org.openmobileis.common.util.collection.Array keys=new org.openmobileis.common.util.collection.Array();
  	int[] keyArray;
  	Integer key;
     if (accessObj instanceof Method) {
      try {
      	keyArray = (int[]) (((Method) accessObj).invoke(obj, new Object[0]));
      	for(int i=0;i<keyArray.length;i++){
      		key=new Integer(keyArray[i]);
      		keys.add(key);
      	}
      }catch (InvocationTargetException ex) { 
        throw new FODBException("Unable to execute index's method", ex);
      } catch (IllegalAccessException ex) { 
        throw new FODBException("Unable to execute index's method", ex);
      }
    }else{
      try{
      	keyArray =  (int[])((Field) accessObj).get(obj);
      	for(int i=0;i<keyArray.length;i++){
      		key=new Integer(keyArray[i]);
      		keys.add(key);
      	}
      } catch (IllegalAccessException ex) {
        throw new FODBException("Unable to access index's field", ex);
      }
    }

    return keys;
  }
  
  protected abstract void writeKeyPtr(Node node, int pos, long newptr) throws IOException, ClassNotFoundException;
  protected abstract void addSearchResult(IntNode node, int pos, LongArray array) throws IOException, ClassNotFoundException;

  protected abstract boolean removeKeyPtr(Node pg, int pos, long pointer) throws IOException, ClassNotFoundException;
}
