/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */

package org.openmobileis.database.fastobjectdb.db.index.node;

import java.io.*;

import org.openmobileis.database.fastobjectdb.db.index.SearchResult;
import org.openmobileis.database.fastobjectdb.db.query.soda.SodaIntIndexComparator;

/**
 * IntNode
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class IntNode extends Node {
  // serialization constant
  static final long serialVersionUID = 5521257935120563452L;

  //-------------------------
  // fields
  //-------------------------
  private int[] keys; // array of key values

  //-------------------------
  // constructors
  //-------------------------
  public IntNode() {
    super();
  }

  public IntNode(int order) {
    super(order);
    // allocate arrays
    keys = new int[order - 1];

  }

  public int getKeyAtPos(int pos) {
    return this.keys[pos];
  }

  public void setKeyPtrAtPos(int key, long ptr, int pos) {
    this.keys[pos] = key;
    super.ptr[pos] = ptr;
  }

  public SearchResult searchNode(int key, SearchResult search) {
    search.node = this;
    if (key < this.getKeyAtPos(0)) {
      search.pos = 0;
      search.found = false;
    } else {
      int i;
      for (i = nbKey - 1; i >= 0 && key < this.getKeyAtPos(i); i--);
      search.found = (key == this.getKeyAtPos(i));
      if (search.found)
        search.pos = i;
      else
        search.pos = i + 1;
    }
    return search;
  }

	public SearchResult searchNode(SodaIntIndexComparator comparator, SearchResult search) {
		search.node = this;
		if (comparator.compareTo(this.getKeyAtPos(0))<0) {
			search.pos = 0;
			search.found = false;
		} else {
			int i;
			for (i = nbKey - 1; i >= 0 && comparator.compareTo(this.getKeyAtPos(i))<0; i--);
			search.found = comparator.isSelected(this.getKeyAtPos(i));
			if (search.found)
				search.pos = i;
			else
				search.pos = i + 1;
		}
		return search;
	}

  public boolean pushInLeaf(int key, int pos, long newPtr) {
    this.branchs[this.nbKey + 1] = Node.NO_NODE;
    for (int j = this.nbKey - 1; j >= pos; j--) {
      this.setKeyPtrAtPos(this.getKeyAtPos(j), this.getNodePtrAtPos(j), j + 1);
    }
    this.setKeyPtrAtPos(key, newPtr, pos);
    this.nbKey++;
    return this.nbKey >= this.getMaxNbKey();
  }

  public boolean promote(int newkey, long keyptr, int pos, long nodePtr) {
    for (int j = this.nbKey; j >= pos + 1; j--) {
      this.setKeyPtrAtPos(keys[j - 1], this.getNodePtrAtPos(j - 1), j);
      this.branchs[j + 1] = this.branchs[j];
    }
    this.branchs[pos + 1] = nodePtr;
    this.setKeyPtrAtPos(newkey, keyptr, pos);
    this.nbKey++;
    return this.nbKey >= this.getMaxNbKey();
  }

  public void removeKeyAtPpos(int pos) {
    for (int j = pos + 1; j < this.nbKey; j++) {
      this.setKeyPtrAtPos(this.getKeyAtPos(j), this.getNodePtrAtPos(j), j - 1);
      this.branchs[j] = this.branchs[j + 1];
    }
    this.branchs[this.nbKey] = Node.NO_NODE;

    this.nbKey--;
  }

  //---------------------
  // serialization
  //---------------------
  public void writeExternal(ObjectOutput out) throws IOException {
    super.writeExternal(out);
    // create raw key data
    super.serializeInArray(out, keys);
  }

  public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
    super.readExternal(in);
    keys = super.unserializeInArray(in);
  }

}
