/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.index.node;

import java.io.*;

/**
 * Base class for all Btree Node.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public abstract class Node implements Externalizable {
  // serialization constant
  static final long serialVersionUID = 5521257935120563452L;
  public final static long NO_NODE = -1;

  //-------------------------
  // fields
  //-------------------------
  public long filePtr = Node.NO_NODE; // location in page file
  public long parentPtr = Node.NO_NODE; // parent in page file
  public long[] ptr; // record pointers
  public long[] branchs; // links to other pages
  public int nbKey;

  //-------------------------
  // constructors
  //-------------------------
  public Node() {
    nbKey = 0;
  }

  public Node(int order) {
    this();
    ptr = new long[order - 1];
    branchs = new long[order];

    // fill arrays with empty values
    for (int n = 0; n < branchs.length; ++n)
      branchs[n] = Node.NO_NODE;
  }

  public long getNodePtrAtPos(int pos) {
    return this.ptr[pos];
  }

  public int getMaxNbKey() {
    return ptr.length;
  }

  //---------------------
  // serialization
  //---------------------
  public void writeExternal(ObjectOutput out) throws IOException {
    // write order and key length
    out.writeLong(filePtr);
    out.writeLong(parentPtr);

    // write pointers and links
		this.serializeLongArray(out,ptr);
		this.serializeLongArray(out,branchs);
    out.writeInt(nbKey);
  }
  
	protected void serializeLongArray(ObjectOutput out, long[] array) throws IOException {
		int size=array.length;
		out.writeInt(size);
		for (int i=0; i<size; i++)	{
			out.writeLong(array[i]);
		}
  
	}
  
	protected long[] unserializeLongArray(ObjectInput in) throws IOException {
		int size=in.readInt();
		long[] ret = new long[size];
		for (int i=0; i<size; i++)	{
			ret[i] = in.readLong();
		}
  	return ret;
	}
  
	protected void serializeInArray(ObjectOutput out, int[] array) throws IOException {
		int size=array.length;
		out.writeInt(size);
		for (int i=0; i<size; i++)	{
			out.writeInt(array[i]);
		}
  
	}
  
	protected int[] unserializeInArray(ObjectInput in) throws IOException {
		int size=in.readInt();
		int[] ret = new int[size];
		for (int i=0; i<size; i++)	{
			ret[i] = in.readInt();
		}
		return ret;
	}

  public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
    filePtr = in.readLong();
    parentPtr = in.readLong();
    ptr = this.unserializeLongArray(in);
    branchs = this.unserializeLongArray(in);
    nbKey = in.readInt();
  }

}
