/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.query.soda;

import org.odbms.Constraint;
import org.openmobileis.common.util.collection.LongArray;
import org.openmobileis.database.fastobjectdb.db.exception.FODBException;

/**
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class FODBSodaConstraint extends BaseConstraint {


	private FODBSodaIndexPath indexpath;
	private boolean error;
	private int constraintId;
	boolean hasOperande = false;

	public FODBSodaConstraint(FODBSodaQuery query, int constraintId) {
		super();
		this.constraintId = constraintId;
	}
  
  void setIndex(FODBSodaIndexPath index)	{
		indexpath = index;
  }
  
  String getIndexMember()	{
  	return indexpath.getIndexMember();
  }
  
  private void setError(boolean err)	{
  	if (!err) this.error = false;
  }
  
  boolean setOperand(Object ope)	{
		if (indexpath != null)	{
			this.setHasOperande(true);
			return indexpath.setOperand(ope);
		}
		return false;
  }
  
  boolean addComparator(int ope)	{
		if (indexpath != null)	{
			return indexpath.addComparator(ope);
		}
		return false;
  }
  
	protected void execute(LongArray longArray) throws FODBException	{
    if (error) throw new FODBException("Bad Query construction.");
  	BaseConstraint nextConst = this.getNextConstraint();
  	if (nextConst != null)	{
  		nextConst.execute(longArray);
  	}
  	// execute current request
		long[] newPtrList = indexpath.execute();
		if (this.getNextConstraint() == null)	{
			longArray.add(newPtrList);
		} else 	if (this.nextContraintLink == BaseConstraint.CONSTRAINT_OR)	{
			longArray.addUnion(newPtrList);
		} else	{
			longArray.addIntersect(newPtrList);
		}
	}

	
	/* (non-Javadoc)
	 * @see org.odbms.Constraint#and(org.odbms.Constraint)
	 */
	public Constraint and(Constraint with) {
		FODBSodaConstraint newConst = (FODBSodaConstraint)with;
		if (newConst.getIndexMember().equals(this.getIndexMember()))	{ //AND are done in the same tree search if possible
			indexpath.mergeIndexComporator(newConst.indexpath.getIndexComparator());
			if (newConst.getNextConstraint() != null)	{
				this.setNextConstraint(newConst.getNextConstraint(), newConst.nextContraintLink);
			}
		} else	{
			this.setNextConstraint((BaseConstraint)with, BaseConstraint.CONSTRAINT_AND);
		}
		return this;
	}

	/* (non-Javadoc)
	 * @see org.odbms.Constraint#or(org.odbms.Constraint)
	 */
	public Constraint or(Constraint with) {
		this.setNextConstraint((BaseConstraint)with, BaseConstraint.CONSTRAINT_OR);
		return this;
	}
	/* (non-Javadoc)
	 * @see org.odbms.Constraint#equal()
	 */
	public Constraint equal() {
		this.setError(this.addComparator(BaseConstraint.EQUALS));
		return this;
	}

	/* (non-Javadoc)
	 * @see org.odbms.Constraint#greater()
	 */
	public Constraint greater() {
		this.setError(this.addComparator(BaseConstraint.GREATER));
		return this;
	}

	/* (non-Javadoc)
	 * @see org.odbms.Constraint#smaller()
	 */
	public Constraint smaller() {
		this.setError(this.addComparator(BaseConstraint.SMALLER));
		return this;
	}

	/* (non-Javadoc)
	 * @see org.odbms.Constraint#identity()
	 */
	public Constraint identity() {
		this.setError(this.addComparator(BaseConstraint.IDENTITY));
		return this;
	}

	/* (non-Javadoc)
	 * @see org.odbms.Constraint#like()
	 */
	public Constraint like() {
		this.setError(this.addComparator(BaseConstraint.LIKE));
		return this;
	}

  /**
   * return only object that are distinct for this constraint.
   * If two object has the same value, only the first on is return.
   * @return linked constraint.
   */
  public Constraint distinct ()	{
	this.setError(this.addComparator(BaseConstraint.DISTINCT));
	return this;
}

	/* (non-Javadoc)
	 * @see org.odbms.Constraint#contains()
	 */
	public Constraint contains() {
		this.setError(this.addComparator(BaseConstraint.CONTAINS));
		return this;
	}

	/* (non-Javadoc)
	 * @see org.odbms.Constraint#not()
	 */
	public Constraint not() {
		this.setError(this.addComparator(BaseConstraint.NOT));
		return this;
	}

	/* (non-Javadoc)
	 * @see org.odbms.Constraint#getObject()
	 */
	public Object getObject() {
		// TODO Auto-generated method stub
		return null;
	}
	
	boolean hasIndex()	{
		return (indexpath!=null);
	}
  /**
   * @return
   */
  public int getConstraintId() {
    return constraintId;
  }
  
  public boolean equals(Object con)	{
  	if (con instanceof FODBSodaConstraint)	{
  		if (((FODBSodaConstraint)con).constraintId == this.constraintId)	{
  			return true;
  		}
  	}
  	return false;
  }

  /**
   * @return
   */
  boolean isHasOperande() {
    return hasOperande;
  }

  /**
   * @param b
   */
  void setHasOperande(boolean b) {
    hasOperande = b;
  }

}
