/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.query.soda;

import org.openmobileis.common.util.collection.Array;

/**
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class SodaLongIndexComparator extends SodaIndexComparator {
  private Array comparatorList;

  class LongOperande {
    int comparator=0;
    long operande;
    int traversal;

    LongOperande() {
    }
  }
  /**
   * 
   */
  public SodaLongIndexComparator() {
    super();
    comparatorList = new Array();
  }

  public boolean setOperand(Object ope) {
  	if (ope instanceof Long)	{
	    LongOperande operand = new LongOperande();
	    operand.operande = ((Long)ope).longValue();
	    comparatorList.add(operand);
	    return true;
  	}
  	return false;
  }

	public void mergeIndexComporator(SodaIndexComparator index)	{
		SodaLongIndexComparator newindex = (SodaLongIndexComparator)index;
		comparatorList.addArray(newindex.comparatorList);
	}

  public boolean addComparator(int comparator) {
    LongOperande comp = (LongOperande) comparatorList.getLastAdded();
    if (comp != null) {
    	if (comp.comparator == BaseConstraint.COMPARATOR_NONE)	{
      	comp.comparator = comparator;
      	comp.traversal = this.getTraversalWayWithComparator(comp.comparator);
    	} else	{
				//manage not or equals.
				if (comp.comparator == BaseConstraint.EQUALS)	{
					if (comparator == BaseConstraint.SMALLER)	{
						comp.comparator = BaseConstraint.EQUALSMALLER;
					} else	if (comparator == BaseConstraint.GREATER)	{
						comp.comparator = BaseConstraint.EQUALGREATER;
					} else	if (comparator == BaseConstraint.NOT)	{
						comp.comparator = BaseConstraint.NOT;
					} else	if (comparator == BaseConstraint.EQUALS)	{
						comp.comparator = BaseConstraint.EQUALS;
					}
				} else if (comp.comparator == BaseConstraint.NOT)	{
					if (comparator == BaseConstraint.SMALLER)	{
						comp.comparator = BaseConstraint.EQUALGREATER;
					} else	if (comparator == BaseConstraint.GREATER)	{
						comp.comparator = BaseConstraint.EQUALSMALLER;
					} else	if (comparator == BaseConstraint.EQUALS)	{
						comp.comparator = BaseConstraint.NOT;
					} else	if (comparator == BaseConstraint.NOT)	{
						comp.comparator = BaseConstraint.NOT;
					}
				} else if (comp.comparator == BaseConstraint.SMALLER)	{
					if (comparator == BaseConstraint.NOT)	{
						comp.comparator = BaseConstraint.EQUALGREATER;
					} else	if (comparator == BaseConstraint.EQUALS)	{
						comp.comparator = BaseConstraint.EQUALSMALLER;
					} else	if (comparator == BaseConstraint.GREATER)	{
						comp.comparator = BaseConstraint.EQUALS;
					} else	if (comparator == BaseConstraint.SMALLER)	{
						comp.comparator = BaseConstraint.SMALLER;
					}
				} else if (comp.comparator == BaseConstraint.GREATER)	{
					if (comparator == BaseConstraint.NOT)	{
						comp.comparator = BaseConstraint.EQUALSMALLER;
					} else	if (comparator == BaseConstraint.EQUALS)	{
						comp.comparator = BaseConstraint.EQUALGREATER;
					} else	if (comparator == BaseConstraint.SMALLER)	{
						comp.comparator = BaseConstraint.EQUALS;
					} else	if (comparator == BaseConstraint.GREATER)	{
						comp.comparator = BaseConstraint.GREATER;
					}
				} else	{
					LongOperande operand = new LongOperande();
					operand.operande = comp.operande;
					operand.comparator = comparator;
					operand.traversal = this.getTraversalWayWithComparator(operand.comparator);
					comparatorList.add(operand);
				}
    	}
      return true;
    }
    return false;
  }

	public int getSearchAlgo()	{
		if (comparatorList.size() == 0) {
			return SodaIndexComparator.FULL_TRAVERSAL;
		} else {
			return ((LongOperande) comparatorList.get(0)).traversal;
		}
	}

  /**
   * return true if the key must be selected for the specified
   * search int
   */
  public boolean isSelected(long searchInt) {
    boolean ret = true;
    int size = comparatorList.size();
    for (int i = 0; i < size && ret; i++) {
      ret = this.selectFisrtOperande((LongOperande) comparatorList.get(i), searchInt);
    }
    return ret;
  }

  /**
   * return >0 if the key is after the specified
   * search int, <0 if before and 0 if equals.
   */
  public int compareTo(long searchInt) {
    if (comparatorList.size() == 0) {
      return -1; //force all tree traversal.
    } else {
      return this.compareFisrtOperande((LongOperande) comparatorList.get(0), searchInt);
    }
  }

	private boolean selectFisrtOperande(LongOperande op, long key) {
		long test = key - op.operande;
		switch (op.comparator) {
			case BaseConstraint.EQUALS :
				return (test==0) ;
			case BaseConstraint.SMALLER :
				return (test <0) ;
			case BaseConstraint.EQUALSMALLER :
				return (test <=0) ;
			case BaseConstraint.GREATER :
				return (test >0)  ;
			case BaseConstraint.EQUALGREATER :
				return (test >=0)  ;
			case BaseConstraint.NOT :
				return (test != 0)  ;
		}
		return true;
	}

  private int compareFisrtOperande(LongOperande op, long key) {
    switch (op.comparator) {
			case BaseConstraint.EQUALS :
			case BaseConstraint.SMALLER :
			case BaseConstraint.EQUALSMALLER :
			case BaseConstraint.NOT :
				return (int)(key - op.operande) ;
			case BaseConstraint.GREATER :
			case BaseConstraint.EQUALGREATER :
				return (int)(op.operande - key)  ;
    }
    return (int)(key - op.operande);
  }

}
