/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.query.soda;

import org.openmobileis.common.util.StringFormater;
import org.openmobileis.common.util.collection.Array;

/**
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class SodaStringIndexComparator extends SodaIndexComparator {
	private Array comparatorList;
	private boolean isCaseSensitive = false;
	private boolean isDistinct;
	
	class StringOperande	{
		int comparator;
		String operande;		
		int traversal;

		StringOperande()	{
		}
	}

	/**
	 * 
	 */
	public SodaStringIndexComparator(boolean caseSensitive) {
		super();
		this.comparatorList = new Array();
		this.isCaseSensitive = caseSensitive;
	}

	public boolean setOperand(Object ope)	{
		if (ope instanceof String)	{
			StringOperande operand = new StringOperande();
			operand.operande = (String)ope;
			if (!this.isCaseSensitive)	{
				operand.operande = StringFormater.removeAccent(operand.operande).toUpperCase();
			}
			comparatorList.add(operand);
			return true;
		}
		return false;
	}

	public void mergeIndexComporator(SodaIndexComparator index)	{
		SodaStringIndexComparator newindex = (SodaStringIndexComparator)index;
		comparatorList.add(newindex.comparatorList);
	}
  
	public boolean addComparator(int comparator)	{
		if (comparator == BaseConstraint.DISTINCT)	{
			this.setDistinct(true);
		}
		StringOperande comp = (StringOperande) comparatorList.getLastAdded();
		if (comp != null) {
			if (comp.comparator == BaseConstraint.COMPARATOR_NONE)	{
				comp.comparator = comparator;
				comp.traversal = this.getTraversalWayWithComparator(comp.comparator);
			} else	{
				//manage not or equals.
				if (comp.comparator == BaseConstraint.EQUALS)	{
					if (comparator == BaseConstraint.SMALLER)	{
						comp.comparator = BaseConstraint.EQUALSMALLER;
					} else	if (comparator == BaseConstraint.GREATER)	{
						comp.comparator = BaseConstraint.EQUALGREATER;
					} else	if (comparator == BaseConstraint.NOT)	{
						comp.comparator = BaseConstraint.NOT;
					} else	if (comparator == BaseConstraint.EQUALS)	{
						comp.comparator = BaseConstraint.EQUALS;
					}
				} else if (comp.comparator == BaseConstraint.NOT)	{
					if (comparator == BaseConstraint.SMALLER)	{
						comp.comparator = BaseConstraint.EQUALGREATER;
					} else	if (comparator == BaseConstraint.GREATER)	{
						comp.comparator = BaseConstraint.EQUALSMALLER;
					} else	if (comparator == BaseConstraint.EQUALS)	{
						comp.comparator = BaseConstraint.NOT;
					} else	if (comparator == BaseConstraint.NOT)	{
						comp.comparator = BaseConstraint.NOT;
					}
				} else if (comp.comparator == BaseConstraint.SMALLER)	{
					if (comparator == BaseConstraint.NOT)	{
						comp.comparator = BaseConstraint.EQUALGREATER;
					} else	if (comparator == BaseConstraint.EQUALS)	{
						comp.comparator = BaseConstraint.EQUALSMALLER;
					} else	if (comparator == BaseConstraint.GREATER)	{
						comp.comparator = BaseConstraint.EQUALS;
					} else	if (comparator == BaseConstraint.SMALLER)	{
						comp.comparator = BaseConstraint.SMALLER;
					}
				} else if (comp.comparator == BaseConstraint.GREATER)	{
					if (comparator == BaseConstraint.NOT)	{
						comp.comparator = BaseConstraint.EQUALSMALLER;
					} else	if (comparator == BaseConstraint.EQUALS)	{
						comp.comparator = BaseConstraint.EQUALGREATER;
					} else	if (comparator == BaseConstraint.SMALLER)	{
						comp.comparator = BaseConstraint.EQUALS;
					} else	if (comparator == BaseConstraint.GREATER)	{
						comp.comparator = BaseConstraint.GREATER;
					}
				} else	{
					StringOperande operand = new StringOperande();
					operand.operande = comp.operande;
					operand.comparator = comparator;
					operand.traversal = this.getTraversalWayWithComparator(operand.comparator);
					comparatorList.add(operand);
				}
			}
			return true;
		}
		//comparator with no operand
		  return false;
	}

	public int getSearchAlgo()	{
		if (comparatorList.size() == 0) {
			return SodaIndexComparator.FULL_TRAVERSAL;
		} else {
			return ((StringOperande) comparatorList.get(0)).traversal;
		}
	}
  
	/**
	 * return true if the key must be selected for the specified
	 * search string
	 */
	public boolean isSelected(String searchString)	{
		boolean ret = true;
		int size = comparatorList.size();
		for (int i = 0; i < size && ret; i++) {
			ret = this.selectFisrtOperande((StringOperande) comparatorList.get(i), searchString);
		}
		return ret;
	}
	
	/**
	 * return >0 if the key is after the specified
	 * search string, <0 if before and 0 if equals.
	 */
	public int compareTo(String searchString)	{
		if (comparatorList.size() == 0) {
			return -1; //force all tree traversal.
		} else {
			return this.compareFisrtOperande((StringOperande) comparatorList.get(0), searchString);
		}
	}

	private boolean selectFisrtOperande(StringOperande op, String key) {
		switch (op.comparator) {
			case BaseConstraint.EQUALS :
				int test = key.compareTo(op.operande);
				return (test==0) ;
			case BaseConstraint.NOT :
				test = key.compareTo(op.operande);
				return (test!=0) ;
			case BaseConstraint.SMALLER :
				test = key.compareTo(op.operande);
				return (test <0) ;
			case BaseConstraint.EQUALSMALLER :
				test = key.compareTo(op.operande);
				return (test <=0) ;
			case BaseConstraint.GREATER :
				test = key.compareTo(op.operande);
				return (test >0)  ;
			case BaseConstraint.EQUALGREATER :
				test = key.compareTo(op.operande);
				return (test >=0)  ;
			case BaseConstraint.LIKE :
				return key.startsWith(op.operande)  ;
			case BaseConstraint.CONTAINS :
				return (key.indexOf(op.operande)!=-1)  ;
			case BaseConstraint.DISTINCT :
				return true  ;
		}
		return true;
	}

	private int compareFisrtOperande(StringOperande op, String key) {
		switch (op.comparator) {
			case BaseConstraint.EQUALS :
			case BaseConstraint.SMALLER :
			case BaseConstraint.EQUALSMALLER :
			case BaseConstraint.LIKE :
			case BaseConstraint.NOT :
				return key.compareTo(op.operande) ;
			case BaseConstraint.EQUALGREATER :
			case BaseConstraint.GREATER :
				return -key.compareTo(op.operande)  ;
			case BaseConstraint.CONTAINS :
			case BaseConstraint.DISTINCT :
				return -1  ;
		}
		return -1;
	}

  public boolean isDistinct() {
    return isDistinct;
  }
  public void setDistinct(boolean isDistinct) {
    this.isDistinct = isDistinct;
  }
}
