/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.store;

import java.io.IOException;

import org.openmobileis.database.fastobjectdb.db.index.FODBIndexHeader;
import org.openmobileis.database.fastobjectdb.db.index.node.Node;


/**
 *
 * FODBCollectionFileLink is a Class that has been created in order
 * to facilitate access of indexs to Collection file.
 * It contains all needed methods : readObject, writeObject,
 * deleteObject, etc...
 * For each of these functions, it opens and closes the Collection
 * File. That makes sure we won't have any error, even if we forgot
 * to open the file before we try to access it.
 */
public class FODBCollectionIndexFile {
	FODBCollectionFile collectionFile;
	
	public FODBCollectionIndexFile(FODBCollectionFile cf) {
		collectionFile = cf;
	}
	
	public Node readNode(long pos)  throws IOException, ClassNotFoundException  {
		return (Node)collectionFile.readNode(pos);
	}
	
	public long writeNode(Node node, FODBIndexHeader header) throws IOException, ClassNotFoundException  {
		// if new page, find a place to write it
		if (node.filePtr == Node.NO_NODE) {
			// write the new page
			node.filePtr = collectionFile.writeNode(node);
		} else { // write in old position  
			collectionFile.rewriteNode(node, node.filePtr);
		}

		// if root, change root pointer
		if (node.parentPtr == Node.NO_NODE) {
			header.rootPtr = node.filePtr;
			// update header
			collectionFile.rewriteNode(header, header.pos);
		}
		return node.filePtr;
	}
	
	public long writeHeader(FODBIndexHeader header) throws IOException, ClassNotFoundException {
		return collectionFile.writeNode(header);
	}
	
	public void rewriteHeader(FODBIndexHeader obj, long pos) throws IOException, ClassNotFoundException  {
		collectionFile.rewriteNode(obj, pos);
	}
	
	public void rewriteNode(Node obj, long pos) throws IOException, ClassNotFoundException  {
		collectionFile.rewriteNode(obj, pos);
	}
	
	public void deleteNode(Node node) throws IOException, ClassNotFoundException  {
		collectionFile.deleteObject(node.filePtr);
	}
	
	public void addIndexPointerToTable(long pointer) throws IOException, ClassNotFoundException  {
		collectionFile.addIndex(pointer);
	}
	
	public long[] readLongPtr(long pos)  throws IOException, ClassNotFoundException  {
		return (long[])collectionFile.readNode(pos);
	}
	
	public long writeLongPtr(long[] ptr) throws IOException, ClassNotFoundException {
		return collectionFile.writeNode(ptr);
	}
	
	public void rewriteLongPtr(long[] obj, long pos) throws IOException, ClassNotFoundException  {
		collectionFile.rewriteNode(obj, pos);
	}
	
	public void deleteLongPtr(long ptr) throws IOException, ClassNotFoundException  {
		collectionFile.deleteObject(ptr);
	}
}
