/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.store;

import java.io.*;

import org.openmobileis.common.util.collection.Array;

/**
 *
 * FODBMainFile is a Class that defines simple functions to
 * access our data file.
 * It's also responsible of the way the database file is structured.
 * 
 * It defines static variables ACTION_CREATE and ACTION_OPEN
 * which are private. It uses them in its constructor. Other functions
 * don't have to know them.
 */
public class FODBMainFile {
  private final static int ACTION_CREATE = 0;
  private final static int ACTION_OPEN = 1;

  private Array collectionTable = null;
  private String mainFilePath;

  private FODBMainFile(String filePath, int action) throws IOException, ClassNotFoundException {
    if (action == ACTION_CREATE) {
      File f1 = new File(filePath);
      f1.delete();
    }

    mainFilePath = filePath;

  // Let's open our file (or create it if it doesn't exist)
    if (action == ACTION_CREATE) {

      // create and write collectionTable
      collectionTable = new Array();
      rewriteCollectionTable(collectionTable);
    } else if (action == ACTION_OPEN) {
      collectionTable = readCollectionTable();
    }
  }

  public static FODBMainFile createDbFile(String filePath) throws IOException, ClassNotFoundException {
    return new FODBMainFile(filePath, ACTION_CREATE);
  }

  public static FODBMainFile openDbFile(String filePath) throws IOException, ClassNotFoundException {
    File dbFile = new File(filePath);
    if (!dbFile.exists()) {
      return null;
    }
    return new FODBMainFile(filePath, ACTION_OPEN);
  }

  public String addCollection(String collectionName) throws IOException {
    collectionTable.add(collectionName);
    rewriteCollectionTable(collectionTable);
    return collectionName;
  }

  private void rewriteCollectionTable(Array obj) throws IOException {			
		ObjectOutputStream out = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(mainFilePath)));
		try	{
			out.writeObject(obj);
		} finally	{
			out.flush();
			out.close();
		}
  }

  private Array readCollectionTable() throws IOException, ClassNotFoundException {
		ObjectInputStream in = new ObjectInputStream(new BufferedInputStream(new FileInputStream(mainFilePath)));
		try	{
			return (Array)in.readObject();
		} finally	{
			in.close();
		}
  }

  public String[] getCollectionsList() {
		String[] collectionsNameDuplic = new String[collectionTable.size()];
		collectionsNameDuplic = (String[])collectionTable.toArray(collectionsNameDuplic);
		return collectionsNameDuplic;
  }
}
