/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.database.fastobjectdb.db.test;

import org.odbms.ObjectSet;
import org.odbms.Query;
import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;
import org.openmobileis.database.fastobjectdb.db.query.soda.FODBSodaObjectSet;
import org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget;
import org.openmobileis.synchro.algo.replication.AlwaysUpdateServerSynchroConflicResolver;
import org.openmobileis.synchro.algo.replication.DefaultSynchroAtomicObject;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;
import org.openmobileis.synchro.algo.replication.SynchroConflicResolver;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.security.auth.Credential;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class TestFODBSyncTarget implements FODBSyncTarget {
  private AlwaysUpdateServerSynchroConflicResolver conflicResolver;

  /**
   * 
   */
  public TestFODBSyncTarget() {
    super();
    conflicResolver = new AlwaysUpdateServerSynchroConflicResolver();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#getCollectionName()
   */
  public String getCollectionName() {
     return "CLIENT";
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#getCollectionObjectWithId(java.lang.String)
   */
  public Object getCollectionObjectWithId(String id) throws OpenMSPException {
    try {
    Query q = FastObjectDBManager.getCurrentFODB().query();
    q.constrain("SERVER");
    Query q2 = q.descend("getKey()");
    q2.constrain(id).equal();
    ObjectSet set = q.execute();
    if (set.hasNext())  {
      return (TestData) set.next();
    }
    return null;
    } catch (Throwable ex)  {
      throw new OpenMSPException(ex);
    }
  }
  
  public Array getAllCollectionObject()  throws OpenMSPException  {
    try {
      Query q = FastObjectDBManager.getCurrentFODB().query();
      q.constrain("SERVER");
      q.descend("getKey()");
      ObjectSet set = q.execute();
      return ((FODBSodaObjectSet)set).getArrayFromSet();
      } catch (Throwable ex)  {
        throw new OpenMSPException(ex);
      }
  }


  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#updateCollectionObject(java.lang.Object)
   */
  public void updateCollectionObject(Object obj) throws OpenMSPException {
    try {
      TestData test = (TestData)obj;
      Object dbobj = this.getCollectionObjectWithId(test.getKey());
      if (dbobj == null)  {
        FastObjectDBManager.getCurrentFODB().add("SERVER", test);
      } else  {
        FastObjectDBManager.getCurrentFODB().replace("SERVER", test);
      }
    } catch (Throwable ex)  {
      throw new OpenMSPException(ex);
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#deleteCollectionObject(java.lang.String)
   */
  public void deleteCollectionObject(String id) throws OpenMSPException {
    try {
      FastObjectDBManager.getCurrentFODB().deleteWithId("SERVER", id);
    } catch (Throwable ex)  {
      throw new OpenMSPException(ex);
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#getConflicResolver()
   */
  public SynchroConflicResolver getConflicResolver() {
    return conflicResolver;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#getAllModifiedAtomicObjectForServiceSince(long)
   */
  public SynchroAtomicObject[] getAllModifiedAtomicObjectSince(long syncnumber) throws OpenMSPException {
//    Object obj = this.getCollectionObjectWithId("FIRSTS");
    SynchroAtomicObject[] ret = new SynchroAtomicObject[0];
    if (syncnumber == 0)  {  // add
      ret = new SynchroAtomicObject[1];
      ret[0] = new DefaultSynchroAtomicObject("FIRSTS", "");
      ret[0].setModificationType(SynchroAtomicObject.ADD);
    }else if (syncnumber == 1)  {  //replace
      TestData data = new TestData("FIRSTS");
      data.setToModify("1S");
      try {
        FastObjectDBManager.getCurrentFODB().replace("SERVER", data);
      } catch (Throwable ex)  {
        throw new OpenMSPException(ex);
      }
      ret = new SynchroAtomicObject[1];
      ret[0] = new DefaultSynchroAtomicObject("FIRSTS", "");
      ret[0].setModificationType(SynchroAtomicObject.REPLACE);
    }else if (syncnumber == 2)  { //delete
      try {
        FastObjectDBManager.getCurrentFODB().deleteWithId("SERVER", "FIRSTS");
      } catch (Throwable ex)  {
        throw new OpenMSPException(ex);
      }
      ret = new SynchroAtomicObject[1];
      ret[0] = new DefaultSynchroAtomicObject("FIRSTS", "");
      ret[0].setModificationType(SynchroAtomicObject.DELETE);
    }else if (syncnumber == 3)  { //complete
      TestData data = new TestData("THIRD");
      data.setToModify("0S");
      try {
        FastObjectDBManager.getCurrentFODB().add("SERVER", data);
      } catch (Throwable ex)  {
        throw new OpenMSPException(ex);
      }
      data = new TestData("FOURTH");
      data.setToModify("0S");
      try {
        FastObjectDBManager.getCurrentFODB().add("SERVER", data);
      } catch (Throwable ex)  {
        throw new OpenMSPException(ex);
      }
      ret = new SynchroAtomicObject[2];
      ret[0] = new DefaultSynchroAtomicObject("THIRD", "");
      ret[0].setModificationType(SynchroAtomicObject.ADD);
      ret[1] = new DefaultSynchroAtomicObject("FOURTH", "");
      ret[1].setModificationType(SynchroAtomicObject.ADD);
    }
    syncnumber++;
    return ret;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#getUpdateMaxNbRow()
   */
  public int getUpdateMaxNbRow() {
    return 1;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#updateSynchroDB(org.openmobileis.database.fastobjectdb.FastObjectDB)
   */
  public void updateSynchroDB(FastObjectDB db) throws OpenMSPException {
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#connect(org.openmobileis.synchro.security.auth.Credential)
   */
  public void connect(Credential cred) throws UserNotFoundException, ServiceException {
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#disconnect()
   */
  public void disconnect() {
  }

}
