/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.HashMap;

import org.odbms.Constraint;
import org.odbms.ObjectSet;
import org.odbms.Query;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.fastobjectdb.FODBIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FODBIntIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FODBLongIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FODBStringIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.db.BackwardUniqueIndexIterator;
import org.openmobileis.database.fastobjectdb.db.FODBCollection;
import org.openmobileis.database.fastobjectdb.db.ForwardUniqueIndexIterator;
import org.openmobileis.database.fastobjectdb.db.crypto.AcmeBlowfishCypher;
import org.openmobileis.database.fastobjectdb.db.exception.FODBException;

/**
 * Test application use to test FODB. Present all functionality of FODB. Base code to see how to use FODB
 * 
 * @author Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public final class TestFastObjectDB {

  private static int NB_SEARCH_CREATED_DATA = 30;

  private HashMap keyMap = new HashMap(500);

  private HashMap objectMap = new HashMap(500);

  private FastObjectDB db = null;
  
  /*class TestRequestThread() {
    TestRequestThread(){
      }
  }*/

  /**
   *  
   */
  public TestFastObjectDB() {
    super();
  }

  protected void testTree(Array inputDataList, FastObjectDB db) throws FODBException {
    //verify index
    for (int i = 0; i < inputDataList.size(); i++) {
      String key = (String) inputDataList.get(i);
      try {
        //    SELECT TESTDATA.KEY FROM TESTDATA where TESTDATA.KEY = 'key'
        Query q = db.query();
        q.constrain(TestData.class);
        Query q2 = q.descend("getKey()");
        q2.constrain(key).equal();
        ObjectSet set = q.execute();
        if (set.size() == 0) {
          throw new FODBException("ERROR Key not found :" + key);
        } else if (!((TestData) set.next()).getKey().equals(key)) {
          throw new FODBException("ERROR Key Not egal to value :" + key);
        }
      } catch (FODBException ex) {
        LogManager.traceError(0, "Bad KEY READ :" + key);
        throw ex;
      }
    }
  }

  protected void testRomvedTree(Array inputDataList, FastObjectDB db) throws FODBException {
    //verify index
    for (int i = 0; i < inputDataList.size(); i++) {
      String key = (String) inputDataList.get(i);
      try {
        //				SELECT TESTDATA.KEY FROM TESTDATA where TESTDATA.KEY = 'key'
        Query q = db.query();
        q.constrain(TestData.class);
        Query q2 = q.descend("getKey()");
        q2.constrain(key).equal();
        ObjectSet set = q.execute();
        if (set.size() != 0) {
          throw new FODBException("ERROR Key found :" + key);
        }
      } catch (FODBException ex) {
        LogManager.traceError(0, "Bad KEY READ :" + key);
        throw ex;
      }
    }
  }

  private void testInitIndexIterator(String collectionName, String memberName, Array objList) throws FODBException {
    ForwardUniqueIndexIterator iterf = db.getCollection(collectionName).getForwardIndexIterator(memberName);
    for (int i = 0; i < NB_SEARCH_CREATED_DATA; i++) {
      iterf.initCurrentNode(new Integer(i));
      for (int j = i + 1; j < NB_SEARCH_CREATED_DATA; j++) { //validate iteration
        TestSearchData data = (TestSearchData) iterf.next();
        if (data.getKey() != j) {
          throw new FODBException("testInitIndexIterator error bad iteration");
        }
      }
    }
    BackwardUniqueIndexIterator iterb = db.getCollection(collectionName).getBackwardIndexIterator(memberName);
    for (int i = 0; i < NB_SEARCH_CREATED_DATA; i++) {
      iterb.initCurrentNode(new Integer(i));
      for (int j = i - 1; j >= 0; j--) { //validate iteration
        TestSearchData data = (TestSearchData) iterb.prev();
        if (data.getKey() != j) {
          throw new FODBException("testInitIndexIterator error bad iteration");
        }
      }
    }

  }

  private void testIndexIterator(String collectionName, String memberName, Array objList) throws FODBException {
    ForwardUniqueIndexIterator iterf = db.getCollection(collectionName).getForwardIndexIterator(memberName);
    Array indexobj = new Array();
    while (iterf.hasNext()) {
      indexobj.add(iterf.next());
    }
    //valide arrays
    if (indexobj.size() != objList.size()) {
      throw new FODBException("testIndexIterator bad size");
    }
    for (int i = 0; i < indexobj.size(); i++) {
      Object obj = objList.get(i);
      if (!indexobj.contains(obj)) {
        throw new FODBException("testIndexIterator obj not found :" + obj);
      }
    }

    //test prev
    BackwardUniqueIndexIterator iterb = db.getCollection(collectionName).getBackwardIndexIterator(memberName);
    indexobj = new Array();
    while (iterb.hasPrev()) {
      indexobj.add(iterb.prev());
    }
    //valide arrays
    if (indexobj.size() != objList.size()) {
      throw new FODBException("testIndexIterator bad size");
    }
    for (int i = 0; i < indexobj.size(); i++) {
      Object obj = objList.get(i);
      if (!indexobj.contains(obj)) {
        throw new FODBException("testIndexIterator obj not found :" + obj);
      }
    }
  }

  private void addKeyToMap(String key) {
    String begin = null;
    if (key.length() >= TestData.NB_BEGIN_KEY_LETTER) {
      begin = key.substring(0, TestData.NB_BEGIN_KEY_LETTER);
    } else {
      begin = key;
    }
    Integer inter = (Integer) keyMap.get(begin);
    if (inter == null) {
      inter = new Integer(0);
    } else {
      keyMap.remove(begin);
    }
    int old = inter.intValue();
    old++;
    keyMap.put(begin, new Integer(old));
  }

  private void removeKeyToMap(String key) {
    String begin = null;
    if (key.length() >= TestData.NB_BEGIN_KEY_LETTER) {
      begin = key.substring(0, TestData.NB_BEGIN_KEY_LETTER);
    } else {
      begin = key;
    }
    Integer inter = (Integer) keyMap.get(begin);
    if (inter == null) {
      return;
    } else {
      keyMap.remove(begin);
    }
    int old = inter.intValue();
    old--;
    if (old == 0) {
      keyMap.remove(begin);
    } else {
      keyMap.put(begin, new Integer(old));
    }
  }

  private int getNbKeyForBegin(String begin) {
    Integer inter = (Integer) keyMap.get(begin);
    if (inter == null)
      return 0;
    else
      return inter.intValue();
  }

  public void testFindLike(Array inputDataList, FastObjectDB db) throws FODBException {
    for (int i = 0; i < inputDataList.size(); i++) {
      String key = (String) inputDataList.get(i);
      String begin = null;
      if (key.length() >= TestData.NB_BEGIN_KEY_LETTER) {
        begin = key.substring(0, TestData.NB_BEGIN_KEY_LETTER);
      } else {
        begin = key;
      }
      //  SELECT TESTDATA.KEY FROM TESTDATA where TESTDATA.KEY like '%begin%'
      Query q = db.query();
      q.constrain(TestData.class);
      Query q2 = q.descend("getKey()");
      q2.constrain(begin).like();
      ObjectSet set = q.execute();
      int listnb = this.getNbKeyForBegin(begin);
      if (listnb != set.size()) {
        throw new FODBException("ERROR BAD NUMBER of KEY LIKE FOUND :" + key);
      } else {
        for (int j = 0; j < set.size(); j++) {
          TestData objkey = (TestData) set.next();
          if (!objkey.getKey().startsWith(begin)) {
            throw new FODBException("ERROR BAD FIND LIKE VALUE FOR KEY :" + key);
          }

        }
      }
    }

  }

  public void testMultipleIndex(Array inputDataList, FastObjectDB db) throws FODBException {
    for (int i = 0; i < inputDataList.size(); i++) {
      String key = (String) inputDataList.get(i);
      String begin = null;
      if (key.length() >= TestData.NB_BEGIN_KEY_LETTER) {
        begin = key.substring(0, TestData.NB_BEGIN_KEY_LETTER);
      } else {
        begin = key;
      }
      //		SELECT TESTDATA.KEY FROM TESTDATA where TESTDATA.DEBKEY = 'begin'
      Query q = db.query();
      q.constrain(TestData.class);
      Query q2 = q.descend("getDebKey()");
      q2.constrain(begin).equal();
      ObjectSet set = q.execute();
      int listnb = this.getNbKeyForBegin(begin);
      if (listnb != set.size()) {
        throw new FODBException("ERROR BAD NUMBER of KEY LIKE FOUND :" + key);
      } else {
        while (set.hasNext()) {
          TestData objkey = (TestData) set.next();
          if (!objkey.getKey().startsWith(begin)) {
            throw new FODBException("ERROR BAD FIND LIKE VALUE FOR KEY :" + key);
          }
        }
      }
    }

  }

  public void openDB() {
    try {
      String tempdir = System.getProperty("user.dir") + File.separator;
      db = FastObjectDB.open(tempdir, "testdb");
      if (!db.isCollectionExist("TESTDATA")) {
        db.createCollection("TESTDATA", TestData.class);
        FODBStringIndexDescriptor KeyDescriptor = new FODBStringIndexDescriptor("KEY", FODBIndexDescriptor.UNIQUE, "getKey()", 12, 15);
        db.addIndex("TESTDATA", KeyDescriptor);
        FODBStringIndexDescriptor debKeyDescriptor = new FODBStringIndexDescriptor("DEBKEY", FODBIndexDescriptor.MULTIPLE, "getDebKey()", 10,
            TestData.NB_BEGIN_KEY_LETTER, 5);
        db.addIndex("TESTDATA", debKeyDescriptor);
      }
      FODBCollection col = db.getCollection("TESTDATA");
      col.setCollectionCypher(new AcmeBlowfishCypher()); //test with encryption
      if (!db.isCollectionExist("TESTDATAWITHARRAY")) {
        db.createCollection("TESTDATAWITHARRAY", TestDataWithArray.class);
        FODBStringIndexDescriptor intIdDescriptor = new FODBStringIndexDescriptor("ID1", FODBIndexDescriptor.UNIQUE, "getId()", 10, 50);
        db.addIndex("TESTDATAWITHARRAY", intIdDescriptor);
        FODBLongIndexDescriptor LongIdDescriptor = new FODBLongIndexDescriptor("ID2", FODBIndexDescriptor.UNIQUE, "getIdLong()", 10);
        db.addIndex("TESTDATAWITHARRAY", LongIdDescriptor);
        FODBStringIndexDescriptor StrDescriptor2 = new FODBStringIndexDescriptor("ELEMENTSTRING", FODBIndexDescriptor.MULTIPLE, "getStringList()", 10, 10);
        db.addIndex("TESTDATAWITHARRAY", StrDescriptor2);
        FODBIntIndexDescriptor IntDescriptor = new FODBIntIndexDescriptor("ELEMENTINT", FODBIndexDescriptor.MULTIPLE, "getIntList()", 10, 10);
        db.addIndex("TESTDATAWITHARRAY", IntDescriptor);
        FODBLongIndexDescriptor LongDescriptor = new FODBLongIndexDescriptor("ELEMENTLONG", FODBIndexDescriptor.MULTIPLE, "getLongList()", 15, 64);
        db.addIndex("TESTDATAWITHARRAY", LongDescriptor);
      }
    } catch (Throwable ex) {
      ex.printStackTrace();
    }
  }

  public void testDBSearch() {
    LogManager.traceInfo(0, "BEGIN testDBSearch");
    try {
      if (!db.isCollectionExist("TESTSEARCH")) {
        db.createCollection("TESTSEARCH", TestSearchData.class);
        FODBIntIndexDescriptor KeyDescriptor = new FODBIntIndexDescriptor("KEY", FODBIndexDescriptor.UNIQUE, "getKey()", 5);
        db.addIndex("TESTSEARCH", KeyDescriptor);
        FODBIntIndexDescriptor debKeyDescriptor = new FODBIntIndexDescriptor("DIZ", FODBIndexDescriptor.MULTIPLE, "getDizaine()", 5, 5);
        db.addIndex("TESTSEARCH", debKeyDescriptor);
      } else	{
        //remove all key
        for (int i = 0; i < NB_SEARCH_CREATED_DATA; i++) {
          TestSearchData data = new TestSearchData(i);
          db.delete("TESTSEARCH", data);
        }
      }
      
      //add db data
      Array listdata = new Array(NB_SEARCH_CREATED_DATA);
      for (int i = 0; i < NB_SEARCH_CREATED_DATA; i++) {
        TestSearchData data = new TestSearchData(i);
        db.add("TESTSEARCH", data);
        listdata.add(data);
   //     this.testIndexIterator("TESTSEARCH", "getKey()", listdata);
      }
      this.testIndexIterator("TESTSEARCH", "getKey()", listdata);
      this.testInitIndexIterator("TESTSEARCH", "getKey()", listdata);

      //query SELECT * FROM TESTSEARCH
      Query q = db.query();
      q.constrain(TestSearchData.class);
      Query subq = q.descend("getKey()"); //specify an index to request.
      ObjectSet set = q.execute(); //execute with no constrain to get all objects
      this.showSet(set, "QUERY select all : ");

      //query SELECT * FROM TESTSEARCH where getKey=1
      q = db.query();
      q.constrain(TestSearchData.class);
      subq = q.descend("getKey()"); //specify an index to request. change to get default unique index if not present.
      subq.constrain(new Integer(1)).equal();
      set = q.execute();
      this.showSet(set, "QUERY SELECT * FROM TESTSEARCH where getKey=1");

      //query SELECT * FROM TESTSEARCH where getKey<14
      q = db.query();
      q.constrain(TestSearchData.class);
      subq = q.descend("getKey()");
      subq.constrain(new Integer(14)).smaller();
      set = q.execute();
      this.showSet(set, "QUERY SELECT * FROM TESTSEARCH where getKey<14");

      //query SELECT * FROM TESTSEARCH where getKey>24
      q = db.query();
      q.constrain(TestSearchData.class);
      subq = q.descend("getKey()");
      subq.constrain(new Integer(24)).greater();
      set = q.execute();
      this.showSet(set, "QUERY SELECT * FROM TESTSEARCH where getKey>24");

      //query SELECT * FROM TESTSEARCH where getKey>16 AND getKey>23
      q = db.query();
      q.constrain(TestSearchData.class);
      subq = q.descend("getKey()");
      Constraint c = subq.constrain(new Integer(16)).greater();
      Constraint d = subq.constrain(new Integer(23)).smaller();
      c.and(d);
      set = q.execute();
      this.showSet(set, "QUERY SELECT * FROM TESTSEARCH where getkey>16 and getkey<23");

      //query SELECT * FROM TESTSEARCH where getKey>16 AND getKey>23
      q = db.query();
      q.constrain(TestSearchData.class);
      subq = q.descend("getKey()");
      c = subq.constrain(new Integer(16)).greater();
      Query subq2 = subq.descend("getDizaine()");
      d = subq2.constrain(new Integer(2)).equal();
      c.and(d);
      set = q.execute();
      this.showSet(set, "QUERY SELECT * FROM TESTSEARCH where getkey>16 and getDizaine=2");

      //query SELECT * FROM TESTSEARCH where getkey<16 OR getDizaine=2
      q = db.query();
      q.constrain(TestSearchData.class);
      subq = q.descend("getKey()");
      c = subq.constrain(new Integer(16)).smaller();
      subq2 = subq.descend("getDizaine()");
      d = subq2.constrain(new Integer(2)).equal();
      c.or(d);
      set = q.execute();
      this.showSet(set, "QUERY SELECT * FROM TESTSEARCH where getkey<16 OR getDizaine=2");

      //remove all key
      for (int i = 0; i < NB_SEARCH_CREATED_DATA; i++) {
        db.delete("TESTSEARCH", new TestSearchData(i));
      }
    } catch (Throwable ex) {
      ex.printStackTrace();
    }
    LogManager.traceInfo(0, "END testDBSearch");

  }

  private void showSet(ObjectSet set, String info) {
    LogManager.traceInfo(0, info);
    while (set.hasNext()) {
      LogManager.traceInfo(0, set.next());
    }
  }

  public void TestDBAndIndex() {
    try {

      Array inputDataList = new Array(100);
      Array inputTestDataList = new Array(100);
      String tempdir = System.getProperty("user.dir") + File.separator;
      BufferedReader reader = new BufferedReader(new FileReader(tempdir + "inputdata.txt"));
      //write index
      String line = null;
      LogManager.traceInfo(0, "Begin ADD");
      while ((line = reader.readLine()) != null) {
        int lineIndex = line.indexOf(' ');
        if (lineIndex != -1) {
          String key = line.substring(0, lineIndex);
          TestData data = new TestData(key);
          db.add("TESTDATA", data);
          inputTestDataList.add(data);
          inputDataList.add(key);
          this.addKeyToMap(key);
          objectMap.put(key, key);
  //        this.testIndexIterator("TESTDATA", "getKey()", inputTestDataList);
        }

      }
      LogManager.traceInfo(0, "END ADD");
      this.testIndexIterator("TESTDATA", "getKey()", inputTestDataList);
      this.testFindLike(inputDataList, db);
      this.testTree(inputDataList, this.db);
      this.testMultipleIndex(inputDataList, this.db);
      LogManager.traceInfo(0, "END INDEX TEST");

      LogManager.traceInfo(0, "BEGIN DELETE");
      Array removeList = (Array) inputDataList.clone();
      int increment = 100;
      while (inputDataList.size() != 0) {
        for (int i = 0; i < inputDataList.size(); i = i + increment) {
          String key = ((String) inputDataList.get(i));
          db.delete("TESTDATA", new TestData(key));
          inputTestDataList.remove(i);
          inputDataList.remove(i);
          this.removeKeyToMap(key);
 //         LogManager.traceInfo(0, "remove key :"+key);
 //        	this.testIndexIterator("TESTDATA", "getKey()", inputTestDataList);
        }
  //      this.testTree(inputDataList, this.db);
  //      this.testMultipleIndex(inputDataList, this.db);
  //      this.testFindLike(inputDataList, this.db);
        increment = increment / 2;
        if (increment <= 0)
          increment = 1;
      }
      LogManager.traceInfo(0, "END DELETE");

      LogManager.traceInfo(0, "BEGIN TEST DELETED");
     this.testRomvedTree(removeList, db);
      LogManager.traceInfo(0, "END TEST DELETED");

    } catch (Throwable ex) {
      ex.printStackTrace();
    }
  }

  public void testDataWithArray() {
    LogManager.traceInfo(0, "BEGIN testDataWithArray");
   try {
      TestDataWithArray fruits = new TestDataWithArray("Fruits", 1000L);
      fruits.addElement("Fraise");
      fruits.addElement("Cerise");
      fruits.addElement("Pomme");
      fruits.addElement("Banane");
      for (int i = 0; i < 2; i++)
        fruits.addElement(i);
      for (long l = 1980L; l < 1984L; l++)
        fruits.addElement(l);
      db.add("TESTDATAWITHARRAY", fruits);

      TestDataWithArray legumes = new TestDataWithArray("Legumes", 2000L);
      legumes.addElement("Patate");
      legumes.addElement("Haricot");
      legumes.addElement("Courgette");
      legumes.addElement("Concombre");
      legumes.addElement("Epinard");
      for (int i = 10; i < 12; i++)
        legumes.addElement(i);
      for (long l = 2005L; l < 2009L; l++)
        legumes.addElement(l);
      db.add("TESTDATAWITHARRAY", legumes);

      Query q = db.query();
      q.constrain(TestDataWithArray.class);
      Query subq = q.descend("getId()"); //specify an index to request. change to get default unique index if not present.
      ObjectSet set = q.execute();
      this.showSet(set, "Tous les objets");
      
     q = db.query();
     q.constrain(TestDataWithArray.class);
     subq = q.descend("getId()");
     subq.constrain("Fruits").equal();
     set = q.execute();
     this.showSet(set,"Les Fruits");
      
     q = db.query();
     q.constrain(TestDataWithArray.class);
     subq = q.descend("getStringList()");
     set = q.execute();
     this.showSet(set,"Toutes les String");
      
     q = db.query();
     q.constrain(TestDataWithArray.class);
     subq = q.descend("getStringList()");
     subq.constrain("Pomme").equal();
     set = q.execute();
     this.showSet(set,"Est-ce que la Pomme c'est un fruit ?");
       
     q = db.query();
     q.constrain(TestDataWithArray.class);
     subq = q.descend("getIntList()");
     set = q.execute();
     this.showSet(set, "Tous les entiers");
     
     q.constrain(TestDataWithArray.class); subq = q.descend("getIntList()");
     subq.constrain(new Integer(1)).equal(); set = q.execute();
     this.showSet(set,"Groupe qui contient 1 (fruits)");
     
     q.constrain(TestDataWithArray.class); subq = q.descend("getIntList()");
     subq.constrain(new Integer(10)).equal(); set = q.execute();
     this.showSet(set,"Groupe qui contient 10 (legumes)");
     
     q.constrain(TestDataWithArray.class); subq = q.descend("getIntList()");
     subq.constrain(new Integer(10)).greater(); set = q.execute();
     this.showSet(set,"Groupe qui contient > 10 (legumes)");
      
     q.constrain(TestDataWithArray.class); subq = q.descend("getIntList()");
     subq.constrain(new Integer(15)).smaller(); set = q.execute();
     this.showSet(set,"Groupe qui contient < 15 (fruits et legumes)");
      
     q.constrain(TestDataWithArray.class); subq = q.descend("getIntList()");
     subq.constrain(new Integer(5)).smaller(); set = q.execute();
     this.showSet(set,"Groupe qui contient < 5 (fruits)");
     
     q.constrain(TestDataWithArray.class); subq = q.descend("getIntList()");
     subq.constrain(new Integer(9)).equal(); set = q.execute();
     this.showSet(set,"Groupe qui contient 9 (aucun)");
 
      q = db.query();
      q.constrain(TestDataWithArray.class);
      subq = q.descend("getLongList()");
      subq.constrain(new Long(1981L)).smaller();
      set = q.execute();
      this.showSet(set, "L'objet qui inf?rieur ? 1981L (fruits)");

      //remove all keys
      db.deleteWithId("TESTDATAWITHARRAY", "Fruits");

      System.out.println("db.deleteWithId(\"TESTDATAWITHARRAY\",\"Fruits\");");
        
      q.constrain(TestDataWithArray.class); subq = q.descend("getIntList()");
      subq.constrain(new Integer(5)).equal(); set = q.execute();
      this.showSet(set,"Groupe qui contient 5 (fruits)");
       
      q.constrain(TestDataWithArray.class); subq = q.descend("getIntList()");
      subq.constrain(new Integer(12)).equal(); set = q.execute();
      this.showSet(set,"Groupe qui contient 12 (legumes)");
      
      db.deleteWithId("TESTDATAWITHARRAY", "Legumes");
      System.out.println("db.deleteWithId(\"TESTDATAWITHARRAY\",\"Legumes\");");
    } catch (Throwable ex) {
      ex.printStackTrace();
    }
    LogManager.traceInfo(0, "END testDataWithArray");
  }
  
  public void testMultithread()	{
    LogManager.traceDebug(0, "testMultithread BEGIN");
    LogManager.traceDebug(0, "testMultithread LOAD DB");
    String tempdir = System.getProperty("user.dir") + File.separator;
    try	{
    BufferedReader reader = new BufferedReader(new FileReader(tempdir + "inputdata.txt"));
    //write index
    String line = null;
	     while ((line = reader.readLine()) != null) {
	      int lineIndex = line.indexOf(' ');
	      if (lineIndex != -1) {
	        String key = line.substring(0, lineIndex);
	        TestData data = new TestData(key);
	        db.add("TESTDATA", data);
	      }
	    }
    } catch (Exception ex)	{
      LogManager.traceError(0, ex);
      return;
    }
    LogManager.traceDebug(0, "testMultithread END LOAD DB"); 
    
    
    LogManager.traceDebug(0, "testMultithread BEGIN test MULTITHREAD "); 
    TestThreadRequest testth1 = new TestThreadRequest(this.db, 1);
    Thread thread1 = new Thread(testth1);
    thread1.start();
    TestThreadRequest2 testth2 = new TestThreadRequest2(this.db, 2);
    Thread thread2 = new Thread(testth2);
    thread2.start();
    TestThreadRequest testth3 = new TestThreadRequest(this.db, 3);
    Thread thread3 = new Thread(testth3);
    thread3.start();
    TestThreadRequest2 testth4 = new TestThreadRequest2(this.db, 4);
    Thread thread4 = new Thread(testth4);
    thread4.start();
    TestThreadRequest testth5 = new TestThreadRequest(this.db, 5);
    Thread thread5 = new Thread(testth5);
    thread5.start();
    TestThreadRequest2 testth6 = new TestThreadRequest2(this.db, 6);
    Thread thread6 = new Thread(testth6);
    thread6.start();
    
    LogManager.traceDebug(0, "testMultithread END test MULTITHREAD "); 
    
  }

  public static void main(String[] args) {
    LogManager.registerLogManager(null);
    //set to test in multi user mode
 //   PropertiesManager.getManager().addProperty("org.openmobileis.database.fastobjectdb.usermode", "false");
    TestFastObjectDB test = new TestFastObjectDB();
    test.openDB();
    test.TestDBAndIndex();
    test.testDBSearch();

    test.testDataWithArray();
    test.testMultithread();
  }
}
