/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.transaction;

import java.io.IOException;
import java.util.Stack;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.database.fastobjectdb.db.exception.FODBException;

/**
 * Transaction Manager use to manage database transaction 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class Transaction {
	private static final int NONE=0; 
	private static final int BEGIN=1;
	private static final int COMMIT=2;
	private static final int ROLLBACK=3;
	private Array fileList;
	private int transacCounter;
	private int state = Transaction.NONE;
	private Stack threadStack;
	private Thread transactionThread;
  /**
   * 
   */
  public Transaction(Thread th) {
    super();
		this.fileList = new Array(10);
		this.transacCounter = 0;
		this.threadStack = new Stack();
		this.transactionThread = th;
  }
  
  public Thread getTransactionThread()	{
    return this.transactionThread;
  }
  
  public void pushThread(Thread th)	{
    this.threadStack.push(th);
  }
  
  public void popallthread()	{
    Thread th = null;
    while (!threadStack.empty())	{
      th=(Thread)threadStack.pop();
      synchronized(th)	{
        th.notifyAll();
      }
    }
  }
  
  public void addTransactionFile(TransactionFile file)	throws FODBException {
  	if (!fileList.contains(file))	{ //add if not already added
		if (transacCounter > 0)	{ //already begin.
			try	{
				file.open();
			} catch (IOException ex)	{
				this.rollback();
				throw new FODBException(ex);
			}
		}
		fileList.add(file);		
  	}
  }
  
	public void begin() throws FODBException {
		if (transacCounter == 0) {
			int i=0; 
			try	{
				for (i=0; i<fileList.size(); i++)	{
					TransactionFile file = (TransactionFile) fileList.get(i);
					file.open();
				}
			} catch (Throwable ex)	{
				transacCounter = 0;
				this.rollback();
				throw new FODBException("ERROR during transaction begin", ex);
			}
		}
		transacCounter++;
		state = Transaction.BEGIN;
	}
  
	public boolean commit() throws FODBException	{
		boolean ret = false;
		transacCounter--;
		if (transacCounter <= 0) {
			state = Transaction.COMMIT;
			try	{
				while (!fileList.isEmpty())	{
					TransactionFile file = (TransactionFile) fileList.get(0);
					file.close();
					fileList.remove(0);
				}
				ret = true;
			} catch (Throwable ex)	{
				transacCounter = 0;
				ret = this.rollback();
				throw new FODBException("ERROR during transaction commit", ex);
			}
			transacCounter = 0;
		}
		return ret;
	}
  
	public boolean rollback() throws FODBException	{
		boolean ret = false;
		state = Transaction.ROLLBACK;
		transacCounter--;
		if (transacCounter <= 0) {
			try	{
				while (!fileList.isEmpty())	{
					TransactionFile file = (TransactionFile) fileList.get(0);
					file.close();
					fileList.remove(0);
				}
				ret = true;
			} catch (Throwable ex)	{
				transacCounter = 0;
				throw new FODBException("ERROR during transaction rollback", ex);
			}
			transacCounter = 0;
		}
		return ret;
	}

}
