/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.database.fastobjectdb.synchro.server;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.util.StringTokenizer;

import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.codec.GeneralCoder;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.database.fastobjectdb.FODBCollectionDescriptor;
import org.openmobileis.database.fastobjectdb.FODBIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.db.FODBCollection;
import org.openmobileis.database.fastobjectdb.db.crypto.FODBCypher;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;
import org.openmobileis.synchro.algo.replication.SynchroConflicResolver;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPDBSynchroTargetListener;
import org.openmobileis.synchro.openmsp.server.util.OpenMISFile;
import org.openmobileis.synchro.openmsp.server.util.ZipEntryMemoryFile;
import org.openmobileis.synchro.security.auth.Credential;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * To specify the temp directory where generated db file is use the property :org.openmobileis.database.fastobjectdb.synchro.server.generateddb.installpath
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class FODBOpenMSPSynchroTargetWrapper extends OpenMSPDBSynchroTargetListener {
  private FODBSyncTarget syncService;
  private String pregenerateddbpath;

  /**
   * Use this constructor to work with pregenerated DB file for complete synchronisation
   * pregenerateddbpath is the path to the directory where pregenerated collection file are stored.
   */
  public FODBOpenMSPSynchroTargetWrapper(FODBSyncTarget syncService, String pregenerateddbpath) {
    this(syncService);
    this.pregenerateddbpath = pregenerateddbpath;
 }

  /**
   * Use this constructor to generate DB file during synchronisation if needed (complete synchronisation)
   * or when no complete synchronisation is needed.
   */
  public FODBOpenMSPSynchroTargetWrapper(FODBSyncTarget syncService) {
    super();
    //init service data
    this.syncService = syncService;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPDBSynchroTargetListener#getTargetName()
   */
  public String getTargetName() {
    return syncService.getCollectionName();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPDBSynchroTargetListener#connect(org.openmobileis.synchro.security.auth.Credential)
   */
  protected void connect(Credential cred) throws UserNotFoundException, ServiceException {
    syncService.connect(cred);

  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPDBSynchroTargetListener#disconnect()
   */
  protected void disconnect() {
    syncService.disconnect();

  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPDBSynchroTargetListener#getSynchroConflicResolver()
   */
  protected SynchroConflicResolver getSynchroConflicResolver() {
     return syncService.getConflicResolver();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPDBSynchroTargetListener#getAllModifiedAtomicObjectSince(long)
   */
  protected SynchroAtomicObject[] getAllModifiedAtomicObjectSince(long syncNumber) throws OpenMSPException {
    return syncService.getAllModifiedAtomicObjectSince(syncNumber);
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPDBSynchroTargetListener#updateTargetWithSynchroObject(java.lang.Object)
   */
  protected void updateTargetWithSynchroObject(Object syncObject) throws OpenMSPException {
    syncService.updateCollectionObject(syncObject);
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPDBSynchroTargetListener#deleteTargetForSynchroObjectId(java.lang.String)
   */
  protected void deleteTargetForSynchroObjectId(String uid) throws OpenMSPException {
    syncService.deleteCollectionObject(uid);

  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPDBSynchroTargetListener#getTargetObjectWithId(java.lang.String)
   */
  protected Object getTargetObjectWithId(String uid) throws OpenMSPException {
    return syncService.getCollectionObjectWithId(uid);
  }

  /**
   * Return the max number of row that update file can contains If more than max
   * row must be updated on the PDA, the database file will be send. If return
   * -1 (default implementation) max row in not taken into account (ie always
   * update file) If return 0 the database file is always send
   */
  protected int getUpdateMaxNbRow() {
    return syncService.getUpdateMaxNbRow();
  }
 
  /**
    call to process sync command meta data
  */
  protected void processSyncActionMetaData(String metadata) throws ServiceException {
    StringTokenizer token = new StringTokenizer(metadata, ":");
    if (token.hasMoreTokens())  {
      String collectionName = token.nextToken();
      SessionContext context = SessionContextManager.getManager().getSessionContext();
      context.setAttribute("FODBSyncServiceWrapper%class", collectionName);
      
      //manage cypher
      String encodedcypher = token.nextToken();
      if (encodedcypher.equals("nocypher")) {
        context.removeAttribute("FODBSyncServiceWrapper%cypher");
      } else  {
        try {
          ByteArrayInputStream bstr = new ByteArrayInputStream(GeneralCoder.decodeBase64(encodedcypher.getBytes()));
          ObjectInputStream ostr = new ObjectInputStream(bstr);
          FODBCypher cypher =  (FODBCypher)ostr.readObject();
          context.setAttribute("FODBSyncServiceWrapper%cypher", cypher);
        } catch (Throwable ex)  {
          throw new ServiceException(ex);
        }
      }     
      
      //manage collection descriptor
      String coldess = token.nextToken();
      try {
        ByteArrayInputStream bstr = new ByteArrayInputStream(GeneralCoder.decodeBase64(coldess.getBytes()));
        ObjectInputStream ostr = new ObjectInputStream(bstr);
        FODBCollectionDescriptor coldesc =  (FODBCollectionDescriptor)ostr.readObject();
        context.setAttribute("FODBSyncServiceWrapper%coldesc", coldesc);
    } catch (Throwable ex)  {
      throw new ServiceException(ex);
    }
     
      int nbToken = token.countTokens();
      FODBIndexDescriptor[] descList = new FODBIndexDescriptor[nbToken];
      int i=0;
      while (token.hasMoreTokens()) {
        String descriptor = token.nextToken();
        try {
            ByteArrayInputStream bstr = new ByteArrayInputStream(GeneralCoder.decodeBase64(descriptor.getBytes()));
            ObjectInputStream ostr = new ObjectInputStream(bstr);
            FODBIndexDescriptor desc =  (FODBIndexDescriptor)ostr.readObject();
            descList[i] = desc;
        } catch (Throwable ex)  {
          throw new ServiceException(ex);
        }
        i++;
      }
      context.setAttribute("FODBSyncServiceWrapper%desclist", descList);
    }
    
  }


  /* (non-Javadoc)
   * @see org.openmobileis.server.synchro.safeserver.services.DefaultDBService#getDatabaseImportFiles()
   */
  protected OpenMISFile[] getDatabaseImportFiles() throws ServiceException {
    if (this.pregenerateddbpath == null)  {
      return this.generateImportDatabaseFiles();
    } else  {
      return this.getPregenerateDatabaseFiles();
    }
  }
  
  private synchronized OpenMISFile[] getPregenerateDatabaseFiles()  throws ServiceException {
    try {
      //get database name
      /*
      int index  = this.pregenerateddbpath.indexOf(File.separatorChar);
      String databasename = null;
      if (index != -1)  { to do that db must temporary copied.
        databasename = this.pregenerateddbpath.substring(index+1, this.pregenerateddbpath.length());
        String dbRootPath =  this.pregenerateddbpath.substring(0, index);
        FastObjectDB db = FastObjectDB.open(dbRootPath, "databasename");
        syncService.updateSynchroDB(db);      
      } */
      
      //get db collection files
      File dbFiles = new File(this.pregenerateddbpath);
      File[] colFiles = dbFiles.listFiles(new FilenameFilter() {
        public boolean accept(File arg0, String arg1) {
          return arg1.endsWith(".col");
        }
      }); 
      OpenMISFile[] retDbFiles = new OpenMISFile[colFiles.length];
      String dbPath = this.pregenerateddbpath+File.separatorChar;
      String filePath = null;
      for (int i=0; i<retDbFiles.length;i++)  {
        String fileColName =colFiles[i].getName(); 
        filePath = dbPath+fileColName;
        InputStream file = new BufferedInputStream(new FileInputStream(filePath));
        retDbFiles[i] = new ZipEntryMemoryFile("/"+fileColName, file);
        ((ZipEntryMemoryFile)retDbFiles[i]).setZipFileName("/dbfiles/"+this.getTargetName()+".zip");
        file.close();
      }
      
      return retDbFiles;
    } catch (Throwable ex)  {
      throw new ServiceException(ex);
    }
  }
  
  private OpenMISFile[] generateImportDatabaseFiles() throws ServiceException {
    try {
      String installPath = PropertiesManager.getManager().getProperty("org.openmobileis.database.fastobjectdb.synchro.server.generateddb.installpath");
      if (installPath == null)  {
        installPath = System.getProperty("user.dir")+File.separator+"fodbgen"; // default Path
      }
      SessionContext context  = SessionContextManager.getManager().getSessionContext();
      String userId = context.getUserId();
      StringBuffer buff = new StringBuffer(installPath).append(File.separatorChar);
      buff.append("temp").append(File.separatorChar).append(userId);
      String dbRootPath = buff.toString();
      buff.append(File.separatorChar).append("fodb");
      String dbPath = buff.toString();
      buff.append(File.separatorChar).append(syncService.getCollectionName()).append(".col");
      String filePath = buff.toString();
      
      File toDelFile = new File(filePath);
      if (toDelFile.exists()) {
        toDelFile.delete();
        toDelFile = new File(dbPath+File.separatorChar+"fodb.db");
        toDelFile.delete();
      }
      FastObjectDB db = FastObjectDB.open(dbRootPath, "fodb");
      //create collection index
      FODBCollectionDescriptor coldesc =  (FODBCollectionDescriptor)context.getAttribute("FODBSyncServiceWrapper%coldesc");
      db.createCollection(coldesc);
      
      //manage cypher
      FODBCypher cypher = (FODBCypher)context.getAttribute("FODBSyncServiceWrapper%cypher");
      if (cypher != null) {
        FODBCollection col = db.getCollection(syncService.getCollectionName());
        col.setCollectionCypher(cypher);
      }
      FODBIndexDescriptor[] descList = (FODBIndexDescriptor[])context.getAttribute("FODBSyncServiceWrapper%desclist");
      for (int i=0; i<descList.length; i++) {
        if (descList[i] != null)  {
          db.addIndex(syncService.getCollectionName(), descList[i]);
        }
      }
      
      Array objList = syncService.getAllCollectionObject();
      for (int i=0; i<objList.size(); i++)  {
        db.add(syncService.getCollectionName(), objList.get(i));
      }
      syncService.updateSynchroDB(db);
      
      FODBCollection[] colList = db.getDatabaseCollectionArray();
      OpenMISFile[] retDbFiles = new OpenMISFile[colList.length];
      dbPath = dbPath+File.separatorChar;
      for (int i=0; i<retDbFiles.length;i++)  {
        String fileColName =colList[i].getName()+".col"; 
        filePath = dbPath+fileColName;
        InputStream file = new BufferedInputStream(new FileInputStream(filePath));
        retDbFiles[i] = new ZipEntryMemoryFile("/"+fileColName, file);
        ((ZipEntryMemoryFile)retDbFiles[i]).setZipFileName("/dbfiles/"+this.getTargetName()+".zip");
        file.close();
        toDelFile = new File(filePath);
        if (toDelFile.exists()) {
          toDelFile.delete();
        }
      }
      toDelFile = new File(dbPath+File.separatorChar+"fodb.db");
      if (toDelFile.exists()) {
        toDelFile.delete();
      }
      return retDbFiles;
    } catch (Throwable ex)  {
      throw new ServiceException(ex);
    }
  }

}
