/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.database.fastobjectdb.synchro.server;

import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;
import org.openmobileis.synchro.algo.replication.SynchroConflicResolver;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.security.auth.Credential;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * To specify the temp directory where generated db file (during complete synchronisation) is use the property :org.openmobileis.database.fastobjectdb.synchro.server.generateddb.installpath
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public interface FODBSyncTarget {
  
	public String getCollectionName();
	public Object getCollectionObjectWithId(String id) throws OpenMSPException ;
	public void updateCollectionObject(Object obj) throws OpenMSPException ;
	public void deleteCollectionObject(String id) throws OpenMSPException ;
	public SynchroConflicResolver getConflicResolver();
	public Array getAllCollectionObject()  throws OpenMSPException;

	/**
	 * return all atomic object uid modified since specified syncnumber
	 */
	public SynchroAtomicObject[] getAllModifiedAtomicObjectSince(long syncnumber) throws OpenMSPException;

	/**
		Return the max number of row that update file can contains
		If more than max row must be updated on the PDA, the database file
		will be send.
		If return -1 (default implementation) max row in not taken into account (ie always update file)
	*/
	public int getUpdateMaxNbRow();
	
	/**
	 * call by synchronisation service during synchronisation when collection db is generated.
	 * Use to modifiy db objects before the db files are send to the PDA.
	 * @param db
	 */
	public void updateSynchroDB(FastObjectDB db) throws OpenMSPException;
  
  public void connect(Credential cred) throws UserNotFoundException, ServiceException;
  public void disconnect();

}
