/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.embedded.modules;

import java.util.Hashtable;

import org.openmobileis.common.util.log.*;
import org.openmobileis.services.Service;
import org.openmobileis.services.common.ServiceManager;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Class that load module and manage the module life cycle.
 * All module service must implements the ModuleService interface.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public  abstract class ModuleLoader {
  private java.util.HashMap serviceMap;
  
  /** Creates a new instance of ModuleLoader */
  public ModuleLoader() {
    serviceMap = new java.util.HashMap(10);
  }
  
  /**
   * call before the loading of the services
   * @param:
   * @return:
   **/
  public abstract void preLoadingInit();
  
  /**
   * call after the loading of the services
   * @param:
   * @return:
   **/
  public abstract void postLoadingInit();
  
  /**
   * load all service attached to this module and reference them in the specified table.
   * @param: Hashtable that will be filed with all services loaded. The key for each 
   * entry is the serviceName (ie URI).
   * @return:
   **/
  public void loadService(java.util.Hashtable servicesList) {
    try {
      this.preLoadingInit();
      String[] serviceList = this.getServiceClassList();
      if (serviceList != null)	{
	      for (int i=0; i<serviceList.length;i++) {
	        if (serviceList[i] != null) {
	          Service service = (Service) Class.forName(serviceList[i]).newInstance();
	          serviceMap.put(service.getName(), service);
	          // calculate service URI (without /services/
	          String serviceName = service.getServiceUri().substring(ServiceManager.getManager().getServiceBaseURI().length(), service.getServiceUri().length());
	          servicesList.put(serviceName, service);
	        }
	      }
      }
      this.postLoadingInit();
    } catch (Exception ex)  {
      LogManager.traceError(LogServices.WEBSERVICE, "Can't load module");
      LogManager.traceError(LogServices.WEBSERVICE, ex);
    }
  }
  
  public void unloadService(Hashtable servicesList) {
	  try {
		  this.preUnloadingInit();
	      String[] serviceList = this.getServiceClassList();
	      if (serviceList != null)	{
	    	  for (int i=0; i<serviceList.length;i++) {
	    		  if (serviceList[i] != null) {
	    			  Service service = (Service) Class.forName(serviceList[i]).newInstance();
	    			  serviceMap.remove(service.getName());
	    			  // calculate service URI (without /services/
	    			  String serviceName = service.getServiceUri().substring(ServiceManager.getManager().getServiceBaseURI().length()+1, service.getServiceUri().length());
	    			  servicesList.remove(serviceName);
	    		  }
	    	  }
	      }
		  this.postUnloadingInit();
	  } catch (Exception ex)  {
		  LogManager.traceError(LogServices.WEBSERVICE, "Can't load module");
		  LogManager.traceError(LogServices.WEBSERVICE, ex);
	  }
  }
  
  public void preUnloadingInit() {
  }
  public void postUnloadingInit() {
  }
  
  /**
   * call to get all service classes to load.
   * @param:
   * @return:
   */
  public abstract String[] getServiceClassList();
  
  /**
   * return the service with the specified service name if exist. Null otherwise.
   * @param: the service name to look for
   * @return: the service found, null if not loaded.
   */
  public Service getService(String serviceName){
    return (Service)serviceMap.get(serviceName);
  }
  
}
