/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.embedded.util;

import java.io.File;

import org.openmobileis.common.util.exception.DefaultException;
import org.openmobileis.common.util.exception.OpenMISException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Helper class to access native OS specific task that is not correcly
 * done by the JVM.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class SystemAPInotPPC {

  /**
  * Execute a program on mobile and wait until the program ended
  * @param: String path of the program
  * @param: Arguments separated by space
  */
  public static void execProgramAndWait(String path, String args) throws OpenMISException	{
		String execCommand[] = new String[2];
		execCommand[0] = path;
		execCommand[1] = args;
		try	{
		  Process process = Runtime.getRuntime().exec(execCommand,null, new File(System.getProperty("user.dir")));
		  process.waitFor();
		} catch (Exception ex)	{
		  throw new DefaultException(ex);
		}
    
  }

  /**
  * Execute a program on mobile. 
  * @param: String path of the program
  * @param: Arguments separated by space
  */
  public static void execProgram (String path, String args) throws OpenMISException	{  
		String execCommand[] = new String[2];
		execCommand[0] = path;
		execCommand[1] = args;
		try	{
		  Runtime.getRuntime().exec(execCommand,null, new File(System.getProperty("user.dir")));
		} catch (Exception ex)	{
		  throw new DefaultException(ex);
		} 
  }
  
  /**
  * Get time stamp, same behavior as system.currentTimeMillis in milli
  * Used to fix Creme V3.1 Bug
  */
  public static long currentTimeInMillis() {
      return System.currentTimeMillis();
  }
}