/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */

package org.openmobileis.embedded.webserver;

import org.openmobileis.common.util.ObjectPool;
import org.openmobileis.common.util.log.*;
import org.openmobileis.common.util.exception.ServerException;
import java.net.*;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Runnable that handle connexion process in a thread.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class ServerThreadPool extends ObjectPool {

  public ServerThreadPool()  throws ClassNotFoundException, IllegalAccessException, InstantiationException  {
    super("org.openmobileis.embedded.webserver.ServerThread", 5, 5);
  }

  public synchronized void disposeObject(Object object) {
  //  maintener.unregisterConnexion(((ServerThread)object).getConnexion());
    super.disposeObject(object);
  }
  
  public void runServerConnexion(WebServer webserver, Socket socket) {
    ServerThread thread = (ServerThread) super.getObject();
    if (thread == null) {
      LogManager.traceEmergency(LogServices.SYNCHROSERVICE, "Object pool error ServerThread from pool NULL. Create one. System must be repared.");
      thread = new ServerThread();
    }

    try {
   //   maintener.registerConnexion(thread.getConnexion());
      thread.init(webserver, socket, this);
    } catch (Exception ex)  {
      LogManager.trace(new ServerException("Error during threadPool init", ex));
      LogManager.traceError(0, ex);
    }
  }
  
	public void freeResources() throws Exception {
		for (int i = 0; i < super.arrayPool.size(); i++) {
			ServerThread thread = (ServerThread)super.arrayPool.get(i);
			thread.notifyStop();
		}
	}
}

