/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */

package org.openmobileis.embedded.webserver;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.net.Socket;
import java.security.Principal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.ServletInputStream;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.SingleThreadModel;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpUtils;

import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;


public class WebServerConnection implements HttpServletRequest, HttpServletResponse {

    public final static String WWWFORMURLENCODE = "application/x-www-form-urlencoded";
    public final static String TRANSFERENCODING = "Transfer-Encoding";
    public final static String CHUNKED = "chunked";
    public final static String CONTENTLENGTH = "Content-Length";
    public final static String CONTENTTYPE = "Content-Type";

    private Socket socket;
    private WebServer webserver;

    private String reqMethod; // == null by default

    private String reqUriPath;

    private boolean oneOne; // HTTP/1.1 or better

    String reqQuery = null;

    private Vector reqHeaderNames = new Vector();
    private Vector reqHeaderValues = new Vector();
    private Locale locale; // = java.util.Locale.getDefault();

    private int uriLen;
    private static final Hashtable EMPTYHASHTABLE = new Hashtable();

    protected long connexionLifeTime = 0;

    private ServletInputStream in;

    private ServletOutputStream out;

    private boolean reqMime;

    private String reqProtocol;
    private String reqCharEncoding;

    private Hashtable formParameters;

    private Hashtable attributes = new Hashtable();

    private int resCode = -1;

    private String resMessage = null;

    private Hashtable resHeaderNames = new Hashtable();

    private boolean headersWritten = false;

    protected static final SimpleDateFormat expdatefmt = new SimpleDateFormat(
            "EEE, dd-MMM-yyyy HH:mm:ss 'GMT'");

    protected static final SimpleDateFormat headerdateformat = new SimpleDateFormat(
            "EEE, dd MMM yyyy HH:mm:ss z");
    
    private PrintWriter printWriter = null;
    
    public void init(WebServer webserver, Socket socket) 	{
    // Save arguments.
    this.webserver = webserver;
    this.socket = socket;
    out = null;
    in = null;
    oneOne = false;
    reqMethod = null;
    reqUriPath = null;
    reqQuery = null;
    reqProtocol = null;
    reqCharEncoding = null;
    reqHeaderNames = new Vector();
    reqHeaderValues = new Vector();
    formParameters = null;
    attributes = new Hashtable();
    locale = null;
    uriLen = -1;
    resCode = -1;
    connexionLifeTime = 0;
    reqMime = false;
    resMessage = "";
    resHeaderNames = new Hashtable();
    headersWritten = false;
    printWriter = null;
  }

    // Methods from Runnable.
    public void run() {
        try {
            // Get the streams.
            in = new ServeInputStream(socket.getInputStream());
            out = new ServeOutputStream(socket.getOutputStream(), this);
        } catch (IOException e) {
            problem("Getting streams: " + e.getMessage(), SC_BAD_REQUEST);
            LogManager.traceError(0, e);
        }

        // parse the request
        parseRequest();

 //       LogManager.traceDebug(LogServices.WEBSERVICE, socket.getInetAddress().toString()
 //              + ' ' + reqMethod + ' ' + reqUriPath + ' ' + resCode
        /*
         * + (serve.isShowReferer() ? "| " + getHeader("Referer") : "") +
         * (serve.isShowUserAgent() ? "| " + getHeader("User-Agent") : "")
         */
 //       );

        try {
            this.execURI(this.reqUriPath);
        } catch (Exception e) {
            problem("Getting streams: " + e.getMessage(), SC_BAD_REQUEST);
            LogManager.traceError(0, e);
        } finally {
            try {
                //socket.getOutputStream().write(byteOut.toByteArray());
                socket.close();
            } catch (IOException e) {
                org.openmobileis.common.util.log.LogManager
                        .traceError(
                                org.openmobileis.common.util.log.LogServices.WEBSERVICE,
                                "Impossible to close socket : WebServerConnection");
                org.openmobileis.common.util.log.LogManager
                        .traceError(
                                org.openmobileis.common.util.log.LogServices.WEBSERVICE,
                                e);
            }
        }
    }

    private void parseRequest() {
        byte[] lineBytes = new byte[4096];
        int len;
        String line;

        try {
            // Read the first line of the request.
            len = in.readLine(lineBytes, 0, lineBytes.length);
            if (len == -1 || len == 0) {
                problem("Empty request", SC_BAD_REQUEST);
                return;
            }
            line = new String(lineBytes, 0, len);
            StringTokenizer ust = new StringTokenizer(line);
            reqProtocol = null;
            if (ust.hasMoreTokens()) {
                reqMethod = ust.nextToken();
                if (ust.hasMoreTokens()) {
                    reqUriPath = Acme.Utils.urlDecoder(ust.nextToken());
                    if (ust.hasMoreTokens()) {
                        reqProtocol = ust.nextToken();
                        oneOne = !reqProtocol.toUpperCase().equals("HTTP/1.0");
                        reqMime = true;
                        // Read the rest of the lines.
                        String s;
                        while ((s = ((ServeInputStream) in).readLine()) != null) {
                            if (s.length() == 0)
                                break;

                            int c = s.indexOf(':', 0);
                            if (c > 0) {
                                String key = s.substring(0, c).trim();
                                String value = s.substring(c + 1, s.length())
                                        .trim();
                                reqHeaderNames.addElement(key.toLowerCase());
                                reqHeaderValues.addElement(value);
                            } else {
                                LogManager.traceError(LogServices.WEBSERVICE, "header field without ':'");
                            }
                        }
                    } else {
                        reqProtocol = "HTTP/0.9";
                        oneOne = false;
                        reqMime = false;
                    }
                }
            }
            if (reqProtocol == null) {
                problem("Malformed request line", SC_BAD_REQUEST);
                return;
            }
            // Check Host: header in HTTP/1.1 requests.
            if (oneOne) {
                String host = getHeader("host");
                if (host == null) {
                    problem("Host header missing on HTTP/1.1 request",
                            SC_BAD_REQUEST);
                    return;
                }
            }

            // Decode %-sequences.
            //reqUriPath = decode( reqUriPath );
            // Split off query string, if any.
            int qmark = reqUriPath.indexOf('?');
            if (qmark > -1) {
                reqQuery = reqUriPath.substring(qmark + 1);
                reqUriPath = reqUriPath.substring(0, qmark);
            }
            if (CHUNKED.equals(getHeader(TRANSFERENCODING))) {
                setHeader(CONTENTLENGTH, null);
                ((ServeInputStream) in).chunking(true);
            }

        } catch (IOException e) {
            problem("Reading request: " + e.getMessage(), SC_BAD_REQUEST);
        }
    }

    private void execURI(String URI) throws ServletException {
        //manage /index for index servlet
        if (URI.equals("/index")) {
            URI = "/services/index";
            reqUriPath = URI;
        }
        Object[] os = webserver.getServletByURLorClassName(URI);
        if (os[0] == null) {
            os = webserver.getServletByURLorClassName("defaultServlet");
            if (os[0] == null) {
                throw new ServletException(
                        "WebServerConnection::execUri Unable to find file servlet");
            }
            //uriLen = ((Integer) os[1]).intValue();
            uriLen = 0;
            runServlet((HttpServlet) os[0]);
        } else {
            uriLen = ((Integer) os[1]).intValue();
            runServlet((HttpServlet) os[0]);
        }
    }

    /// Writes the status line and message headers for this response to the
    // output stream.
    // @exception IOException if an I/O error has occurred
    void writeHeaders() throws IOException {
        synchronized (this) {
            if (headersWritten)
                return;

            headersWritten = true;
        }
        if (reqMime) {
            //boolean chunked_out = false;
            out.println(reqProtocol + " " + resCode + " " + resMessage);

            Enumeration he = resHeaderNames.keys();
            while (he.hasMoreElements()) {
                String name = (String) he.nextElement();
                Object o = resHeaderNames.get(name);
                if (o instanceof String) {
                    String value = (String) o;
                    if (value != null) // just in case
                        out.println(name + ": " + value);
                    /*
                     * moved to servlet if (TRANSFERENCODING.equals(name) &&
                     * CHUNKED.equals(value)) chunked_out = true;
                     */
                } else if (o instanceof String[]) {
                    String[] values = (String[]) o;
                    out.println(name + ": " + values[0]);
                    for (int i = 0; i < values.length; i++)
                        out.print("," + values[i]);
                    out.println();
                }
            }
            //add specific header to remove page from cache
            out.println("Pragma: no-cache");
        		out.println("Cache-Control: no-cache, must-revalidate");
        		out.println("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
        		out.println("Date: Mon, 26 Jul 1997 05:00:00 GMT");
        		
        		out.println("Last-Modified: Mon, 26 Jul 1997 05:00:00 GMT");
        		out.println("Connection: close" );
            // Marks the beginning of body-content.
            out.println();
            out.flush();
            //if (chunked_out) ((ServeOutputStream)out).setChunked(true);
        }
    }

    private void runServlet(HttpServlet servlete) {
        // Set default response fields.
        setStatus(SC_OK);
        /*
         * setDateHeader("Date", System.currentTimeMillis());
         * setHeader("Server", Serve.Identification.serverName + "/" +
         * Serve.Identification.serverVersion); setHeader("MIME-Version",
         * "1.0");
         */
        try {
          	authenticate();
            if (servlete instanceof SingleThreadModel)
                synchronized (servlete) {
                    servlete.service((ServletRequest) this,
                            (ServletResponse) this);
                }
            else {
                servlete.service((ServletRequest) this, (ServletResponse) this);
            }
            if (((ServeOutputStream)out).isReturnedAsWriter() && printWriter != null) {
                printWriter.flush();
            } else {
                out.flush();
            }
        } catch (IOException e) {
            e.printStackTrace();
            problem("IO problem running servlet: " + e.toString(),
                    SC_BAD_REQUEST);
        } catch (ServletException e) {
            problem("problem running servlet: " + e.toString(), SC_BAD_REQUEST);
        } catch (Exception e) {
            problem("unexpected problem running servlet: " + e.toString(),
                    SC_INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
    }
    
    private boolean authenticate() throws IOException {
        // We don't use any authentification
        return true;
    }
    
    // send an error page
    private void problem(String logMessage, int resCode) {
        LogManager.traceError(LogServices.WEBSERVICE, logMessage);
        try {
          sendError(resCode);
        } catch (IllegalStateException e) { /* ignore */
        } catch (IOException e) { /* ignore */
        }
    }

    private Hashtable getParametersFromRequest() {
        Hashtable result = null;
        //System.out.println("Req:"+reqMethod+" con:"+getContentType()+" eq
        // "+WWWFORMURLENCODE.equals(getContentType()));
        if ("GET".equals(reqMethod)) {
            if (reqQuery != null)
                try {
                    result = HttpUtils.parseQueryString(reqQuery);
                } catch (IllegalArgumentException ex) {
                }
        } else if ("POST".equals(reqMethod))
            if (WWWFORMURLENCODE.equals(getContentType()))
                try {
                    result = HttpUtils.parsePostData(getContentLength(), getInputStream());
                    if (reqQuery != null && reqQuery.length() > 0) {
                        Acme.Utils.putAll(result, HttpUtils.parseQueryString(reqQuery));
                    }
                } catch (Exception ex) {
                    LogManager.traceError(0, ex);
                    LogManager.traceError(LogServices.WEBSERVICE, "Exception " + ex + " at parsing post data of length " + getContentLength());
                }
            else
                try {
                    if (reqQuery != null)
                        result = HttpUtils.parseQueryString(reqQuery);
                } catch (Exception ex) {
                }
        return result != null ? result : EMPTYHASHTABLE;
    }
    
    private void realSendError() throws IOException {
        if (isCommitted())
            throw new IllegalStateException(
                    "Can not send error, headers have been already written");
        synchronized (out) {
            // no more out after error
            ((ServeOutputStream) out).setReturnedAsStream(true);
            ((ServeOutputStream) out).setReturnedAsWriter(true);
        }
        setContentType("text/html");
        StringBuffer sb = new StringBuffer(100);
        sb.append("<HTML><HEAD>").append(
                "<TITLE>" + resCode + " " + resMessage + "</TITLE>").append(
                "</HEAD><BODY BGCOLOR=\"#F1D0F2\">").append(
                "<H2>" + resCode + " " + resMessage + "</H2>").append("<HR>");
        Identification.writeAddress(sb);
        sb.append("</BODY></HTML>");
        setContentLength(sb.length());
        out.print(sb.toString());
        out.flush();
    }




     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     // HttpServletRequest METHODS
     // ***************************************************************************************************************
    
    /// Returns the value of the named attribute of the request, or null if
    // the attribute does not exist. This method allows access to request
    // information not already provided by the other methods in this interface.
    public Object getAttribute(String name) {
        return attributes.get(name);
    }

    // from ServletRequest
    public Enumeration getAttributeNames() {
        return attributes.keys();
    }
     
     public String getAuthType() {
         //return HttpServletRequest.BASIC_AUTH;
         return null;
     }
     
     /// Returns the size of the request entity data, or -1 if not known.
     // Same as the CGI variable CONTENT_LENGTH.
     public int getContentLength() {
         try {
             return this.getIntHeader(CONTENTLENGTH);
         } catch (Exception ex) {
             return -1;
         }
     }
     
     /// Returns the MIME type of the request entity data, or null if
     // not known.
     // Same as the CGI variable CONTENT_TYPE.
     public String getContentType() {
         return getHeader(CONTENTTYPE);
     }
     
     public String getContextPath() {
         return "";
     }
     
     public Cookie[] getCookies() {
         // We don't use cookies
         return null;
     }
     
     public long getDateHeader(String name) {
         String val = getHeader(name);
         if (val == null)
             return 0;
         try {
             return headerdateformat.parse(val).getTime();
         } catch (Exception e) {
             throw new IllegalArgumentException("Value " + val
                     + " can't be converted to Date using "
                     + headerdateformat.toPattern());
         }
     }
     
     public String getHeader(String name) {
         int i = reqHeaderNames.indexOf(name.toLowerCase());
         if (i == -1)
             return null;
         return (String) reqHeaderValues.elementAt(i);
     }
     
     public Enumeration getHeaderNames() {
         return reqHeaderNames.elements();
     }
     
     public Enumeration getHeaders(String header) {
         Vector result = new Vector();
         int i = -1;
         while ((i = reqHeaderNames.indexOf(header.toLowerCase(), i + 1)) >= 0)
             result.addElement(reqHeaderValues.elementAt(i));
         return result.elements();
     }
     
     /// Returns an input stream for reading request data.
     // @exception IllegalStateException if getReader has already been called
     // @exception IOException on other I/O-related errors
     public ServletInputStream getInputStream() throws IOException {
         synchronized (in) {
             if (((ServeInputStream) in).isReturnedAsReader())
                 throw new IllegalStateException("Already returned as a reader.");
             ((ServeInputStream) in).setReturnedAsReader(true);
         }
         return in;
     }

     /// Returns the value of an integer header field.
     // @param name the header field name
     public int getIntHeader(String name) {
         String val = getHeader(name);
         if (val == null)
             return -1;
         
         // We don't use any try catch, as this method must return
         // a NumberFormatException in case of problem
         // (According to the Servlet 2.4 specifications)
         return Integer.parseInt(val);
     }
     
     /**
      * Returns the Internet Protocol (IP) address of the interface on which the
      * request was received.
      * 
      * @return a <code>String</code> containing the IP address on which the
      *         request was received.
      * 
      * @since 2.4
      *  
      */
     public String getLocalAddr() {
       return null; // TODO:
     }
     
     // should decide about supported locales and charsets, no a good decision yet
     public Enumeration getLocales() {
       // TODO: get locales from Accept-Language (RFC 2616)
       //Locale.getAvailableLocales() 
       return null;
     }
     
     /**
      * Returns the host name of the Internet Protocol (IP) interface on which
      * the request was received.
      * 
      * @return a <code>String</code> containing the host name of the IP on
      *         which the request was received.
      * 
      * @since 2.4
      */
     public String getLocalName() {
       return null; // TODO: get bind address
     }
     
     /**
      * Returns the Internet Protocol (IP) port number of the interface on which
      * the request was received.
      * 
      * @return an integer specifying the port number
      * 
      * @since 2.4
      */
     public int getLocalPort() {
         return webserver.port;
     }
     
     public String getMethod() {
         return reqMethod;
     }
     
     /// Returns the value of the specified query string parameter, or null
     // if not found.
     // @param name the parameter name
     public String getParameter(String name) {
         String[] params = getParameterValues(name);
         if (params == null || params.length == 0)
             return null;

         return params[0];
     }
     
     /** 
      * Returns a java.util.Map of the parameters of this request.
      * Request parameters are extra information sent with the request.
      * For HTTP servlets, parameters are contained in the query string or posted form data.
        * @return an immutable java.util.Map containing parameter names as keys
        *  and parameter values as map values. The keys in the parameter map are of type String.
        *  The values in the parameter map are of type String array.
      * @since 2.3
      */
     public java.util.Map getParameterMap() {
       return (java.util.Map) formParameters;
     }
     
     /// Returns the parameter names for this request.
     public Enumeration getParameterNames() {
         if (formParameters == null)
             formParameters = getParametersFromRequest();
         return formParameters.keys();
     }
     
     /// Returns the values of the specified parameter for the request as an
     // array of strings, or null if the named parameter does not exist.
     public String[] getParameterValues(String name) {
         if (formParameters == null)
             getParameterNames();

         return (String[]) formParameters.get(name);
     }
     
     public String getPathInfo() {
         // In this server, the entire path is regexp-matched against the
         // servlet pattern, so there's no good way to distinguish which
         // part refers to the servlet.
         return uriLen >= reqUriPath.length() ? null : reqUriPath.substring(uriLen);
     }
     
     public String getPathTranslated() {
         // In this server, the entire path is regexp-matched against the
         // servlet pattern, so there's no good way to distinguish which
         // part refers to the servlet.
         return getRealPath(getPathInfo());
     }
     
     /// Returns the protocol and version of the request as a string of
     // the form <protocol>/<major version>.<minor version>.
     // Same as the CGI variable SERVER_PROTOCOL.
     public String getProtocol() {
       return reqProtocol;
     }
     
     // Returns the query string part of the servlet URI, or null if not known.
     // Same as the CGI variable QUERY_STRING.
     public String getQueryString() {
         return reqQuery;
     }
     
     /// Returns a buffered reader for reading request data.
     // @exception UnsupportedEncodingException if the character set encoding isn't supported
     // @exception IllegalStateException if getInputStream has already been called
     // @exception IOException on other I/O-related errors
     public BufferedReader getReader() throws IOException {
       synchronized (in) {
         if (((ServeInputStream) in).isReturnedAsStream())
           throw new IllegalStateException("Already returned as a stream.");
         ((ServeInputStream) in).setReturnedAsStream(true);
       }
       if (reqCharEncoding != null)
         try {
           return new BufferedReader(new InputStreamReader(in, reqCharEncoding));
         } catch (UnsupportedEncodingException uee) {
         }
       return new BufferedReader(new InputStreamReader(in));
     }
     
     /// Applies alias rules to the specified virtual path and returns the
     // corresponding real path, or null if the translation can not be
     // performed for any reason.  For example, an HTTP servlet would
     // resolve the path using the virtual docroot, if virtual hosting is
     // enabled, and with the default docroot otherwise.  Calling this
     // method with the string "/" as an argument returns the document root.
     public String getRealPath(String path) {
       return webserver.getRealPath(path);
     }
     
     /// Returns the IP address of the agent that sent the request.
     // Same as the CGI variable REMOTE_ADDR.
     public String getRemoteAddr() {
       return socket.getInetAddress().getHostAddress().toString();
     }

     /// Returns the fully qualified host name of the agent that sent the
     // request.
     // Same as the CGI variable REMOTE_HOST.
     public String getRemoteHost() {
       String result = socket.getInetAddress().getHostName();
       return result != null ? result : getRemoteAddr();
     }
     
     /**
      * Returns the Internet Protocol (IP) source port of the client or last
      * proxy that sent the request.
      * 
      * @return an integer specifying the port number
      * 
      * @since 2.4
      */
     public int getRemotePort() {
         // TODO : Actually returns local port
         return webserver.port;
     }
     
     /// Returns the name of the user making this request, or null if not known.
     // Same as the CGI variable REMOTE_USER.
     public String getRemoteUser() {
         // We don't use user authentification.
         return null;
     }
     
     public RequestDispatcher getRequestDispatcher(String urlpath) {
         // TODO: calculate dispacter relatively the current path
         return null; // we don't provide resource dispatching in this way
       }
     
     public String getRequestedSessionId() {
         // This WEB server only handles one session.
         return this.getSession().getId();
     }
     
     public String getRequestURI() {
         return reqUriPath;
     }

     public StringBuffer getRequestURL() {
         return new StringBuffer().append(getScheme()).append("://").append(this.getServerName())
         		.append(webserver.port == 80 ? "" : ":"+String.valueOf(webserver.port)).append(getRequestURI());
     }
     
     ///  Returns the scheme of the URL used in this request, for example
     // "http", "https", or "ftp".  Different schemes have different rules
     // for constructing URLs, as noted in RFC 1738.  The URL used to create
     // a request may be reconstructed using this scheme, the server name
     // and port, and additional information such as URIs.
     public String getScheme() {
       if (socket.getClass().getName().toUpperCase().indexOf("SSL") < 0)
         return "http";
       else
         return "https";
     }
     
     /// Returns the host name of the server as used in the <host> part of
     // the request URI.
     // Same as the CGI variable SERVER_NAME.
     public String getServerName() {
       String serverName;
       int serverPort = 80;
       serverName = getHeader("Host");
       if (serverName != null && serverName.length() > 0) {
         int colon = serverName.indexOf(':');
         if (colon >= 0) {
           if (colon < serverName.length())
             serverPort = Integer.parseInt(serverName.substring(colon + 1));
           serverName = serverName.substring(0, colon);
         }
       }

       if (serverName == null) {
         try {
           serverName = InetAddress.getLocalHost().getHostName();
         } catch (java.net.UnknownHostException ignore) {
           serverName = "127.0.0.0";
         }
       }

       int slash = serverName.indexOf("/");
       if (slash >= 0)
         serverName = serverName.substring(slash + 1);
       return serverName;
     }

     /// Returns the port number on which this request was received as used in
     // the <port> part of the request URI.
     // Same as the CGI variable SERVER_PORT.
     public int getServerPort() {
       return socket.getLocalPort();
     }
     
     public String getServletPath() {
         // In this server, the entire path is regexp-matched against the
         // servlet pattern, so there's no good way to distinguish which
         // part refers to the servlet.
         return uriLen > 0 ? reqUriPath.substring(0, uriLen) : "";
     }
     
     public HttpSession getSession() {
         return this.getSession(true);
     }

     public HttpSession getSession(boolean create) {
         // Only one seesion context
       SessionContext session = SessionContextManager.getManager().getSessionContext("OPENMOBILEISSESSION");
       if (session == null)  {
         synchronized (SessionContextManager.getManager()) {
           session = SessionContextManager.getManager().getSessionContext("OPENMOBILEISSESSION");
           if (session == null)  {
             session = SessionContextManager.getManager().createSessionContext("OPENMOBILEISSESSION");
           }
         }
       }
       SessionContextManager.getManager().joinSessionContext(session.getId());
       return SessionContextManager.getManager().getSessionContext();
     }
     
     public Principal getUserPrincipal() {
         // We don't use user authentification.
         return null;
     }
     
     public boolean isRequestedSessionIdFromCookie() {
         // We don't use cookies.
         return false;
     }
     
     public boolean isRequestedSessionIdFromURL() {
         // We only use on session for this web browser.
         return false;
     }

     public boolean isRequestedSessionIdFromUrl() {
         // We only use on session for this web browser.
         return false;
     }
     
     public boolean isRequestedSessionIdValid() {
         HttpSession session = this.getSession();
         if (session != null) {
             return true;
         }
         return false;
     }
     
     public boolean isSecure() {
         return false;
       }
     
     public boolean isUserInRole(String arg0) {
         // We don't use authentification.
         return false;
     }
     
     public void removeAttribute(String name) {
         // Do nothing
     }
     
     public void setAttribute(String key, Object o) {
         attributes.put(key, o);
       }
     
     /**
      * Overrides the name of the character encoding used in the body of this request.
      *  This method must be called prior to reading request parameters or reading input using getReader().
      * @param a - String containing the name of the chararacter encoding.
      * @throws java.io.UnsupportedEncodingException - if this is not a valid encoding
      * @since JSDK 2.3
      */
     public void setCharacterEncoding(String _enc) {
       reqCharEncoding = _enc;
     }
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     // HttpServletRequest AND HttpServletResponse METHODS
     // ***************************************************************************************************************
     
     /// Returns the character set encoding used for this MIME body. The
     // character encoding is either the one specified in the assigned
     // content type, or one which the client understands. If no content
     // type has yet been assigned, it is implicitly set to text/plain.
     public String getCharacterEncoding() {
         String ct = (String) resHeaderNames.get(CONTENTTYPE);
         if (ct != null) {
             int scp = ct.indexOf(';');
             if (scp > 0) {
                 scp = ct.toLowerCase().indexOf("charset=", scp);
                 if (scp >= 0) {
                     ct = ct.substring(scp + 8);
                     scp = ct.indexOf(' ');
                     if (scp > 0)
                         ct = ct.substring(0, scp);
                     scp = ct.indexOf(';');
                     if (scp > 0)
                         ct = ct.substring(0, scp);
                     return ct;
                 }
             }
         }
         return null;
     }
     
     public Locale getLocale() {
         return locale;
     }
      

     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     // HttpServletResponse METHODS
     // ***************************************************************************************************************
     
     /// Adds the specified cookie to the response. It can be called
     // multiple times to set more than one cookie.
     public void addCookie(Cookie cookie) {
         // We don't use cookies.
     }
     
     public void addDateHeader(String header, long date) {
         addHeader(header, expdatefmt.format(new Date(date)));
     }
     
     public void addHeader(String header, String value) {
         Object o = resHeaderNames.get(header);
         if (o == null)
             setHeader(header, value);
         else {
             if (o instanceof String[]) {
                 String[] oldVal = (String[]) o;
                 String[] newVal = new String[oldVal.length + 1];
                 System.arraycopy(oldVal, 0, newVal, 0, oldVal.length);
                 newVal[oldVal.length] = value;
                 resHeaderNames.put(header, newVal);
             } else if (o instanceof String) {
                 String[] newVal = new String[2];
                 newVal[0] = (String) o;
                 newVal[1] = value;
                 resHeaderNames.put(header, newVal);
             } else
                 throw new RuntimeException("Invalid content of header hash - "
                         + o.getClass().getName());
         }
     }
     
     public void addIntHeader(String header, int value) {
         addHeader(header, Integer.toString(value));
     }
     
     /// Checks whether the response message header has a field with the
     // specified name.
     public boolean containsHeader(String name) {
         return resHeaderNames.contains(name);
     }
     
     /// Encodes the specified URL for use in the sendRedirect method or, if
     // encoding is not needed, returns the URL unchanged. The
     // implementation of this method should include the logic to determine
     // whether the session ID needs to be encoded in the URL. Because the
     // rules for making this determination differ from those used to
     // decide whether to encode a normal link, this method is seperate
     // from the encodeUrl method.
     // <P>
     // All URLs sent to the HttpServletResponse.sendRedirect method should be
     // run through this method. Otherwise, URL rewriting cannot be used with
     // browsers which do not support cookies.
     public String encodeRedirectUrl(String url) {
         return url;
     }
     
     public String encodeRedirectURL(String url) {
         return url; // as is
     }
     
     // URL session-encoding stuff. Not implemented, but the API is here
     // for compatibility.

     /// Encodes the specified URL by including the session ID in it, or, if
     // encoding is not needed, returns the URL unchanged. The
     // implementation of this method should include the logic to determine
     // whether the session ID needs to be encoded in the URL. For example,
     // if the browser supports cookies, or session tracking is turned off,
     // URL encoding is unnecessary.
     // <P>
     // All URLs emitted by a Servlet should be run through this method.
     // Otherwise, URL rewriting cannot be used with browsers which do not
     // support cookies.
     public String encodeUrl(String url) {
         return url;
     }
     
     // JSDK 2.1 extension
     public String encodeURL(String url) {
         return url; // as is
     }
     
     // 2.2
     // do not use buffer
     public void flushBuffer() {
     }
     
	public int getBufferSize() {
         return 0;
	}
	
    /// Returns an output stream for writing response data.
    public ServletOutputStream getOutputStream() throws IOException {
      synchronized (out) {
        if (((ServeOutputStream) out).isReturnedAsWriter())
          throw new IllegalStateException("Already returned as a writer");
        ((ServeOutputStream) out).setReturnedAsStream(true);
      }
      return out;
    }
    
    /// Returns a print writer for writing response data.  The MIME type of
    // the response will be modified, if necessary, to reflect the character
    // encoding used, through the charset=... property.  This means that the
    // content type must be set before calling this method.
    // @exception UnsupportedEncodingException if no such encoding can be provided
    // @exception IllegalStateException if getOutputStream has been called
    // @exception IOException on other I/O errors
    public PrintWriter getWriter() throws IOException {
      synchronized (out) {
        if (((ServeOutputStream) out).isReturnedAsStream())
          throw new IllegalStateException("Already was returned as servlet output stream");
        ((ServeOutputStream) out).setReturnedAsWriter(true);
      }
      String encoding = getCharacterEncoding();
      if (encoding != null) {
        //return new PrintWriter(new OutputStreamWriter(out, encoding));
          printWriter = new PrintWriter(new OutputStreamWriter(out, encoding));
      } else {
        //return new PrintWriter(out);
          printWriter = new PrintWriter(out);
      }
      return printWriter;
    }
    
    /**
     * Returns a boolean indicating if the response has been committed.
     *  A commited response has already had its status code and headers written.
     * @return
     * a boolean indicating if the response has been committed
     * @see 
     * setBufferSize(int), getBufferSize(), flushBuffer(), reset()
     */
     // a caller should think about syncronization 
     public boolean isCommitted() {
       return headersWritten;
     }
     
     /** Clears any data that exists in the buffer as well as the status code and headers.
      *  If the response has been committed, this method throws an IllegalStateException.
      *  @throws java.lang.IllegalStateException - if the response has already been committed
      *  @see  
      *     setBufferSize(int), getBufferSize(), flushBuffer(), isCommitted()
      */
     public void reset() {
       if (!isCommitted()) {
         resHeaderNames.clear();
       } else
         throw new IllegalStateException("Header have already been committed.");
     }
     
     public void resetBuffer() {
         throw new IllegalStateException("The method not implemented");
     }
	
     /// Writes an error response using the specified status code and message.
     // @param resCode the status code
     // @param resMessage the status message
     // @exception IOException if an I/O error has occurred
     public void sendError(int resCode, String resMessage) throws IOException {
         setStatus(resCode, resMessage);
         realSendError();
     }

     /// Writes an error response using the specified status code and a default
     // message.
     // @param resCode the status code
     // @exception IOException if an I/O error has occurred
     public void sendError(int resCode) throws IOException {
         setStatus(resCode);
         realSendError();
     }

     /// Sends a redirect message to the client using the specified redirect
     // location URL.
     // @param location the redirect location URL
     // @exception IOException if an I/O error has occurred
     public void sendRedirect(String location) throws IOException {
         if (isCommitted())
             throw new IllegalStateException(
                     "Can not redirect, headers have been already written");
         synchronized (out) {
             // no more out after redirect
             ((ServeOutputStream) out).setReturnedAsStream(true);
             ((ServeOutputStream) out).setReturnedAsWriter(true);
         }
         setHeader("Location", location);
         setStatus(SC_MOVED_TEMPORARILY);
         setContentType("text/html");
         StringBuffer sb = new StringBuffer(200);
         sb.append("<HTML><HEAD>" + "<TITLE>" + SC_MOVED_TEMPORARILY
                 + " Moved</TITLE>" + "</HEAD><BODY BGCOLOR=\"#F1D0F2\">"
                 + "<H2>" + SC_MOVED_TEMPORARILY + " Moved</H2>"
                 + "This document has moved <a href=" + location + ">here.<HR>");
         Identification.writeAddress(sb);
         sb.append("</BODY></HTML>");
         setContentLength(sb.length());
         // to avoid further out
         out.print(sb.toString());
         out.flush();
     }
     
     // 2.2
     // do not use buffer
     public void setBufferSize(int size) {
     }
     
     /// Sets the content length for this response.
     // @param length the content length
     public void setContentLength(int length) {
       setIntHeader(CONTENTLENGTH, length);
     }
     
     /// Sets the content type for this response.
     // @param type the content type
     public void setContentType(String type) {
       setHeader(CONTENTTYPE, type != null ? type : "Unknown");
     }
     
     /// Sets the value of a date header field.
     // @param name the header field name
     // @param value the header field date value
     public void setDateHeader(String name, long value) {
         setHeader(name, expdatefmt.format(new Date(value)));
     }
     
     /// Sets the value of a header field.
     // @param name the header field name
     // @param value the header field value
     public void setHeader(String name, String value) {
         resHeaderNames.put(name, value);
     }
     
     /// Sets the value of an integer header field.
     // @param name the header field name
     // @param value the header field integer value
     public void setIntHeader(String name, int value) {
         setHeader(name, Integer.toString(value));
     }
     
     /**
      * Sets the locale of the response, setting the headers (including the Content-Type's charset)
      *  as appropriate. This method should be called before a call to getWriter().
      *  By default, the response locale is the default locale for the server.
      * @param loc - the locale of the response
      * @see  
      * getLocale()
      */
     public void setLocale(Locale locale) {
       this.locale = locale;
     }
     
     /// Sets the status code and a default message for this response.
     // @param resCode the status code
     public void setStatus(int resCode) {
         switch (resCode) {
         case SC_CONTINUE:
             setStatus(resCode, "Continue");
             break;
         case SC_SWITCHING_PROTOCOLS:
             setStatus(resCode, "Switching protocols");
             break;
         case SC_OK:
             setStatus(resCode, "Ok");
             break;
         case SC_CREATED:
             setStatus(resCode, "Created");
             break;
         case SC_ACCEPTED:
             setStatus(resCode, "Accepted");
             break;
         case SC_NON_AUTHORITATIVE_INFORMATION:
             setStatus(resCode, "Non-authoritative");
             break;
         case SC_NO_CONTENT:
             setStatus(resCode, "No content");
             break;
         case SC_RESET_CONTENT:
             setStatus(resCode, "Reset content");
             break;
         case SC_PARTIAL_CONTENT:
             setStatus(resCode, "Partial content");
             break;
         case SC_MULTIPLE_CHOICES:
             setStatus(resCode, "Multiple choices");
             break;
         case SC_MOVED_PERMANENTLY:
             setStatus(resCode, "Moved permanentently");
             break;
         case SC_MOVED_TEMPORARILY:
             setStatus(resCode, "Moved temporarily");
             break;
         case SC_SEE_OTHER:
             setStatus(resCode, "See other");
             break;
         case SC_NOT_MODIFIED:
             setStatus(resCode, "Not modified");
             break;
         case SC_USE_PROXY:
             setStatus(resCode, "Use proxy");
             break;
         case SC_BAD_REQUEST:
             setStatus(resCode, "Bad request");
             break;
         case SC_UNAUTHORIZED:
             setStatus(resCode, "Unauthorized");
             break;
         case SC_PAYMENT_REQUIRED:
             setStatus(resCode, "Payment required");
             break;
         case SC_FORBIDDEN:
             setStatus(resCode, "Forbidden");
             break;
         case SC_NOT_FOUND:
             setStatus(resCode, "Not found");
             break;
         case SC_METHOD_NOT_ALLOWED:
             setStatus(resCode, "Method not allowed");
             break;
         case SC_NOT_ACCEPTABLE:
             setStatus(resCode, "Not acceptable");
             break;
         case SC_PROXY_AUTHENTICATION_REQUIRED:
             setStatus(resCode, "Proxy auth required");
             break;
         case SC_REQUEST_TIMEOUT:
             setStatus(resCode, "Request timeout");
             break;
         case SC_CONFLICT:
             setStatus(resCode, "Conflict");
             break;
         case SC_GONE:
             setStatus(resCode, "Gone");
             break;
         case SC_LENGTH_REQUIRED:
             setStatus(resCode, "Length required");
             break;
         case SC_PRECONDITION_FAILED:
             setStatus(resCode, "Precondition failed");
             break;
         case SC_REQUEST_ENTITY_TOO_LARGE:
             setStatus(resCode, "Request entity too large");
             break;
         case SC_REQUEST_URI_TOO_LONG:
             setStatus(resCode, "Request URI too large");
             break;
         case SC_UNSUPPORTED_MEDIA_TYPE:
             setStatus(resCode, "Unsupported media type");
             break;
         case SC_INTERNAL_SERVER_ERROR:
             setStatus(resCode, "Internal server error");
             break;
         case SC_NOT_IMPLEMENTED:
             setStatus(resCode, "Not implemented");
             break;
         case SC_BAD_GATEWAY:
             setStatus(resCode, "Bad gateway");
             break;
         case SC_SERVICE_UNAVAILABLE:
             setStatus(resCode, "Service unavailable");
             break;
         case SC_GATEWAY_TIMEOUT:
             setStatus(resCode, "Gateway timeout");
             break;
         case SC_HTTP_VERSION_NOT_SUPPORTED:
             setStatus(resCode, "HTTP version not supported");
             break;
         default:
             setStatus(resCode, "");
             break;
         }
     }
     
     /// Sets the status code and message for this response.
     // @param resCode the status code
     // @param resMessage the status message
     public void setStatus(int resCode, String resMessage) {
         //	if (this.resCode > 0 && this.resCode != SC_OK)
         //		throw new IllegalStateException("Result code "+this.resCode+" was
         // already set.");
         this.resCode = resCode;
         this.resMessage = resMessage;
     }


     
     
}