/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2005 Modification Romain Beaugrand
 * 
 */

package org.openmobileis.embedded.webserver.test.services;

import java.io.IOException;
import java.util.Enumeration;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmobileis.services.TemplateService;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;
import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * URL : http://127.0.0.1:9091/services/test
 * @author Romain Beaugrand
 *
 */
public class TestServletService extends TemplateService {
    
    private static final String templateName = "testservlettemplates/testservlet.htm";

    public String runTemplate(HttpServletRequest req, HttpServletResponse res, TemplateModelRoot templateData) throws ServletException, IOException {
        
        templateData.put("authType", new SimpleScalar(req.getAuthType()));
        
        Enumeration enuma = req.getAttributeNames();
        SimpleList attributesList = new SimpleList();
        while (enuma.hasMoreElements()) {
            SimpleHash attribute = new SimpleHash();
            String attributeName = (String)enuma.nextElement();
            String attributeValue = (String)req.getAttribute(attributeName);
            attribute.put("name", attributeName);
            attribute.put("value", attributeValue);
            attributesList.add(attribute);
        }
        templateData.put("attributes", attributesList);
        
        templateData.put("charEncoding", new SimpleScalar(req.getCharacterEncoding()));
        templateData.put("contentLength", new SimpleScalar(Integer.toString(req.getContentLength())));
        templateData.put("contentType", new SimpleScalar(req.getContentType()));
        templateData.put("contextPath", new SimpleScalar(req.getContextPath()));
        
        enuma = req.getHeaderNames();
        SimpleList headerList = new SimpleList();
        while (enuma.hasMoreElements()) {
            SimpleHash header = new SimpleHash();
            String headerName = (String)enuma.nextElement();
            String headerValue = req.getHeader(headerName);
            header.put("name", headerName);
            header.put("value", headerValue);
            headerList.add(header);
        }
        templateData.put("headers", headerList);
        
        //templateData.put("localAddr", new SimpleScalar(req.getLocalAddr()));
        if (req.getLocale() != null) {
            templateData.put("locale", new SimpleScalar(req.getLocale().toString()));
        }
        //templateData.put("localName", new SimpleScalar(req.getLocalName()));
        templateData.put("method", new SimpleScalar(req.getMethod()));
        
        enuma = req.getParameterNames();
        SimpleList parametersList = new SimpleList();
        while (enuma.hasMoreElements()) {
            SimpleHash parameter = new SimpleHash();
            String parameterName = (String)enuma.nextElement();
            String parameterValue = req.getParameter(parameterName);
            parameter.put("name", parameterName);
            parameter.put("value", parameterValue);
            parametersList.add(parameter);
        }
        templateData.put("parameters", parametersList);
        
        templateData.put("pathInfo", new SimpleScalar(req.getPathInfo()));
        templateData.put("pathTranslated", new SimpleScalar(req.getPathTranslated()));
        templateData.put("protocol", new SimpleScalar(req.getProtocol()));
        templateData.put("queryString", new SimpleScalar(req.getQueryString()));
        templateData.put("remoteAddr", new SimpleScalar(req.getRemoteAddr()));
        templateData.put("remoteHost", new SimpleScalar(req.getRemoteHost()));
        templateData.put("remoteUser", new SimpleScalar(req.getRemoteUser()));
        templateData.put("requestSessionId", new SimpleScalar(req.getRequestedSessionId()));
        templateData.put("requestURI", new SimpleScalar(req.getRequestURI()));
        //templateData.put("remotePort", new SimpleScalar(Integer.toString(req.getRemotePort())));
        templateData.put("requestURL", new SimpleScalar(req.getRequestURL().toString()));
        templateData.put("scheme", new SimpleScalar(req.getScheme()));
        templateData.put("serverName", new SimpleScalar(req.getServerName()));
        templateData.put("servletPath", new SimpleScalar(req.getServletPath()));
        templateData.put("serverPort", new SimpleScalar(Integer.toString(req.getServerPort())));
        
        //wait a few second
        try	{
          Thread.currentThread().sleep(10000);
        } catch (Throwable ex)	{
          ex.printStackTrace();
        }
        
        return templateName;
    }
    
    public String getName() {
    	return "TestServlet";
    }

}
