/*
 *        OpenMobileIS - a free Java Framework for mobile applications
 *
 *   Copyright (C) 2002  Philippe Delrieu.
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Philippe Delrieu kept the rigth to distribute all code Copyrighted by philippe Delrieu
 *  under other licence term even commercial one.
 *  
 *  Modifications :
 *  2002 Creation P.Delrieu
 * 
 */

package org.openmobileis.modules.common.log.client;

import java.io.File;

import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.codec.GeneralCoder;
import org.openmobileis.common.util.file.FileUtilities;
import org.openmobileis.common.util.log.FileOpenCloseLogManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.embedded.util.ServicePropertiesManager;
import org.openmobileis.synchro.client.SynchroDescriptor;
import org.openmobileis.synchro.journal.JournalEntry;
import org.openmobileis.synchro.journal.JournalManager;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener;
import org.openmobileis.synchro.openmsp.client.OpenMSPSynchroManager;
import org.openmobileis.synchro.openmsp.protocol.Command;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.DataItem;
import org.openmobileis.synchro.openmsp.protocol.Element;
import org.openmobileis.synchro.openmsp.protocol.Item;
import org.openmobileis.synchro.openmsp.protocol.Message;
import org.openmobileis.synchro.openmsp.protocol.RequestCommand;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.security.auth.Credential;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  mail synchro listeners
 * 
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class LogsSynchroListener implements OpenMSPSyncListener {

  public static final String LOG_LISTENER_NAME = "FWKLogSynchroListener";
  
  private boolean syncGoOn = false;

  /** 
   * status of last synchro 
   */

  private int errorStatus;

  public LogsSynchroListener() {
    String status = ServicePropertiesManager.getManager().getProperty(this.getSyncName(), "synchroStatus");
    if (status == null) {
      errorStatus = Status.STATUS_OK;
    }  else {
      errorStatus = Integer.parseInt(status);
    }
 //   JournalManager.getManager().registerJournalLogRenderer(new SimpleLogRenderer(LogsSynchroListener.LOG_LISTENER_NAME, "Log"));
  }

  public void setSynchroStatus(int status) {
    errorStatus = status;
  }
  
  public int getSynchroStatus() {
    return this.errorStatus;
  }

  public boolean isSyncOK() {
    return (errorStatus == Status.STATUS_OK);
  }

  public void notifySynchroFailure() {
    this.setSynchroStatus(Status.STATUS_FAILED);
    JournalManager.getManager().saveJournalEntry(LogsSynchroListener.LOG_LISTENER_NAME, JournalEntry.ERROR_SYNC, "echec", Status.STATUS_FAILED);
  }

  public void receiveStatusCommand( Status statusCommande, ContainerMessage initialCommand)  throws OpenMSPException  {
    // manage error message first
    // manage mail send. delete sent mails.
    if (statusCommande.getStatus() != Status.STATUS_OK) {
      OpenMSPSynchroManager.getManager().notifyFailure(this.getSyncName());
    } else {
			String fileName = PropertiesManager.getManager().getProperty("org.openmobileis.common.log.archivefile");
			File file = new File(fileName);
      file.delete();
		}
		this.setSynchroStatus(statusCommande.getStatus());
  }

  /*
  
  * Event sent when the sync manager sends a OpenMSP message to the server
  
  */
  public void sendData(Message message) throws OpenMSPException   {
    if (!this.isSyncOK()) {
      return; // return if an error occurs during synchro.
    }

    if (!syncGoOn) { // synchronize data only for the first call.
    return;
    }
    syncGoOn = false;
    try {
      //get all log files; 
      if (LogManager.getInstance() instanceof FileOpenCloseLogManager)	{ 
        Command syncCommand = new Command(Element.SYNC, LogsSynchroListener.LOG_LISTENER_NAME, "FWKSyncLogsManager");
        ContainerMessage syncContainer = new ContainerMessage(syncCommand);
        Item data = null;
        // manage new mail
        RequestCommand addCommand = null;
        addCommand = new RequestCommand(Element.ADD);
        ContainerMessage addMailContainer = new ContainerMessage(addCommand);
        File logfile = ((FileOpenCloseLogManager)LogManager.getInstance()).getLogFile();
        byte[] fileData = GeneralCoder.encodeBase64(FileUtilities.readFile(logfile.getAbsolutePath()));
        data = new DataItem(Element.ITEM, logfile.getName(), new String(fileData), null, null);
        addMailContainer.add(data);
        syncContainer.add(addMailContainer);
        message.add(syncContainer);
      }
    } catch (Throwable ex)  {
      this.setSynchroStatus(Status.STATUS_FAILED);
      LogManager.traceError(0, ex);
    }
 }

  /**
   * Event sent when the sync manager receives Sync command openML messages from the server
   * @param ContainerMessage container : container containing the message
   * @param long newSessionID : the session id use for this synchro for this listener message.
   */
  public void receiveSyncCommand( ContainerMessage syncContainer, long newSyncNumber)  throws OpenMSPException  {
    
  }

  /**
   * Event sent when the sync manager receives MAP command openML messages from the server
   * @param ContainerMessage container : container containing the message
   * @param long newSessionID : the session id use for this synchro for this listener message.
   */
  public void receiveMapCommand( ContainerMessage mapContainer)   throws OpenMSPException {
    
  }

  /**
   * Event sent when the sync manager receives RESULT openML messages from the server
   * @param Result resultCommande : resultCommande containing the result of a command
   * @param long newSessionID : the session id use for this synchro for this listener message.
   * @param ContainerMessage : command sent that generate the status. 
   */
  public void receiveResultCommand(ContainerMessage resultContainer, ContainerMessage initialCommand)  throws OpenMSPException  {
    
  }

  /*
  
   * Return the name used to identiy the receiver of the syncML messages (source and target)
  
   */

  public String getSyncName() {
    return LogsSynchroListener.LOG_LISTENER_NAME;
  }

  /*
  
  * Fire this event once a synchro is starting
  
  */

  public void startSync(Credential cred, SynchroDescriptor synchrodescriptor) throws OpenMSPException {
    this.syncGoOn = true;
    this.setSynchroStatus(Status.STATUS_OK);
    JournalManager.getManager().deleteAllJournalEntryForService(this.getSyncName());
  }

  /*
  
  * Fire this event once the current synchro is ended
  
  */

  public void endSync() {
    String status  = Integer.toString(errorStatus);
    ServicePropertiesManager.getManager().saveProperty(this.getSyncName(), "synchroStatus", status);
   // log journal only if a synchro has been processed
   if (this.isSyncOK()) {
       JournalManager.getManager().saveJournalEntry(this.getSyncName(), JournalEntry.ERROR_SYNC, JournalManager.OK_MESSAGE, Status.STATUS_OK);
   }
  }

  /**
   * @return
   */
  public int getErrorStatus() {
    return errorStatus;
  }

}