/*
 *        OpenMobileIS - a free Java Framework for mobile applications
 *
 *   Copyright (C) 2002  Philippe Delrieu.
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Philippe Delrieu kept the rigth to distribute all code Copyrighted by philippe Delrieu
 *  under other licence term even commercial one.
 *  
 *  Modifications :
 *  2002 Creation P.Delrieu
 * 
 */

package org.openmobileis.modules.common.log.server;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;

import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.util.codec.GeneralCoder;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.protocol.AbstractCommand;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.DataItem;
import org.openmobileis.synchro.openmsp.protocol.Element;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener;
import org.openmobileis.synchro.openmsp.server.synctarget.SyncTargetAnswer;
import org.openmobileis.synchro.security.auth.Credential;

/**
 * Title:         Safe Server
 * Description:
 * Copyright:    Copyright (c) 2002
 * Company:      
 * @author P.Delrieu
 * @version 1.0
 */

public final class LogSyncService implements OpenMSPSynchroTargetListener {

  public LogSyncService() {
    // load mail connectors
  }

 public SyncTargetAnswer processCommand(Credential cred, ContainerMessage containerMessage) throws OpenMSPException {
    AbstractCommand syncCommand = (AbstractCommand) containerMessage.getElement();
    int commandType = syncCommand.getElementType();
    Status status = new Status(syncCommand.getCmdId(), Status.STATUS_WRONG_FORMAT);
   if (commandType == Element.SYNC) {
      // process incomming data
      while (containerMessage.hasMoreMessage()) {
        ContainerMessage commandContainer = containerMessage.nextMessage();
        Element command = commandContainer.getElement();
        if (command.getElementType() == Element.ADD) {
          this.processAddCommand(commandContainer);
          status = new Status(syncCommand.getCmdId(), Status.STATUS_OK);
        }
      }
    }
    SyncTargetAnswer answer = new SyncTargetAnswer();
    status.setCmdRef(syncCommand.getCmdId());
    ContainerMessage[] returnContainer = new ContainerMessage[1];
    returnContainer[0] = new ContainerMessage(status);
    answer.containerMessage = returnContainer;
    return answer;
  }

  private Status processAddCommand(ContainerMessage addContainer) {
    try {
      SessionContext context = SessionContextManager.getManager().getSessionContext();
      String rep = context.getUserId();
      String basePath = System.getProperty("user.dir");
      Status resStat = null;
      String filePath = basePath + File.separator + "pdalogs" + File.separator + rep + File.separator;
      while (addContainer.hasMoreMessage()) {
        DataItem item = (DataItem) addContainer.nextMessage().getElement();
        String fileName = item.getMetaInformation();
        byte[] fileData = item.getData().getBytes();
        fileData = GeneralCoder.decodeBase64(fileData);
        File newFile = new File(filePath);
        if (!newFile.exists()) {
          newFile.mkdirs();
        }
        OutputStream destFile = new BufferedOutputStream(new FileOutputStream(filePath + File.separator + fileName, true));
        try {
          destFile.write(fileData);
        } finally {
          destFile.flush();
          destFile.close();
        }
      }
      resStat = new Status(addContainer.getElement().getCmdId(), Status.STATUS_OK);
      return resStat;
    } catch (Throwable ex) {
      LogManager.traceError(0, ex);
      return new Status(addContainer.getElement().getCmdId(), Status.STATUS_FAILED);
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.server.synchro.safeserver.services.DefaultSafeServerService#getServiceName()
   */
  public String getTargetName() {
    return "FWKSyncLogsManager";
  }

}