/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2005 Creation P.Delrieu
 *  2005 Modified F.Salque
 * 
 */

package org.openmobileis.modules.crm.data.common;

import org.openmobileis.common.intl.IntlResourceManager;
import org.openmobileis.common.util.StringFormater;
import org.openmobileis.common.util.collection.Orderable;
import org.openmobileis.common.util.log.*;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: USe to manage the pair identifier/label used by article, visite type and contact type.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class DefaultLabel	implements Orderable, Label {

	static final long serialVersionUID = 5521257935120563452L;
	private String labelId;
	private String label = null;
	private String langue;
	private String category;
	private int order;

	protected DefaultLabel() {
		super();
	}

	public DefaultLabel(String id, String category, String label) {
		this.label = label;
		this.labelId = id;
		this.category = category;
		langue = IntlResourceManager.DEFAULT;
	}

	public DefaultLabel(String id, String category, String label, String language) {
		this.label = label;
		this.labelId = id;
		this.category = category;
		langue = language;
	}

	// < 0 if p1 < p2, 0 if equal, > 0 if p1 > p2
	public int compareTo(Object parm1, Object parm2) {
		if (!(parm1 instanceof DefaultLabel) || !(parm2 instanceof DefaultLabel)) {
			LogManager.traceError(
				LogServices.WEBSERVICE,
				"Attempt to compare wrong objects, not label");
			return 0;
		} else {
			return ((DefaultLabel) parm1).compareTo(((DefaultLabel) parm2));
		}
	}

	public String getLabel() {
		return label;
	}
		
	public String getLabelLanguage()	{
		return langue;
	}

	public String getCategory()	{
		return this.category;
	}

	public String getLabelId() {
		return labelId;
	}
	
	/**
	 * Use to sort label in a specific order. Sorting id made  :
	 * sort with labelOrder if equals use label.
	 * @param o label sort order
	 */
	public void setLabelSortOrder(int o)	{
		this.order = o;
	}
	public int getLabelSortOrder()	{
		return this.order;
	}

	public String toString() {
		return this.getLabelId()+"/"+label;
	}

	public boolean equals(Object obj) {
		if (!(obj instanceof DefaultLabel)) {
			throw new ClassCastException();
		} else {
			DefaultLabel newLab = ((DefaultLabel) obj);
			if ((newLab.labelId.equals(this.labelId)) &&(newLab.category.equals(this.category)))
				return true;
			return false;
		}
	}

	public int hashCode() {
		StringBuffer strbuff = new StringBuffer(this.category);
		strbuff.append('/').append(this.labelId);
		return strbuff.hashCode();
	}
	
	// < 0 if p1 < p2, 0 if equal, > 0 if p1 > p2
	public int compareTo(Object parm2) {
		if (!(parm2 instanceof DefaultLabel)) {
			LogManager.traceError(LogServices.WEBSERVICE, "DefaultLabel Attempt to compare wrong objects, not label");
			return 0;
		} else {
			DefaultLabel lab2 = (DefaultLabel)parm2;
			int comp = this.order - lab2.order;
			if (comp == 0)	{
				comp = StringFormater.removeAccent(this.getLabel()).toUpperCase().compareTo(StringFormater.removeAccent(lab2.getLabel()).toUpperCase());
			}
			return comp;
		}
	}

}
