/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.modules.crm.data.common;

import org.openmobileis.database.DatabaseException;
import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.collection.Sort;


/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class LabelManager {
	
	 private static LabelManager manager;
	 private LabelFactory factory;
	/**
	 * Constructor.
	 */
	public LabelManager(LabelFactory factory) {
		super();
		this.factory = factory;
	}
	
	public static void registerManager(LabelManager m)	{
		manager = m;
		ApplicationContextManager.getManager().addManager(manager);
	}
	
  public static LabelManager getManager() {
    return manager;
  }
  
	public static void removeManager()	{
		manager = null;
	}
	
	public LabelFactory getFactory() {
		return this.factory;
	}
  
  public Array getLabelListForCategorie(String categorie)	{
    return this.factory.getLabelListForCategorie(categorie);
  }
  
  public String[][] getCategoriesList()	{
  	return this.factory.getCategoriesList();
  }
 
  
 public Label getLabelWithIds(String id, String category)	{
    return this.factory.getLabelWithIds(id, category);
 }

 
   
  /**
   * call by synchro server to create the label DB
   */
  public void storeLabel(Label label) throws ServiceException, DatabaseException {
    this.factory.storeLabel(label);
  }
  
	public void deleteLabel(String id, String category) throws ServiceException, DatabaseException {
		this.factory.deleteLabel(id, category);
	}
	
	public Array sortLabelArray(Array labelList)	{
		if (labelList.size() > 0)	{
			Label label = (Label)labelList.get(0);
			Sort.shellsort(labelList, label);
		}
		return labelList;
	}

}
