/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2005 Modification F.Salque
 * 
 */
package org.openmobileis.modules.crm.database.common.fodb;

import org.odbms.Constraint;
import org.odbms.ObjectSet;
import org.odbms.Query;
import org.openmobileis.database.DatabaseException;
import org.openmobileis.database.fastobjectdb.FODBIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FODBStringIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;
import org.openmobileis.database.fastobjectdb.db.exception.FODBException;
import org.openmobileis.database.fastobjectdb.synchro.client.SynchroFastObjectDB;
import org.openmobileis.embedded.util.ServicePropertiesManager;
import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.modules.crm.data.common.Representant;
import org.openmobileis.modules.crm.data.common.RepresentantFactory;
import org.openmobileis.common.util.exception.BadDataFormatException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;

public abstract class FODBRepresentantFactory implements RepresentantFactory {
	
	private String[] repIds;
  
	
	/**
	 * Constructor for FODBRepresentantFactory.
	 */
	public FODBRepresentantFactory() {
		super();				
		String dbPath = PropertiesManager.getManager().getProperty("fastobjectdb.database.path");
		if (dbPath == null)	{
			LogManager.traceError(0, "Representant Factory FODB ERROR : FODB path property is not set. Can't create db");
		}
		try	{
			FastObjectDB db = FastObjectDBManager.getCurrentFODB();
			if (!db.isCollectionExist(this.getCollectionName()))	{
				db.createCollection(this.getCollectionName(), this.getDataType());
				FODBStringIndexDescriptor IDDescriptor = new FODBStringIndexDescriptor("ID", FODBIndexDescriptor.UNIQUE, "getRepresentantId()", 12, this.getMaxRepresentantIdLength());
				db.addIndex(this.getCollectionName(), IDDescriptor);
				
				this.initDB(db);
			}
			//always register notification listener because it's not persistent.
			((SynchroFastObjectDB)db).registerSynchroFODBReturnListener(this.getCollectionName(), this.getSynchroListener());
      //register the log renderer.
      ((SynchroFastObjectDB)db).registerJournalLogRenderer(this.getCollectionName(), "Utilisateurs :");
			this.updateSessionId();

		} catch (Throwable ex)	{
			LogManager.traceError(0, "Representant Factory FODB ERROR : Unknown error during creation");
			LogManager.traceError(0, ex);
		}
	}
  
	private void updateSessionId()	{
		SessionContext context = (SessionContext) SessionContextManager.getManager().getSessionContext();
		Representant rep =this.getInstallRepresentant();
		if (rep != null)	{
			context.setAttribute("userId",rep.getRepresentantId());
		}
	}
	
	public String[] getRepIdsList()	{
		try	{
			Representant[] repList = this.getAllRepresentants();
			String[] repIds = new String[repList.length];
			for (int i=0; i<repList.length; i++)	{
				repIds[i] = repList[i].getRepresentantId();
			}
			return repIds;
		} catch (Throwable ex)	{
			LogManager.traceError(0, ex);
		}
		return new String[0];
	}
  
  public Representant getRepresentant(String id)	{
		try {
			Query q = FastObjectDBManager.getCurrentFODB().query();
			q.constrain(this.getDataType());
			Query subq = q.descend("getRepresentantId()");
			Constraint c = subq.constrain(id).equal();
			ObjectSet set = q.execute(); 
			if (set.hasNext())	{
				return (Representant)set.next();
			} else	{
				return null;
			}
		} catch (Exception e) {
			LogManager.traceError(LogServices.DATABASESERVICE, e);
		} 
		return null;
  }
  
  public Representant getSelectedRepresentant()	{
		String repId = ServicePropertiesManager.getManager().getProperty("FWKRepresentant","selectedrepid");
		if (repId != null)	{
			Representant rep = this.getRepresentant(repId);
			return rep;
		}
		return null;
  }
  
	public void setSelectedRepresentant(String id) throws ServiceException, DatabaseException {
		ServicePropertiesManager.getManager().saveProperty("FWKRepresentant","selectedrepid", id);
	}
  
  public Representant getInstallRepresentant()	{
		String repId = ServicePropertiesManager.getManager().getProperty("FWKRepresentant","installrepid");
		if (repId != null)	{
			Representant rep = this.getRepresentant(repId);
			return rep;
		}
		return null;
	}
  
	public void setInstallRepresentant(String id) throws ServiceException, DatabaseException {
		ServicePropertiesManager.getManager().saveProperty("FWKRepresentant","installrepid", id);
	}
    
  public Representant[] getAllRepresentants() throws DatabaseException	{
		try {
			Query q = FastObjectDBManager.getCurrentFODB().query();
			q.constrain(this.getDataType());
			Query subq = q.descend("getRepresentantId()");
			ObjectSet set = q.execute(); 
			Representant[] repList = new Representant[set.size()];
			int i=0;
			while (set.hasNext())	{
				repList[i++] = (Representant) set.next();
			}
			return repList;
		} catch (Exception e) {
			LogManager.traceError(LogServices.DATABASESERVICE, e);
		} 
		return new Representant[0];
  }
   
  /**
   * call by synchro server to create the representant DB
   */
  public void addRepresentant(Representant rep) throws ServiceException, DatabaseException {
		Representant dbrep = this.getRepresentant(rep.getRepresentantId());
		try	{
			if (dbrep == null)	{
				FastObjectDBManager.getCurrentFODB().add(this.getCollectionName(), rep);
			} else	{
				FastObjectDBManager.getCurrentFODB().replace(this.getCollectionName(), rep);
			}
		} catch (Throwable ex)	{
			throw new ServiceException(ex);
		}
  }
  
  public void removeRepresentant(String repId) throws ServiceException, DatabaseException {
		try	{
//			FastObjectDBManager.getCurrentFODB().delete(this.getCollectionName(), repId);
			FastObjectDBManager.getCurrentFODB().deleteWithId(this.getCollectionName(), repId);
		} catch (Throwable ex)	{
			throw new ServiceException(ex);
		}
  }
  
	protected RepresentantSynchroFODBReturnListener getSynchroListener() {
		return new RepresentantSynchroFODBReturnListener();
	}
	
	public String getCollectionName()	{
		return "Representant";
	}
	
	protected abstract int getMaxRepresentantIdLength();
	 
	protected abstract Class getDataType();
		
	protected abstract void initDB(FastObjectDB db) throws FODBException, BadDataFormatException;
}
