/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2005 Creation F.Salque
 * 
 */
package org.openmobileis.modules.crm.database.common.jdbc;

import java.sql.ResultSet;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.DatabaseException;
import org.openmobileis.modules.crm.data.common.Label;
import org.openmobileis.modules.crm.data.common.LabelFactory;

/**
 * 
 * JDBC Factory for internationalized labels (labels with a language) 
 */
public abstract class JDBCIntlLabelFactory implements LabelFactory {
	protected LabelJDBCQuery query;
	/**
	 * 
	 */
	public JDBCIntlLabelFactory(LabelJDBCQuery q) {
		super();
		query = q;
		this.initLabelDB();
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.modules.crm.data.common.LabelFactory#getLabelListForCategorie(java.lang.String)
	 */
	public Array getLabelListForCategorie(String categorie) {
		Array labelList = new Array();
		try	{
			String language=this.getLanguage();
			String[] param = new String[]{categorie,language};
			ResultSet result = query.getLabelListForCategorie(param);
			while (result.next()) {
				labelList.add(query.convertResultSetToLabel(result));
			}
			result.close();
		} catch (Exception ex)	{
			LogManager.traceError(0, ex);
		} finally {
			query.getDbManager().garbageOpenedConnection();
		}
		return labelList;
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.modules.crm.data.common.LabelFactory#getCategoriesList()
	 */
	public String[][] getCategoriesList() {
		//cached label list is not used anymore
		return null;
	}

	public Label getLabelWithIds(String id,String category,String lang){
		Label label = null;
		try	{
			String[] param = new String[]{id, category, lang};
			ResultSet result = query.getLabelWithIds(param);
			if (result.next()) {
				label = query.convertResultSetToLabel(result);
			}
			result.close();
		} catch (Exception ex)	{
			LogManager.traceError(0, ex);
		} finally {
			query.getDbManager().garbageOpenedConnection();
		}
		return label;
	}
	/* (non-Javadoc)
	 * @see org.openmobileis.modules.crm.data.common.LabelFactory#getLabelWithIds(java.lang.String, java.lang.String)
	 */
	public Label getLabelWithIds(String id, String category) {
		String lang=this.getLanguage();
		return this.getLabelWithIds(id,category,lang);
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.modules.crm.data.common.LabelFactory#storeLabel(org.openmobileis.modules.crm.data.common.Label)
	 */
	public void storeLabel(Label label) throws ServiceException,
			DatabaseException {
		try	{		  
			//test because of trigger PB could not delete that does not exist in db
			Label dblabel = this.getLabelWithIds(label.getLabelId(), label.getCategory(),label.getLabelLanguage());
			if (dblabel != null)	{		  
				String[] param = query.getUpdateParamFromLabel(label);
				query.updateLabel(param);
			} else	{
				String[] param = query.getInsertParamFromLabel(label);
				query.insertLabel(param);
			}			
			this.notifyLabelUpdate(label);
		} catch (Throwable ex)	{
			throw new ServiceException(ex);
		} finally {
			query.getDbManager().garbageOpenedConnection();
		}
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.modules.crm.data.common.LabelFactory#deleteLabel(java.lang.String, java.lang.String)
	 */
	public void deleteLabel(String id, String category) throws ServiceException, DatabaseException {
		this.deleteLabel(id,category,this.getLanguage());
	}
	
	public void deleteLabel(String id, String category,String language) throws ServiceException, DatabaseException {
		try	{		  
			Label dblabel = this.getLabelWithIds(id, category,language);
			if (dblabel != null)	{		  
				String[] param = {id, category,language};
				query.deleteLabel(param);
				this.notifyLabeldelete(id, category,language);
			}
		} catch (Throwable ex)	{
			throw new ServiceException(ex);
		} finally {
			query.getDbManager().garbageOpenedConnection();
		}
	}
	public String convertIdsToLabelObjectIds(String labelid, String category)	{
		StringBuffer str = new StringBuffer(labelid);
		str.append('%');
		str.append(category);
		String language=this.getLanguage();
		if(language!=null){
			str.append('%');
			str.append(language);
		}
		return str.toString();
	}
	
	public abstract void initLabelDB();
	public abstract String getLanguage();
	
	/**
	 * Get all labels for all languages for the category.
	 * @param category
	 * @return
	 */
	public abstract Array getLabelListForCategorieAdmin(String category);

	public abstract void notifyLabelUpdate(Label label) throws SynchroException;
	public abstract void notifyLabeldelete(String labelId, String categoryId,String language) throws SynchroException;
}
