/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2005 Modification F.Salque
 * 
 */

package org.openmobileis.modules.crm.database.common.jdbc;

import java.sql.ResultSet;
import java.util.HashMap;

import org.openmobileis.database.DatabaseException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;
import org.openmobileis.modules.crm.data.common.Label;
import org.openmobileis.modules.crm.data.common.LabelFactory;
import org.openmobileis.modules.crm.data.common.LabelManager;
import org.openmobileis.common.util.collection.Array;


/**
 * Title:        OpenMobileIS Project
 * Description:  
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public abstract class JDBCLabelFactory implements LabelFactory {
	protected LabelJDBCQuery query;
	private HashMap categoriesMap;
	/**
	 * Constructor for LabelManager.
	 */
	public JDBCLabelFactory(LabelJDBCQuery q) {
		super();
		query = q;
		this.initLabelDB();
		this.loadLabelList();
	}
  
	private void loadLabelList()	{
		try	{
			categoriesMap = new HashMap(10);
			String[][] catList = this.getCategoriesList();
			Label l = null;
			for (int i=0; i<catList.length; i++)	{
				HashMap labelMap = new HashMap(20);
				categoriesMap.put(catList[i][1], labelMap);
				Array labelList = this.getDBLabelListForCategorie(catList[i][1]);
				for (int j=0; j<labelList.size(); j++)	{
					l = (Label)labelList.get(j);
					labelMap.put(l.getLabelId(), l);
				}
			}
		} catch (Exception ex)	{
			LogManager.traceError(0, ex);
		}
	}
	
	public Array getLabelListForCategorie(String categorie)	{
		if (categoriesMap == null) this.loadLabelList();
		Array labelList = new Array();	
		HashMap map = (HashMap) categoriesMap.get(categorie);
		if (map!= null)	{
			labelList = new Array(map.values().toArray());
		}
		LabelManager.getManager().sortLabelArray(labelList);
		return labelList;
	}
  
	protected Array getDBLabelListForCategorie(String categorie)	{
		Array labelList = new Array();
		try	{
			String[] param = new String[]{categorie};
			ResultSet result = query.getLabelListForCategorie(param);
			while (result.next()) {
			    Label label = query.convertResultSetToLabel(result);
			    if (label != null) {
			        labelList.add(label);
			    } else {
			        LogManager.traceWarning(LogServices.DATABASESERVICE, "JDBCLabelFactory : getDBLabelListForCategorie : Null Label");
			    }
			}
			result.close();
		} catch (Exception ex)	{
			LogManager.traceError(0, ex);
		} finally {
			query.getDbManager().garbageOpenedConnection();
		}
		return labelList;
	} 
	
	public Label getLabelWithIds(String id, String category)	{
		if (categoriesMap == null) this.loadLabelList();
		Label label = null;
		HashMap map = (HashMap) categoriesMap.get(category);
		if (map!= null)	{
			label = (Label)map.get(id);
		}
		return label;
	}
  
 protected Label getDBLabelWithIds(String id, String category)	{
	Label label = null;
	try	{
		String[] param = new String[]{id, category};
		ResultSet result = query.getLabelWithIds(param);
		if (result.next()) {
			label = query.convertResultSetToLabel(result);
		}
		result.close();
	} catch (Exception ex)	{
		LogManager.traceError(0, ex);
	} finally {
		query.getDbManager().garbageOpenedConnection();
	}
	return label;
 }
 
 public String convertIdsToLabelObjectIds(String labelid, String category)	{
		StringBuffer str = new StringBuffer(labelid);
		str.append('%');
		str.append(category);
		return str.toString();
 }
 
   
  /**
   * call by synchro server to create the label DB
   */
  public void storeLabel(Label label) throws ServiceException, DatabaseException {
		try	{		  
			//test because of trigger PB could not delete that does not exist in db
			Label dblabel = this.getLabelWithIds(label.getLabelId(), label.getCategory());
			if (dblabel != null)	{		  
				String[] param = query.getUpdateParamFromLabel(label);
				query.updateLabel(param);
			} else	{
				String[] param = query.getInsertParamFromLabel(label);
				query.insertLabel(param);
			}			
			this.notifyLabelUpdate(label);
			//reload lab cache
			this.loadLabelList();
		} catch (Throwable ex)	{
			throw new ServiceException(ex);
		} finally {
			query.getDbManager().garbageOpenedConnection();
		}
  }

	public void deleteLabel(String id, String category) throws ServiceException, DatabaseException {
		try	{		  
			//test because of trigger PB could not delete that does not exist in db
			Label dblabel = this.getLabelWithIds(id, category);
			if (dblabel != null)	{		  
				//select tache if exist
				String[] param = {id, category};
				query.deleteLabel(param);
				//update never delete because after each modification for complet synchro
				//reload lab cache
				this.notifyLabeldelete(id, category);
				this.loadLabelList();
			}
		} catch (Throwable ex)	{
			throw new ServiceException(ex);
		} finally {
			query.getDbManager().garbageOpenedConnection();
		}
	}
	
	public abstract String[][] getCategoriesList();
	public abstract void initLabelDB();

	public abstract void notifyLabelUpdate(Label label) throws SynchroException;
	public abstract void notifyLabeldelete(String labelId, String categoryId) throws SynchroException;
}
