/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2005 Modification F.Salque
 * 
 */

package org.openmobileis.modules.crm.database.common.jdbc;

import java.sql.ResultSet;

import org.openmobileis.database.DatabaseException;
import org.openmobileis.modules.crm.data.common.Representant;
import org.openmobileis.modules.crm.data.common.RepresentantFactory;
import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.log.LogManager;

/**
 * Title:        OpenMobileIS Project
 * Description:   
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public abstract class JDBCRepresentantFactory implements RepresentantFactory {
  
	protected RepresentantJDBCQuery query;
	/**
	 * Constructor for RepresentantManager.
	 */
	public JDBCRepresentantFactory(RepresentantJDBCQuery q) {
		super();				
		this.query = q;
	}

	public String[] getRepIdsList()	{
		Array idList = new Array();
		try	{
			ResultSet result = query.getRepIdsList();
			while (result.next()) {
				idList.add(result.getString(1));
			}
			result.close();
		} catch (Exception ex)	{
			LogManager.traceError(0, ex);
		} finally {
			query.getDbManager().garbageOpenedConnection();
		}
		String[] ids = new String[idList.size()];
		idList.toArray(ids);
		return ids;
	}
  
  public Representant getRepresentant(String id)	{
		Representant rep = null;
		try	{
			String[] param = new String[]{id};
			ResultSet result = query.getRepresentant(param);
			if (result.next()) {
				rep = query.convertResultSetToRepresentant(result);
			}
			result.close();
		} catch (Exception ex)	{
			LogManager.traceError(0, ex);
		} finally {
			query.getDbManager().garbageOpenedConnection();
		}
		return rep;
  }
  
  public Representant getSelectedRepresentant()	{
		SessionContext context = SessionContextManager.getManager().getSessionContext();
		String repId = (String)context.getAttribute("userId");
    return this.getRepresentant(repId);
  }
  
	public void setSelectedRepresentant(String id) throws ServiceException, DatabaseException {
		//do nothing no way to change
	}
  
  public Representant getInstallRepresentant()	{
		return this.getSelectedRepresentant();
  }
  
	public void setInstallRepresentant(String id) throws ServiceException, DatabaseException {
		//do nothing no way to change
	}
    
  public Representant[] getAllRepresentants() throws DatabaseException	{
		Array idList = new Array();
		try	{
			ResultSet result = query.getAllRepresentants();
			while (result.next()) {
				idList.add(query.convertResultSetToRepresentant(result));
			}
			result.close();
		} catch (Exception ex)	{
			LogManager.traceError(0, ex);
		} finally {
			query.getDbManager().garbageOpenedConnection();
		}
		Representant[] ids = new Representant[idList.size()];
		idList.toArray(ids);
		return ids;
  }
   
  /**
   * call by synchro server to create the representant DB
   */
  public void addRepresentant(Representant rep) throws DatabaseException {
		try	{		  
			//test because of trigger PB could not delete that does not exist in db
			Representant dbrep = this.getRepresentant(rep.getRepresentantId());
			if (dbrep != null)	{		  
				String[] param = query.getUpdateParamFromRepresentant(rep);
				query.updateRepresentant(param);
			} else	{
				String[] param = query.getInsertParamFromRepresentant(rep);
				query.insertRepresentant(param);
			}			
			this.notifyRepresentantUpdate(rep);
		} catch (Throwable ex)	{
			throw new DatabaseException(ex);
		} finally {
			query.getDbManager().garbageOpenedConnection();
		}
  }
  
  public void removeRepresentant(String repId) throws DatabaseException {
		try	{		  
			//test because of trigger PB could not delete that does not exist in db
			Representant dbrep = this.getRepresentant(repId);
				if (dbrep != null)	{		  
				//select tache if exist
				String[] param = {repId};
				query.deleteRepresentant(param);
				//update never delete because after each modification for complet synchro
				this.notifyRepresentantdelete(repId);
			}
		} catch (Throwable ex)	{
			throw new DatabaseException(ex);
		} finally {
			query.getDbManager().garbageOpenedConnection();
		}
  }

	public abstract void notifyRepresentantUpdate(Representant rep) throws SynchroException;
	public abstract void notifyRepresentantdelete(String repId) throws SynchroException;
  
}
