package org.openmobileis.modules.profils.embedded.data;

import java.io.File;
import java.io.FileReader;

import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServerException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;

public class ProfilManager {

	private static ProfilManager manager;
	private String profilFile;
	private Array profilRubrics = null;
	private Array profilModules = null;
	  
	private ProfilManager() {
		profilFile = PropertiesManager.getManager().getProperty("org.openmobileis.profil.file");
		// TODO : get it a nicer way :
		profilFile = System.getProperty("user.dir")+File.separator+"WEB-INF"+File.separator+"conf"+File.separator+"properties"+File.separator+"profilData";
		System.out.println("profilFile : "+profilFile);
	}
	
	public static ProfilManager getManager() {
		if (manager == null)  {
			synchronized(org.openmobileis.modules.profils.embedded.data.ProfilManager.class) {
				if (manager == null)  {
					manager = new ProfilManager();
				}
			}
		}
		return manager;
	}
	
	public Array getProfilRubrics() {
		if(profilRubrics != null) {
			return profilRubrics;
		}
		this.parseFile();

		return profilRubrics;
	}
	
	public Array getProfilModules() {
		if(profilModules != null) {
			return profilModules;
		}
		this.parseFile();

		return profilModules;
	}
	
	private void parseFile () {
		try {
			FileReader reader = new FileReader(this.profilFile);
			ProfilParser parser = new ProfilParser(reader);
			parser.parse();
			
			this.profilRubrics = parser.getRubrics();
			this.profilModules = parser.getModules();
		} catch (java.io.FileNotFoundException e) {
			LogManager.traceError(LogServices.WEBSERVICE, "Can not read profil file : " + this.profilFile);
		} catch (java.io.IOException e) {
			LogManager.traceError(LogServices.WEBSERVICE, "Can not read profil file : " + this.profilFile);
		} catch (Exception e) {
			LogManager.trace(new ServerException("Error during parsing XML data for profils ",e));
			return;
		}
	}
	
	public String toString() {
		if (profilModules == null || profilRubrics == null) {
			this.parseFile();
		}
		
		StringBuffer strBuf = new StringBuffer();
		for (int i = 0; i < profilRubrics.size(); i++) {
			ProfilRubric rubric = (ProfilRubric)profilRubrics.get(i);
			strBuf.append("Rubric : "+rubric.getName()+"\n");
			for (int j = 0; j < rubric.getAllPropertiesNames().length; j++) {
				strBuf.append("  Property : "+rubric.getAllPropertiesNames()[j]+" ==> "+rubric.getProperty(rubric.getAllPropertiesNames()[j])+"\n");
			}
		}
		
		for (int i = 0; i < profilModules.size(); i++) {
			ProfilModule module = (ProfilModule)profilModules.get(i);
			strBuf.append("Module : "+module.getName()+" version : "+module.getVersion()+"\n");
			Array rubrics = module.getRubrics();
			for (int j = 0; j < rubrics.size(); j++) {
				strBuf.append("  Rubric : "+(String)rubrics.get(j)+"\n");
			}
			for (int j = 0; j < module.getAllPropertiesNames().length; j++) {
				strBuf.append("  Property : "+module.getAllPropertiesNames()[j]+" ==> "+module.getProperty(module.getAllPropertiesNames()[j])+"\n");
			}
		}
		
		return strBuf.toString();
	}
}
