/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */
package org.openmobileis.services;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.services.common.ServiceManager;

import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Base classe to develop an edit service
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public abstract class SimpleEditService extends TemplateService {
		
	public String runTemplate (HttpServletRequest req, HttpServletResponse res , TemplateModelRoot templateData) throws ServletException, IOException {
		try	{
			Object sessionDatas = this.getSessionData(req);
			boolean createmode = false;
			if (sessionDatas == null)	{ //first call for create.
				sessionDatas = this.createSessionObject(req);
				if (sessionDatas == null)	{
					this.addParametersErrorToTemplate(req, templateData);
					return this.getTemplateName();
				}
				HttpSession session = req.getSession(true);
				session.setAttribute(this.getSessionDatasName(), sessionDatas);				
				createmode = true;
			}
			
			this.fillSessionObjectWithRequestData(sessionDatas, req);
			
			if ((!createmode) &&(!this.hasParameterError(req)) && (this.isStoreMode(req)))	{
				this.storeSessionObjectInDB(sessionDatas, req, templateData);
				if (!this.hasParameterError(req))	{
					HttpSession session = req.getSession(true);
					session.removeAttribute(this.getSessionDatasName());
					session.removeAttribute(this.getErrorSessionAttributName());
          ServiceManager.getManager().redirectToServiceURI(this.getDisplayServiceURI(), req, res);
					return null;
				}
			}
			
			this.fillTemplateWithSessionData(sessionDatas, templateData);
						
			this.addParametersErrorToTemplate(req, templateData);
			return this.getTemplateName();
		} catch (Exception ex) {
			HttpSession session = req.getSession(true);
			session.removeAttribute(this.getSessionDatasName());
			session.removeAttribute(this.getErrorSessionAttributName());
			throw new ServletException(ex);
		}
	}
		
	protected void setInputRequestParameterError(HttpServletRequest req, String message) {
		HttpSession session = req.getSession(true);
		StringBuffer errorMessage = (StringBuffer)session.getAttribute(this.getErrorSessionAttributName());
		if (errorMessage == null)	{
			errorMessage = new StringBuffer();
		} else	{
			errorMessage.append("<BR>");
		}
		errorMessage.append(message);
		session.setAttribute(this.getErrorSessionAttributName(),errorMessage);
	}
	
	protected String getErrorSessionAttributName()	{
		return this.getSessionDatasName()+"-errorMessage";
	}
	
	protected void deleteSessionData(HttpServletRequest req)	{
		HttpSession session = req.getSession(true);
		session.removeAttribute(this.getSessionDatasName());
		session.removeAttribute(this.getErrorSessionAttributName());
	}
	
	private Object getSessionData(HttpServletRequest req)	{
		HttpSession session = req.getSession(true);
		return session.getAttribute(this.getSessionDatasName());
	}
	
	private boolean hasParameterError(HttpServletRequest req)	{
		HttpSession session = req.getSession(true);
		return (session.getAttribute(this.getErrorSessionAttributName()) != null);
	}
	
	private void addParametersErrorToTemplate(HttpServletRequest req, TemplateModelRoot templateData)	{
		HttpSession session = req.getSession(true);
		if (this.hasParameterError(req))	{
			StringBuffer error = (StringBuffer)session.getAttribute(this.getErrorSessionAttributName());
			templateData.put("hasErrorMessage", new SimpleScalar(true));
			templateData.put("errorMessage", new SimpleScalar(error.toString()));
			//remove for next call		
		}
		session.removeAttribute(this.getErrorSessionAttributName());
	}
	
	protected abstract void fillTemplateWithSessionData(Object sessionDatas, TemplateModelRoot templateData) throws ServiceException;	
	protected abstract void storeSessionObjectInDB(Object sessionDatas, HttpServletRequest req, TemplateModelRoot templateData) throws ServiceException;	
	protected abstract void fillSessionObjectWithRequestData(Object sessionDatas, HttpServletRequest req) throws ServiceException;	
	protected abstract String getSessionDatasName();
	protected abstract String getTemplateName();
	protected abstract Object createSessionObject(HttpServletRequest req) throws ServiceException;
	protected abstract String getDisplayServiceURI();
	
	
	/**
	 * return true if this request is use to store object in db (default mode).
	 * To force form edition return false. The storeSessionObjectInDB is not call.
	 *
	 * @return true to store object false to force edit from edition.
	 */
	protected boolean isStoreMode(HttpServletRequest req)	{
		return true;
	}

}
