/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */

package org.openmobileis.services;

import freemarker.template.*;
import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmobileis.common.util.log.*;
import org.openmobileis.embedded.webserver.templates.TemplateManager;
import org.openmobileis.embedded.webserver.templates.TemplateNotFoundException;
import org.openmobileis.services.common.ServletTools;
import org.openmobileis.services.navigation.NavigationBarManager;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Class that all service that use template must implements. Provide default template
 * management that is compatible with the template synchro process.
 * Use the IntlResourceManager to provide template with the rigth language.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public abstract class TemplateService extends Service {

  public TemplateService() {
    TemplateManager.getManager();
  }

  /* Process the service
 * @param req the servlet request
 * @param req the servlet response
 *  @exception ServletException when an exception has occurred
 */
  public void run(HttpServletRequest req, HttpServletResponse res ) throws ServletException, IOException {
	//	LogManager.traceNotice(0, "TemplateService :Begin run");
    try {
      TemplateModelRoot modelRoot = TemplateManager.getManager().getTemplateModelRoot();
      String templateName = this.runTemplate(req, res, modelRoot);
      
			//manage navigation bar.
			if (this.isNavigationService())	{
				//manage return
				String navBar = NavigationBarManager.getManager().getNavigationBar();
				modelRoot.put("navbarNOSY", new SimpleScalar(navBar));	
			}
      
      if (templateName != null) {
        try {
          TemplateManager.getManager().sendResponse(templateName, modelRoot, res);
        } catch (TemplateNotFoundException ex)  {
          try {
            org.openmobileis.common.intl.IntlResourceManager resourceManager = org.openmobileis.common.intl.IntlResourceManager.getManager();
            String title = resourceManager.getLocalizedProperty("TemplateService.ExecError");
            res.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, title);
          } catch (Exception exi)  {
          }
          LogManager.traceAlert(LogServices.WEBSERVICE, "Error Template not found name :"+templateName);
          LogManager.traceAlert(LogServices.WEBSERVICE, ex);
        } catch (Exception ex) {
          try {
            org.openmobileis.common.intl.IntlResourceManager resourceManager = org.openmobileis.common.intl.IntlResourceManager.getManager();
            String title = resourceManager.getLocalizedProperty("TemplateService.ExecError");
            res.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, title);
          } catch (Exception exi)  {
          }
          LogManager.traceAlert(LogServices.WEBSERVICE, "Error during execution of Service"+this.getName());
          LogManager.traceAlert(LogServices.WEBSERVICE, ex);
        }
      }
    } catch (Exception ex) {
			LogManager.traceAlert(LogServices.WEBSERVICE, "Error during execution of Template Service"+this.getName());
			LogManager.traceAlert(LogServices.WEBSERVICE, ex);
      org.openmobileis.common.intl.IntlResourceManager resourceManager = org.openmobileis.common.intl.IntlResourceManager.getManager();
      String message = resourceManager.getLocalizedProperty("TemplateService.UnknownError");
      String title = resourceManager.getLocalizedProperty("TemplateService.Title");
      ServletTools.sendErrorPage(title, message, "/index", res);
    }
  }

  /**
   * Extends this method to develop specific process.
   * Return the template relatif path to be use to process TemplateModel.
   */
  public abstract String runTemplate ( HttpServletRequest req, HttpServletResponse res , TemplateModelRoot templateData) throws ServletException, IOException ;

}
