/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.services.cache;

import java.util.Hashtable;

import javax.servlet.http.HttpServletRequest;

import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.services.Service;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Main manage to register a service that has its page cached.
 * The use of web page cache for service that use complet processis to generate an answer
 * with data that doen't change often. When a request arrive for this service, if it has already
 * been cached, the cached page is return with no service processing.
 * To register a service, you must use a WebPageCacheServiceStruct. A default implementation is provided.
 * This structure define cache properties and store cached page.
 * 
 * When registering a service register for a spcific event that will signal when the cache has to be cleared.
 * A service can register the same WebPageCacheServiceStruct for several event.
 * When an event is send, all cached page of all service registered for this event is validated with the WebPageCacheServiceStruct implementation.
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class WebPageCacheManager {
	
	private static WebPageCacheManager manager;
	private Hashtable eventList;
	private Array serviceList;
	/**
	 * Constructor for WebPageCacheManager.
	 */
	private WebPageCacheManager() {
		super();
		eventList = new Hashtable(5);
		serviceList = new Array(10);
	}
	
	public static WebPageCacheManager getManager()	{
		if (manager == null)	{
			synchronized(WebPageCacheManager.class)	{
				if (manager == null)	{
					manager = new WebPageCacheManager();
					ApplicationContextManager.getManager().addManager(manager);
				}
			}
		}
		return manager;
	}
	
	public void registerService(WebPageCacheServiceStruct service, WebPageCacheEvent event)	{
		Array services = (Array)eventList.get(event);
		if (services == null)	{
			services = new Array(10);
			eventList.put(event, services);
		}
		services.add(service);	
		serviceList.add(service);	
	}
	
	public byte[] getCachedPage(HttpServletRequest req, Service service)	{
		byte[] resp = null;
		for(int i=0; i<serviceList.size();i++)	{
			WebPageCacheServiceStruct serviceStruct = (WebPageCacheServiceStruct)serviceList.get(i);
			if (serviceStruct.getService().equals(service))	{
			    resp = serviceStruct.getResponseForRequest(new Hashtable(req.getParameterMap()));
				// resp = serviceStruct.getResponseForRequest(((WebServerConnection)req).getParametersTable());
			}
		}
		return resp;
	}
	
	public void setCachedPage(HttpServletRequest req, byte[] cachedData, Service service)	{
		for(int i=0; i<serviceList.size();i++)	{
			WebPageCacheServiceStruct serviceStruct = (WebPageCacheServiceStruct)serviceList.get(i);
			if (serviceStruct.getService().equals(service))	{
			    serviceStruct.addWebPage(new Hashtable(req.getParameterMap()), cachedData);
				// serviceStruct.addWebPage((Hashtable)((WebServerConnection)req).getParametersTable().clone(), cachedData);
			}
		}
	}
	
	public void sendCacheEvent(WebPageCacheEvent event)	{
		Array eventArray = (Array)eventList.get(event);
		if (eventArray != null)	{
			for (int i=0; i<eventArray.size(); i++)	{
				WebPageCacheServiceStruct struct = (WebPageCacheServiceStruct)eventArray.get(i);
				struct.notifyCacheEvent(event);
			}
		}
	}

}
