/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.services.common;

import java.util.Calendar;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class DateServiceUtil {


  /**
  * Return a simple hash containing 5 lists: dayList, monthList, yearList, hourList and minuteList set with the date giving by parameters.
  * Use these HTML/ template to show the result for day month year.
  *      <code>
         <SELECT NAME="day">
            <list startDate.dayList as day>
              <OPTION VALUE="${day.value}" <if day.select>selected</if>>${day.value}</OPTION>
            </list>
          </SELECT>
          <SELECT NAME="month">

            <list startDate.monthList as month>
              <OPTION VALUE="${month.value}" <if month.select>selected</if>>${month.value}</OPTION>
            </list>
          </SELECT>
          <SELECT NAME="year">

            <list startDate.yearList as year>
              <OPTION VALUE="${year.value}" <if year.select>selected</if>>${year.value}</OPTION>
            </list>
          </SELECT>    
          <SELECT NAME="hour">
            <list startDate.hourList as hour>
              <OPTION VALUE="${hour.value}" <if hour.select>selected</if>>${hour.value}</OPTION>
            </list>
          </SELECT>    
          <SELECT NAME="minute">
            <list startDate.minutesList as minute>
              <OPTION VALUE="${minute.value}" <if minute.select>selected</if>>${minute.value}</OPTION>
            </list>
          </SELECT>    
          </code>  
To show current date :
    Calendar calendar = java.util.Calendar.getInstance();
    SimpleHash startDate =
      Utils.getUtilsManager().getForTemplateInputDate(
        calendar.get(java.util.Calendar.DAY_OF_MONTH),
        calendar.get(java.util.Calendar.MONTH),
        calendar.get(java.util.Calendar.YEAR),
        calendar.get(java.util.Calendar.HOUR_OF_DAY),
        0);
    templateData.put("startDate", startDate);


  */
  public static SimpleHash getForTemplateInputDate(
    int selectedDay,
    int selectedMonth,
    int selectedYear,
    int selectedHours,
    int selectedMinutes,
    int selectedDuration) {

    SimpleHash inputDate = new SimpleHash();
    int cpt;
    SimpleList list;
    SimpleHash element;
    // fill day
    list = new SimpleList();
    for (cpt = 1; cpt <= 31; cpt++) {
      element = new SimpleHash();
      element.put("select", selectedDay == cpt);
      element.put("value", formatDateDec(cpt));
      list.add(element);
    }
    inputDate.put("dayList", list);
    // fill month
    list = new SimpleList();
    for (cpt = 1; cpt <= 12; cpt++) {
      element = new SimpleHash();
      element.put("select", selectedMonth + 1 == cpt);
      element.put("value", formatDateDec(cpt));
      list.add(element);
    }
    inputDate.put("monthList", list);

    list = new SimpleList();
    Calendar cal = Calendar.getInstance();
    int currentYear = cal.get(Calendar.YEAR);
    for (cpt = currentYear - 1; cpt <= currentYear + 5; cpt++) {
      element = new SimpleHash();
      element.put("value", String.valueOf(cpt));
      element.put("select", selectedYear == cpt);
      list.add(element);
    }
    inputDate.put("yearList", list);
    // fill hours
    list = new SimpleList();
    for (cpt = 0; cpt <= 23; cpt++) {
      element = new SimpleHash();
      element.put("value", formatDateDec(cpt));
      element.put("select", selectedHours == cpt);
      list.add(element);
    }
    inputDate.put("hourList", list);
    // fill minutes
    list = new SimpleList();
    for (cpt = 0; cpt <= 59; cpt += 5) {
      element = new SimpleHash();
      element.put("value", formatDateDec(cpt));
      element.put("select", selectedMinutes == cpt);
      list.add(element);
    }
    inputDate.put("minutesList", list);

    // return the input date
    return inputDate;
  }

  private static String formatDateDec(int value) {
    if (value < 10) {
      return "0" + value;
    } else {
      return String.valueOf(value);
    }
  }

}
