/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */

package org.openmobileis.services.servlet;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.Principal;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Map;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletInputStream;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.openmobileis.common.context.SessionContextManager;

/**
 *
 * This class wraps HttpServletRequest.
 * It gives methods to modify the parameters list. It makes the request more flexible.
 * 
 * Its constructor takes a HttpServletRequest, which is the object to be wrapped.
 * 
 * The added methods are the following :
 *    public void addParameter(String key, Object value);
 *    public void addParameters(Hashtable table);
 *    public void removeParameter(String key);
 * 
 */
public class OpenmisHttpServletRequest implements HttpServletRequest {

    private HttpServletRequest req;
    private Hashtable parameters;
    private String newrequestURI;
    
    public OpenmisHttpServletRequest(HttpServletRequest req) {
        this.req = req;
        
        // We use our own parameters table.
        // We first initialize it with the request's values.
        Enumeration paramEnum = req.getParameterNames();
        this.parameters = new Hashtable();
        while (paramEnum.hasMoreElements()) {
            String paramName = (String)paramEnum.nextElement();
            
            this.parameters.put(paramName, req.getParameterValues(paramName));
        }
    }
    
    public String getAuthType() {
        return req.getAuthType();
    }

    public Cookie[] getCookies() {
        return req.getCookies();
    }

    public long getDateHeader(String arg0) {
        return req.getDateHeader(arg0);
    }

    public String getHeader(String arg0) {
        return req.getHeader(arg0);
    }

    public Enumeration getHeaders(String arg0) {
        return req.getHeaders(arg0);
    }

    public Enumeration getHeaderNames() {
        return req.getHeaderNames();
    }

    public int getIntHeader(String arg0) {
        return req.getIntHeader(arg0);
    }

    public String getMethod() {
        return req.getMethod();
    }

    public String getPathInfo() {
        return req.getPathInfo();
    }

    public String getPathTranslated() {
        return req.getPathTranslated();
    }

    public String getContextPath() {
        return req.getContextPath();
    }

    public String getQueryString() {
        return req.getQueryString();
    }

    public String getRemoteUser() {
        return req.getRemoteUser();
    }

    public boolean isUserInRole(String arg0) {
        return req.isUserInRole(arg0);
    }

    public Principal getUserPrincipal() {
        return req.getUserPrincipal();
    }

    public String getRequestedSessionId() {
        return req.getRequestedSessionId();
    }

    public String getRequestURI() {
      if (newrequestURI != null) return newrequestURI;
      return req.getRequestURI();
    }
    
    public void setRequestURI(String uri) {
      this.newrequestURI = uri;
    }

    public StringBuffer getRequestURL() {
        return req.getRequestURL();
    }

    public String getServletPath() {
        return req.getServletPath();
    }

    public HttpSession getSession(boolean arg0) {
        return SessionContextManager.getManager().getSessionContext();
    }

    public HttpSession getSession() {
        return this.getSession(true);
    }

    public boolean isRequestedSessionIdValid() {
        return req.isRequestedSessionIdValid();
    }

    public boolean isRequestedSessionIdFromCookie() {
        return req.isRequestedSessionIdFromCookie();
    }

    public boolean isRequestedSessionIdFromURL() {
        return req.isRequestedSessionIdFromURL();
    }

    public boolean isRequestedSessionIdFromUrl() {
        return req.isRequestedSessionIdFromUrl();
    }

    public Object getAttribute(String arg0) {
        return req.getAttribute(arg0);
    }

    public Enumeration getAttributeNames() {
        return req.getAttributeNames();
    }

    public String getCharacterEncoding() {
        return req.getCharacterEncoding();
    }

    public void setCharacterEncoding(String arg0)
            throws UnsupportedEncodingException {
        req.setCharacterEncoding(arg0);
    }

    public int getContentLength() {
        return req.getContentLength();
    }

    public String getContentType() {
        return req.getContentType();
    }

    public ServletInputStream getInputStream() throws IOException {
        return req.getInputStream();
    }


/*********************************************************************************************
 * 
 * The following 4 methods don't call the wrapped HttpServletRequest,
 * as we use our own parameters table.
 * 
 */    
    public String getParameter(String name) {
        String[] params = getParameterValues(name);
        if (params == null || params.length == 0)
            return null;

        return params[0];
    }

    public Enumeration getParameterNames() {
        return this.parameters.keys();
    }

    public String[] getParameterValues(String arg0) {
        return (String[])this.parameters.get(arg0);
    }

    public Map getParameterMap() {
        return (java.util.Map)this.parameters;
    }
/**    
**********************************************************************************************/

    public String getProtocol() {
        return req.getProtocol();
    }

    public String getScheme() {
        return req.getScheme();
    }

    public String getServerName() {
        return req.getServerName();
    }

    public int getServerPort() {
        return req.getServerPort();
    }

    public BufferedReader getReader() throws IOException {
        return req.getReader();
    }

    public String getRemoteAddr() {
        return req.getRemoteAddr();
    }

    public String getRemoteHost() {
        return req.getRemoteHost();
    }

    public void setAttribute(String arg0, Object arg1) {
        req.setAttribute(arg0, arg1);
    }

    public void removeAttribute(String arg0) {
        req.removeAttribute(arg0);
    }
    
/*********************************************************************************************
 * 
 * The following 3 methods are not part of the Servlet API.
 * We use them to make the requests more flexible.
 * 
 */    
    public void addParameter(String key, Object value) {
        if (parameters.containsKey(key)) {
            String[] values = (String[])this.parameters.get(key);
            String[] newValues = new String[values.length+1];
            for (int i = 0; i < values.length; i++) {
                newValues[i] = values[i];
            }
            newValues[values.length] = (String)value;
        }
        else {
            String[] values = new String[] {(String)value};
            this.parameters.put(key, values);
        }
    }
    
    public void addParameters(Hashtable table)	{
        Enumeration enuma = table.keys();
        while (enuma.hasMoreElements())	{
            Object key = enuma.nextElement();
            Object value = table.get(key);
            parameters.put(key, value);  		
        }
	}

    public void removeParameter(String key) {
        this.parameters.remove(key);
    }
/**    
**********************************************************************************************/


    public Locale getLocale() {
        return req.getLocale();
    }

    public Enumeration getLocales() {
        return req.getLocales();
    }

    public boolean isSecure() {
        return req.isSecure();
    }

    public RequestDispatcher getRequestDispatcher(String arg0) {
        return req.getRequestDispatcher(arg0);
    }

    public String getRealPath(String arg0) {
        return req.getRealPath(arg0);
    }

    public int getRemotePort() {
        return req.getRemotePort();
    }

    public String getLocalName() {
        return req.getLocalName();
    }

    public String getLocalAddr() {
        return req.getLocalAddr();
    }

    public int getLocalPort() {
        return req.getLocalPort();
    }

}
