/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.algo.replication;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public final class AlwaysUpdateServerSynchroConflicResolver implements SynchroConflicResolver {

	/**
	 * Constructor for SSynchroConflicResolver.
	 */
	public AlwaysUpdateServerSynchroConflicResolver() {
		super();
	}

	/**
	 * @see org.openmobileis.server.synchro.safeserver.synchronization.SynchroConflicResolver#resolveAddandPresent(SynchroAtomicObject)
	 */
	public ReplicationSAOEvent resolveAddandPresent(SynchroAtomicObject ao) {
    ReplicationSAOEvent eventAO = new ReplicationSAOEvent(ao.getUID());
    eventAO.setUpdateSide(ReplicationSAOEvent.SERVER_SIDE);
    eventAO.setTypeModif(ReplicationSAOEvent.UPDATE);
    eventAO.setAobject(ao);
    return eventAO;
	}

	/**
	 * @see org.openmobileis.server.synchro.safeserver.synchronization.SynchroConflicResolver#resolveReplaceandModified(SynchroAtomicObject)
	 */
	public ReplicationSAOEvent resolveReplaceandModified(SynchroAtomicObject ao) {
    ReplicationSAOEvent eventAO = new ReplicationSAOEvent(ao.getUID());
    eventAO.setUpdateSide(ReplicationSAOEvent.SERVER_SIDE);
    eventAO.setTypeModif(ao.getModificationType());
    eventAO.setAobject(ao);
    return eventAO;
	}

	/**
	 * @see org.openmobileis.server.synchro.safeserver.synchronization.SynchroConflicResolver#resolveReplaceandDeleted(SynchroAtomicObject)
	 */
	public ReplicationSAOEvent resolveReplaceandDeleted(SynchroAtomicObject ao) {
    ReplicationSAOEvent eventAO = new ReplicationSAOEvent(ao.getUID());
    eventAO.setUpdateSide(ReplicationSAOEvent.SERVER_SIDE);
    eventAO.setTypeModif(ReplicationSAOEvent.CREATE);
    eventAO.setAobject(ao);
    return eventAO;
	}

	/**
	 * @see org.openmobileis.server.synchro.safeserver.synchronization.SynchroConflicResolver#resolveDeleteandModified(SynchroAtomicObject)
	 */
	public ReplicationSAOEvent resolveDeleteandModified(SynchroAtomicObject ao) {
    ReplicationSAOEvent eventAO = new ReplicationSAOEvent(ao.getUID());
    eventAO.setUpdateSide(ReplicationSAOEvent.SERVER_SIDE);
    eventAO.setTypeModif(ReplicationSAOEvent.DELETE);
    eventAO.setAobject(ao);
    return eventAO;
	}

}
