/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */
package org.openmobileis.synchro.algo.replication;

import org.openmobileis.common.util.collection.Orderable;
import org.openmobileis.common.util.collection.Sort;
import org.openmobileis.common.util.exception.SynchroException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class AtomicObjectArrayReplication implements Orderable {
	
	
	protected SynchroConflicResolver conflicResolver;
	
	public AtomicObjectArrayReplication (SynchroConflicResolver conflicResolver) {
		this.conflicResolver = conflicResolver;
	}
	
  /**
   * Compare client and server SynchroAtomicObject list and return all event associated to manage client or server update.
   */
	public ReplicationReturnData replicates(SynchroAtomicObject[] listAOClient, SynchroAtomicObject[] listAOServ)throws SynchroException {
    ReplicationReturnData SAOEventList = new ReplicationReturnData();
		try {
			if (listAOServ.length >1)	{
				Sort.shellsort(listAOServ, this);
			}
			if (listAOClient.length >1)	{
				Sort.shellsort(listAOClient, this);
			}
			
			int comptServ = 0;
			int comptClient = 0;
			
			while (comptServ < listAOServ.length && comptClient < listAOClient.length) {
				
				SynchroAtomicObject serverAtomicObject = listAOServ[comptServ];
				SynchroAtomicObject clientAtomicObject = listAOClient[comptClient];
				
				String serverUID = serverAtomicObject.getUID();
				String clientUID = clientAtomicObject.getUID();
			 
				if (serverUID.equals(clientUID)){
			    
					// conflict
					ReplicationSAOEvent eventAO = null;
          if (serverAtomicObject.getModificationType() != SynchroAtomicObject.DELETE) {
            if ((clientAtomicObject.getModificationType() == SynchroAtomicObject.ADD))  {					
              eventAO = conflicResolver.resolveAddandPresent(clientAtomicObject);
            } else if (clientAtomicObject.getModificationType() == SynchroAtomicObject.DELETE) {
              eventAO = conflicResolver.resolveDeleteandModified(clientAtomicObject);
            } else  {
              eventAO = conflicResolver.resolveReplaceandModified(clientAtomicObject);
            }
          } else  {
            eventAO = conflicResolver.resolveReplaceandDeleted(clientAtomicObject);
          }				
          SAOEventList.addReplicationSAOEvent(eventAO);
					
					comptServ++;
					comptClient++;
				} else if (serverUID.compareTo(clientUID) > 0) {
			    
					//gerer listAO2[comptClient]
					ReplicationSAOEvent eventAO = new ReplicationSAOEvent(clientUID);
					eventAO.setUpdateSide(ReplicationSAOEvent.CLIENT_SIDE);
					eventAO.setTypeModif(clientAtomicObject.getModificationType());
          eventAO.setAobject(clientAtomicObject);
          SAOEventList.addReplicationSAOEvent(eventAO);
					
					comptClient++;
				} else {
			    
					//gerer listAO1[comptServ]
					ReplicationSAOEvent eventAO = new ReplicationSAOEvent(serverUID);
					eventAO.setUpdateSide(ReplicationSAOEvent.SERVER_SIDE);
					eventAO.setTypeModif(serverAtomicObject.getModificationType());
          eventAO.setAobject(serverAtomicObject);
          SAOEventList.addReplicationSAOEvent(eventAO);
					
					comptServ++;
				}
			}
			
			if (comptServ >= listAOServ.length) {
				if (comptClient < listAOClient.length) {
					for (; comptClient < listAOClient.length; comptClient++) {
						
						SynchroAtomicObject clientAtomicObject = (SynchroAtomicObject)listAOClient[comptClient];
						
						ReplicationSAOEvent eventAO = new ReplicationSAOEvent(clientAtomicObject.getUID());
						eventAO.setUpdateSide(ReplicationSAOEvent.CLIENT_SIDE);
						eventAO.setTypeModif(clientAtomicObject.getModificationType());
            eventAO.setAobject(clientAtomicObject);
            SAOEventList.addReplicationSAOEvent(eventAO);
					}
				}
			} else { // firstTab not finished ==> second one is finished.
				for (; comptServ < listAOServ.length; comptServ++) {
					
					SynchroAtomicObject serverAtomicObject = (SynchroAtomicObject)listAOServ[comptServ];
					
					ReplicationSAOEvent eventAO = new ReplicationSAOEvent(serverAtomicObject.getUID());
					eventAO.setUpdateSide(ReplicationSAOEvent.SERVER_SIDE);
					eventAO.setTypeModif(serverAtomicObject.getModificationType());
          eventAO.setAobject(serverAtomicObject);
          SAOEventList.addReplicationSAOEvent(eventAO);
				}
			}
			
			
		} catch (Throwable e) {
			throw new SynchroException(e);
		}
    return SAOEventList;
	}

  
	// < 0 if p1 < p2, 0 if equal, > 0 if p1 > p2
	public int compareTo(Object AO1, Object AO2) {
		String AoID1 = ((SynchroAtomicObject)AO1).getUID();
		String AoID2 = ((SynchroAtomicObject)AO2).getUID();
   
		return AoID1.compareTo(AoID2);
	}

}
