/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.algo.replication;

import org.openmobileis.common.util.OpenMISSerializable;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Base interface to identify Atomic Object. All object managed by the synchronisation procesus
 * are Atomic Object. To be synchronised Object must implements this interface..
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public interface SynchroAtomicObject extends OpenMISSerializable{
  static final long serialVersionUID = 5521257935120563452L;
  public static final short ADD=1;
  public static final short REPLACE=2;
  public static final short DELETE=3;

  /**
   * return the Atomic Object UID. All synchronizeb object must have an unique UID.
   */ 
  public String getUID();
  
  /**
   * return the checksum of the atomic Object. If checksum has changed, the object is synchronized.
   */ 
  public String getCheckSum();

  /**
   * set the checksum of the atomic Object. If checksum has changed, the object is synchronized.
   */ 
  public void setCheckSum(String sum);

  /**
   * return the atomic Object creation date.
   */ 
  public long getCreationDate();

  /**
   * return the sync number of the last modification.
   * sync number can be anything that identifie the modification to manage synchronization.
   * It can be a timestamp or any incremental number.
   */ 
  public long getModifSyncNumber();

  /**
   * set the synchro number of the last modification.
   * sync number can be anything that identifie the modification to manage synchronization.
   * It can be a timestamp or any incremental number.
   */ 
  public void setModifSyncNumber(long ts);

  /**
   * return the last modification type (ADD, REPLACE, DELETE).
   */ 
  public short getModificationType();

  /**
   * set the last modification type (ADD, REPLACE, DELETE).
   */ 
  public void setModificationType(short type);

  public boolean equals(Object obj);

  public int hashCode();
}