/*
 *        OpenMobileIS - a free Java Framework for mobile applications
 *
 *   Copyright (C) 2002  Philippe Delrieu.
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Philippe Delrieu kept the rigth to distribute all code Copyrighted by philippe Delrieu
 *  under other licence term even commercial one.
 *  
 *  Modifications :
 *  2002 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.algo.syncnumber.impl;

import java.sql.ResultSet;
import org.openmobileis.common.util.log.*;
import org.openmobileis.common.util.database.ManagerDB;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.synchro.algo.syncnumber.SyncNumberManagerDelegate;
import org.openmobileis.synchro.algo.syncnumber.SyncNumberNotFoundException;
import org.openmobileis.synchro.algo.syncnumber.SynchroNumber;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class DefaultSyncNumberManagerDelegate implements SyncNumberManagerDelegate {
  private long currentSyncNumber;
  protected SyncNumberQueryManager syncQueryManager = null;

	public DefaultSyncNumberManagerDelegate() {
	}

	public DefaultSyncNumberManagerDelegate(SyncNumberQueryManager queryManager) {
		this.syncQueryManager = queryManager;
	}
  
	public void initDelegate()	{
		try {
	//		syncQueryManager.createSyncTable();
      
			// get last sync number
			syncQueryManager.createSyncTable();
			ResultSet result = null;
			result = syncQueryManager.getLastSyncNumber();
			if (result == null) {
				String[] syncParams = {"0", "0"};
				syncQueryManager.create(syncParams);
				result = syncQueryManager.getLastSyncNumber();
			}
			if (result.next())  {
				currentSyncNumber =  result.getLong(1);
			} else  {
				LogManager.traceAlert(LogServices.SERVERSERVICE," No current synchro number set to 0");
				currentSyncNumber = 0;
			}
		}  catch (java.sql.SQLException e) {
			LogManager.traceCritique(LogServices.SERVERSERVICE, new ServiceException("Error init Syncho Number manager. Exit System",e));
	//		System.exit(1);
		}  catch (ServiceException e) {
			LogManager.traceCritique(LogServices.SERVERSERVICE, new ServiceException("Error init Syncho Number manager. Exit System",e));
//			System.exit(1);
		}
	}
  
  public synchronized void setSyncNumberFactory(SyncNumberQueryManager fac)  {
    syncQueryManager = fac;
  }

  public SynchroNumber getSynchroNumber(long ns) throws SyncNumberNotFoundException {
    String[] syncParams = {String.valueOf(ns)};
    try {
      ResultSet result = syncQueryManager.getSyncNumberbysn(syncParams);
      if (!result.next()) {
        throw new SyncNumberNotFoundException(LogPriorities.INFO);
      } else {
        return new SynchroNumber(ns, result.getLong(1));
      }
    }  catch (java.sql.SQLException e) {
      LogManager.traceError(LogServices.SERVERSERVICE, new ServiceException("Error during getSynchroNumberBySN for synchro number :"+ns,e));
      throw new SyncNumberNotFoundException(LogPriorities.INFO);
    }  catch (ServiceException e) {
      LogManager.traceError(LogServices.SERVERSERVICE, e);
      throw new SyncNumberNotFoundException(LogPriorities.INFO);
    } finally {
      ManagerDB.getManager().garbageOpenedConnection();;
    }
  }

  public SynchroNumber getNextSynchroNumber() throws SyncNumberNotFoundException {
    try {
	    // update sync number
	    long date = System.currentTimeMillis();
	    long ns = this.incrementSynchoNumber();
	    String[] syncParams = {String.valueOf(ns), String.valueOf(date)};
	    syncQueryManager.create(syncParams);
	    SynchroNumber syncNumber = new SynchroNumber(ns, date);
	    return syncNumber;
    } catch (ServiceException ex)  {
      throw new SyncNumberNotFoundException(ex);
    }
  }

  public void deleteSyncNumber(SynchroNumber ns) {
    try {
	    String[] syncParams = {String.valueOf(ns.getSynchroNumber())};
	    syncQueryManager.delete(syncParams);
    } catch (ServiceException ex)  {
      LogManager.trace(ex);
    }
  }

  public void createSynchroNumber(long ns, long timestamp)  throws ServiceException  {
    String nsString = Long.toString(ns);
    String[] syncParams = {nsString, Long.toString(timestamp)};
    syncQueryManager.create(syncParams);
  }
  
  /**
   * set the new timestamp for the specified syncnumber. If it does not exist it is created.
   */
  public void setTimeStampForSynchroNumber(long syncNumber)	{
  	try	{
	  	long timestamp = System.currentTimeMillis();
	  	try	{
				this.getSynchroNumber(syncNumber);  //see if already exist.
		    String nsString = Long.toString(syncNumber);
	    	String[] syncParams = {nsString, Long.toString(timestamp), nsString};	
	    	syncQueryManager.update(syncParams);	
	  	} catch (SyncNumberNotFoundException ex)	{
	  		createSynchroNumber(syncNumber, timestamp);
	  	}
    } catch (ServiceException ex)  {
      LogManager.trace(ex);
    }
  }
  
  /**
   * return the last timestamp of the specified sync number. Throws SyncNumberNotFoundException if not found.
   */
  public long getTimeStampForSynchroNumber(long syncNumber)  throws SyncNumberNotFoundException {
    return this.getSynchroNumber(syncNumber).getTimeStamp();
  }

  private synchronized long incrementSynchoNumber() {
    return ++currentSyncNumber;
  }
}