/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.journal;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Implement a default behaviour for the log rendered that interface with the default Journal service.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 * @see JournalLogRenderer, JournalService
 */
import freemarker.template.*;

public abstract class DefaultLogRenderer implements JournalLogRenderer {

  protected String syncName;

  public DefaultLogRenderer() {}

  public DefaultLogRenderer(String syncName) {
      this.syncName = syncName;
  }

  public void render(SimpleList templatelist, JournalEntry[] entryList) {
    SimpleHash hash = new SimpleHash();
    SimpleList loglist = new SimpleList();
    for (int i=0; i<entryList.length; i++)  {
          // By default, the index 0 is used for the main entry (listener entry)
          if (entryList[i].getServiceEntryID() == 0)  {
            hash.put("serviceName", this.getSyncLabel());
            hash.put("synchroMessage", new SimpleScalar(entryList[i].getEntryData()));
            hash.put("synchroDate", entryList[i].getFormatedDate());
          } else  {
            SimpleHash hashlog = new SimpleHash();
            hashlog.put("detailMessage", entryList[i].getEntryData());
            if (entryList[i].getStatus() != -1)  {
              hashlog.put("status", String.valueOf(entryList[i].getStatus()));
            }
            loglist.add(hashlog);
          }
        }
    hash.put("detailLogList", loglist);
    hash.put("hasEntries", new SimpleScalar(entryList.length > 1));
    templatelist.add(hash);
  }

  public String getSyncName() {
    return syncName;
  }
	
  /**
   * Label tht describe the synchro listener for log rendering (ex : Mail Synchronisation)
   */
  public abstract String getSyncLabel();


}