/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.journal;

import org.odbms.ObjectSet;
import org.odbms.Query;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.database.DatabaseException;
import org.openmobileis.database.fastobjectdb.FODBCollectionDescriptor;
import org.openmobileis.database.fastobjectdb.FODBIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FODBStringIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;
import org.openmobileis.database.fastobjectdb.db.query.soda.FODBSodaObjectSet;
import org.openmobileis.synchro.openmsp.client.db.ActionDB;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Implement the database access for all journal entry.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 * @see DefaultLogRenderer, JournalService, JournalEntry
 */
public class JournalQueryManager {

  public JournalQueryManager() {
  }
  /*
  * check if the sync table exists. IF it does not exist, create it
  */
  public void createJournalTable() throws ServiceException, DatabaseException {
    String dbPath = PropertiesManager.getManager().getProperty("fastobjectdb.database.path");
    if (dbPath == null) {
      LogManager.traceError(0, "JournalQueryManager FPDB ERROR : FODB path property is not set. Can't create db");
    }
    try {
      FastObjectDB db = FastObjectDBManager.getCurrentFODB();
      if (!db.isCollectionExist("OMJournalEntry")) {
        FODBCollectionDescriptor descriptor = new FODBCollectionDescriptor("OMJournalEntry", JournalEntry.class);
        descriptor.setSynchronize(false);
        db.createCollection(descriptor);
        FODBStringIndexDescriptor IDDescriptor = new FODBStringIndexDescriptor("ID", FODBIndexDescriptor.UNIQUE, "getEntryID()", 12, 60);
        db.addIndex("OMJournalEntry", IDDescriptor);
        FODBStringIndexDescriptor serviceNameDescriptor = new FODBStringIndexDescriptor("ServiceName", FODBIndexDescriptor.MULTIPLE, "getServiceName()", 12, 50, 10);
        db.addIndex("OMJournalEntry", serviceNameDescriptor);
      }
    } catch (Throwable ex) {
      LogManager.traceError(0, "Client Factory FPDB ERROR : Unknown error during creation");
      LogManager.traceError(0, ex);
    }
  }
  /*
  * Add a Service journal entry in the database
  */
  public void createOrUpdate(JournalEntry entry) throws DatabaseException {
    try {
			Query q = FastObjectDBManager.getCurrentFODB().query();
			q.constrain(JournalEntry.class);
			Query subq = q.descend("getEntryID()");
			subq.constrain(entry.getEntryID()).equal();
			ObjectSet set = q.execute(); 
			if (set.hasNext())	{
        FastObjectDBManager.getCurrentFODB().replace("OMJournalEntry", entry);
      } else { //add
        FastObjectDBManager.getCurrentFODB().add("OMJournalEntry", entry);
      }
    } catch (Throwable ex) {
      throw new DatabaseException(ex);
    }
  }

  /*
  * Delete all entries for a service
  * parameters SERVICE_NAME
  */
  public void deleteAllEntryForService(String serviceName) throws ServiceException, DatabaseException {
    try {
      Array entryList = this.getAllEntryForService(serviceName);
      for (int i = 0; i < entryList.size(); i++) {
        JournalEntry entry = (JournalEntry) entryList.get(i);
        FastObjectDBManager.getCurrentFODB().delete("OMJournalEntry", entry);
      }
    } catch (Throwable ex) {
      throw new DatabaseException(ex);
    }
  }

  /*
  * get all entries for service
  * parameters SERVICE_NAME
  */
  public Array getAllEntryForService(String serviceName) throws ServiceException, DatabaseException {
    try {
			Query q = FastObjectDBManager.getCurrentFODB().query();
			q.constrain(JournalEntry.class);
			Query subq = q.descend("getServiceName()");
			subq.constrain(serviceName).equal();
			ObjectSet set = q.execute(); 
			return ((FODBSodaObjectSet)set).getArrayFromSet();
    } catch (Throwable ex) {
      throw new DatabaseException(ex);
    }
  }
}