/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.client.conduit;

import java.io.*;
import java.util.zip.GZIPInputStream;

import HTTPClient.*;

import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.codec.GeneralCoder;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.file.FileUtilities;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Synchronisation conduit use for debug purpose. Only generate 
 * the synchronisation request in a file sendOpenML.xml in the synchro directory.
 * Use tu see if the send synchro message is correctly generated.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class HttpOpenMSPSynchroConduit implements SynchroConduit {

  private HTTPConnection con = null;
  private boolean connected = false;
  private String userAgent;
  private String intallPath;
  
  public HttpOpenMSPSynchroConduit(String synchroDirectoryPath, String userAgent)  {
    this.userAgent = userAgent; // "OpenMSPPlug-WI-V1.0-WINCE[DR-FR-WINCE/04-02-00-00]"
    intallPath = synchroDirectoryPath;
  }

  public boolean isConnected() {
    return connected;
  }

  public String sendRequest(ConduitParameter[]parameters, String openMSPMessage, String url) throws SynchroException, UserNotFoundException{
    intallPath = FileUtilities.convertFileNameToSystem(intallPath);
    try {
      boolean usessl = false;
      int index = url.indexOf("http://");
      if (index == -1) { //test for ssl
        index = url.indexOf("https://");
        usessl = true;
        //TO BE DONE SSL IMPLEMENTATION
      }

      String shortURL;
      if (index == -1) {
        shortURL = url;
      } else if (index == 0) {
        shortURL = url.substring(7);
      } else {
        throw new Exception("invalid URL : " + url);
      }
      index = shortURL.indexOf("/");
      String realURL, resp;
      int port;
      if (index == -1) {
        resp = "/";
      } else {
        resp = shortURL.substring(index);
        shortURL = shortURL.substring(0, index);
      }
      index = shortURL.indexOf(":");
      if (index == -1) {
        realURL = shortURL;
        port = 80;
      } else {
        realURL = shortURL.substring(0, index);
        port = Integer.parseInt(shortURL.substring(index + 1));
      }

      if (con == null || !con.getHost().equals(url)) {
        con = new HTTPConnection(realURL, port);
				CookieModule.setCookiePolicyHandler(null);
        NVPair headerData[] = new NVPair[1];
        headerData[0] = new NVPair("User-Agent", userAgent);
        con.setDefaultHeaders(headerData);
      }
 
      /***************** DEBUGGING ************************/
        //save send request
 /*       byte[] openmsmessage = this.uncompressAndDecodeRequest(openMSPMessage.getBytes());
        // save send request
        String sendFileName = intallPath + "cn" + File.separator + "synchro" + File.separator + "sendCyberML.xml";
        FileOutputStream sendFileStream = new FileOutputStream(sendFileName);
        try {
          sendFileStream.write(openmsmessage);
        } finally {
          sendFileStream.flush();
          sendFileStream.close();
        }
        System.out.println(new String(openmsmessage)); 
        */
      /***************** END DEBUGGING ********************/

      //send request
			openMSPMessage.getBytes(); //put here to correct PC2003 PDA bug
			// no parameters is send for the fisrt request if not set.
      NVPair form_data[] = new NVPair[2];
      form_data[0] = new NVPair("openMISData", openMSPMessage);
      form_data[1] = new NVPair("openMISGroup", parameters[0].value);
      HTTPResponse rsp = con.Post(resp, form_data);

      if (rsp.getStatusCode() >= 300) {
        //  System.out.println("TestSynchroService Error in HTTP Request :" + rsp.getStatusCode());
        if ((rsp.getStatusCode() == 403) || (rsp.getStatusCode() == 401)) {
          throw  new UserNotFoundException("Error during HTTP send request conduit. Server answer :"+rsp.getStatusCode());
        } else  {
          throw  new SynchroException("Error during HTTP send request conduit. Server answer :"+rsp.getStatusCode());
        }
      } else {
        byte[] data = rsp.getData();
 //         System.out.println("sendRequest HTTP request OK");
 //         System.out.println("SyncNum :" + rsp.getHeader("SyncNum"));
 //         System.out.println("data length :" + data.length);
        ByteArrayInputStream inst  = new ByteArrayInputStream(data);
        try {
          org.openmobileis.common.util.file.ZipUtilities.unZipData(inst, intallPath);
        } finally {
          inst.close();
        }

        /************* DEBUGING ********************/
  /*       try {
            java.io.FileOutputStream writer = new java.io.FileOutputStream(intallPath + "cn" + File.separator + "synchro.zip");
            writer.write(data);
            writer.flush();
            writer.close();
            System.out.println("write synchro.zip");
          } catch (Exception e) {
            e.printStackTrace();
          }
          System.out.println("HTTP request answer saved in " + intallPath + "cn");
          */
        /************** END DEBUGGING ***********************/
        String openMLFileName = intallPath+File.separator+"cyberML.xml";
        StringBuffer buffer = new StringBuffer();
        java.io.FileInputStream in = new java.io.FileInputStream(openMLFileName);
        try {
          byte[] bytes = new byte[512];
            int len;
            while ( ( len = in.read( bytes ) ) != -1 ) {
                buffer.append(new String(bytes, 0, len));
            }
        } finally {
          in.close();
        }
        return buffer.toString();
      }
    } catch (UserNotFoundException ioe) {
      throw ioe;
    } catch (Throwable ioe) {
      throw  new SynchroException(ioe);
    }
  }

  /*
  * Open RAS connection
  * @param : isSSLRequired : boolean indicates wether or not is a SSL connection
  * @return: boolean (true : connection OK, false : connection failed
  */
  public void openRAS() throws SynchroException { //do nothing
    HTTPClient.CookieModule.setCookiePolicyHandler(null);
    connected = true;
   }

  /*
  * Close RAS connection
  * @return boolean (status OK, not OK)
  */
  public void closeRAS() throws SynchroException { // do nothing
    connected = false;
    if (con != null) con.stop();
   }

  public String getRASConnectionNames() {
    return "connection1;connection2";
  }

  private byte[] uncompressAndDecodeRequest(byte[] request) throws IOException {
    // decode compressed data
    byte[] decodedData = GeneralCoder.decodeBase64(request);

    // uncompresse data
    ByteArrayOutputStream byteStream = new ByteArrayOutputStream(1024);
    ByteArrayInputStream inStream = new ByteArrayInputStream(decodedData);
    GZIPInputStream zipStream = new GZIPInputStream(inStream);
    try {
      byte[] buff = new byte[1024];
      int length;
      while ((length = zipStream.read(buff)) != -1) {
        byteStream.write(buff, 0, length);
      }
    } finally {
      zipStream.close();
      byteStream.close();
    }
    return byteStream.toByteArray();
  }

  public String getIntallPath() {
    return intallPath;
  }
}
