/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.client.conduit;

import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.exception.SynchroException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Base interface that all synchronisation conduit must implements.
 * A synchronisation conduit manage all physical connection process to the nomdLink server.
 * The connexion protocol and OS native access is managed by the conexion conduit.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public interface SynchroConduit {

  /**
  * Return true if it exists a connexion to internet or desktop
  */
  public boolean isConnected();

  /**
  * Send the request to the server.
  * return a string containing the server response.
  */
   public String sendRequest (ConduitParameter[] parameters, String request, String url) throws SynchroException, UserNotFoundException;

  /**
  * Open RAS connection
  * @param : isSSLRequired : boolean indicates wether or not is a SSL connection
  * @return: 
  */
  public void openRAS () throws SynchroException;

  /**
  * Close RAS connection
  * @return 
  */
  public void closeRAS () throws SynchroException;

  /**
   *return a list of all disponible ras connection.
   * Every connexion name is separated by a ; .
   */
  public String getRASConnectionNames();
  
  /**
   * Return the directory path where synchronized file are stored.
   * @return
   */
  public String getIntallPath();

  
}