/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.client.core;

import java.util.Properties;

import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.log.*;


/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Synchro number storage classe.
 * Synchro number are store with service name in the numSync.properties file
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class NumSyncQueryDB  {

  // key serviceName, value syncNumber
  private java.util.Properties props = new java.util.Properties();


  public NumSyncQueryDB() {
    String propsFilePath  = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.client.openmsp.numsyncpropsfile");
   java.io.FileInputStream input = null;
    try {
      input = new java.io.FileInputStream(propsFilePath);
      props.load(input);
    } catch (Exception e) {
      LogManager.traceWarning(LogServices.WEBSERVICE, "NumSyncQueryDB Can not load " + propsFilePath+". Create it.");
      props = new Properties();
      this.saveProps();
    } finally {
      if (input != null)  {
        try {
          input.close();
        } catch (Exception e) {
          LogManager.traceWarning(LogServices.WEBSERVICE, "Can not close " + propsFilePath);
        }
      }
    }
  }
  
  public void initAllSyncNumber()	{
    props.clear();
    this.saveProps();
  }


  public void saveSyncNumber (String serviceName, String syncNumber) {
    props.put(serviceName, syncNumber);
    this.saveProps();
  }

/*
* delete a row by giving the SERVICE_NAME in parameters
*/
 public void delete (String serviceName) {
   props.remove(serviceName);
   this.saveProps();
}


/*
* get  Sync number of specified service
* @param : one param giving service name (SERVICE_NAME)
*/
 public String getServiceSN(String serviceName) {
  return props.getProperty(serviceName);
 }

 private void saveProps() {
    java.io.FileOutputStream output = null;
    String propsFilePath  = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.client.openmsp.numsyncpropsfile");
    try {
      output = new java.io.FileOutputStream(propsFilePath);
        // since the method #store is not available in jdk1.1.6, dot it manually
       for (java.util.Enumeration e = props.keys(); e.hasMoreElements();) {
              String key = (String)e.nextElement();
              String val = (String)props.get(key);
              output.write((key + "=" + val + "\n").getBytes());
       }
        output.flush();
    } catch (java.io.IOException e) {
      LogManager.traceError(LogServices.WEBSERVICE, "Impossible to store property file : " + propsFilePath);
   } finally {
      if (output != null)  {
        try {
         output.close();
        } catch (java.io.IOException e) {
          LogManager.traceError(LogServices.WEBSERVICE, "Impossible to close property file : " + propsFilePath);
        }
      }
   }
 }

}
