/*
 *        OpenMobileIS - a free Java Framework for mobile applications
 *
 *   Copyright (C) 2002  Philippe Delrieu.
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Philippe Delrieu kept the rigth to distribute all code Copyrighted by philippe Delrieu
 *  under other licence term even commercial one.
 *  
 *  Modifications :
 *  2002 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.client.db;

import org.openmobileis.common.util.log.*;
import org.openmobileis.common.util.exception.SynchroException;
//import org.openmobileis.embedded.syncho.db.*;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Default implementation of the listener use to decode synchro ROw from the synchronisation,process
 * If you change the synchronisation process the the way dtabase update rows are encoded you have to modify this listener. 
 * from the synchro process. Abstrct helper methods are provided to get the database.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class ImportDecodedRowListener implements DecodedRowListener {

  private ImportQueryManager importQM;

  public ImportDecodedRowListener(ImportQueryManager qm) {
    importQM = qm;
  }

  public void notifyNewRowDecoded(char action, String[] rowData) {
    try {
      //make query
      if (action == DBImportFileCoder.ADDAction)  {
        importQM.executeCreateQuery(rowData);
      } else if (action == DBImportFileCoder.REPLACEAction)  {
        importQM.executeUpdateQuery(rowData);
      } else if (action == DBImportFileCoder.DELETEAction)  {
        // get PDA uid for delete. TBD
        importQM.executeDeleteQuery(rowData);
      }
    } catch (Exception ex)  {
      LogManager.trace(new SynchroException("ImportDecodedRowListener Error in notifyNewRowDecoded", ex) );
    }

  }

  public void notifyError(int ligne, char errorCharacter) {
    LogManager.traceError(LogServices.SYNCHROSERVICE, "Error during decoding import Data file at line"+ligne+" for char '"+errorCharacter+"'" );
  }
}