/*
 *        OpenMobileIS - a free Java Framework for mobile applications
 *
 *   Copyright (C) 2002  Philippe Delrieu.
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Philippe Delrieu kept the rigth to distribute all code Copyrighted by philippe Delrieu
 *  under other licence term even commercial one.
 *  
 *  Modifications :
 *  2002 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.client.db;

import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.file.FileUtilities;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.DatabaseException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Base implemtnation of ImportQueryManager use to import object to the database.
 * All snchronisation rows are serialized object.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public abstract class ObjectDBImportQueryManager implements ImportQueryManager {

  /** Creates new ObjectDBImportQueryManager */
  public ObjectDBImportQueryManager() {
  }

	public void importDataFile(String fileName)  {
		ImportDecodedRowListener listener = new ImportDecodedRowListener(this);
		String goodFileName = FileUtilities.convertFileNameToSystem(fileName);
		try {
			this.beginImport();
			DBImportFileCoder.getCoder().decodeFileData(goodFileName, listener);
		} catch (Throwable ex)  {
			LogManager.trace(new SynchroException("DefaultDBImportQueryManager importDataFile Error ", ex));
		} finally {
			try {
				this.endImport();
			} catch (Throwable ex)  {
				LogManager.trace(new SynchroException("DefaultDBImportQueryManager importDataFile Error ", ex));
			}
		}
	}


	protected String[] cleanData(String[] data) {

		for (int i=0; i<data.length; i++) {
			data[i] =  DBImportFileCoder.getCoder().removeAntiSlash(data[i]);
		}
		return data;
	}

   
  /**
   * execute the creation query
   */
  public void executeCreateQuery(String[] data) throws DatabaseException  {
    try {
      Object obj = DBImportFileCoder.getCoder().unserializeDBObject(data);
      this.addObject(data[0], obj);
    } catch (Throwable ex)  {
      throw new DatabaseException(ex);
    }
  }

  /**
   *  execute the update query
   */
  public void executeUpdateQuery(String[] data) throws DatabaseException  {
    try {
      Object obj = DBImportFileCoder.getCoder().unserializeDBObject(data);
      this.replaceObject(data[0], obj);
    } catch (Throwable ex)  {
      throw new DatabaseException(ex);
    }
  }

  /**
   *  execute the delete query
   */
  public void executeDeleteQuery(String[] data) throws DatabaseException  {
    this.deleteObject(data[0]);
  }

  public abstract void addObject(String uid, Object object) throws DatabaseException;
    
  public abstract void replaceObject(String uid, Object object) throws DatabaseException;

  public abstract void deleteObject(String uid) throws DatabaseException;

	protected abstract void beginImport() throws DatabaseException;
	protected abstract void endImport() throws DatabaseException;

}
