/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.client.services;

import freemarker.template.*;
import java.io.IOException;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.navigation.NavigationBarService;
import org.openmobileis.synchro.journal.JournalEntry;
import org.openmobileis.synchro.journal.JournalLogRenderer;
import org.openmobileis.synchro.journal.JournalManager;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Service that show synchronisation result. Each Synchro listener can log
 * its synchro result has a JOurnalEntry with the JournalManager.
 * The journal service show saved synchronisation result. 
 * The show each entry, the JournalService use a JournalRenderer. 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 * @see DefaultLogRenderer, JournalService, JournalEntry
 */
public class SynchroJournalDisplayService  extends TemplateService implements NavigationBarService  {

  private static final String templatePath="openmischannel"+java.io.File.separator+"journalsynchro.htm";

  public SynchroJournalDisplayService() {
    // initialize tables at the launching
    JournalManager.getManager();
  }

  public String runTemplate(HttpServletRequest req, HttpServletResponse res, TemplateModelRoot templateData)  throws ServletException, IOException {
    Array list = JournalManager.getManager().getRendererList();
    SimpleList templatelist = new SimpleList();
    JournalLogRenderer renderer;
    int size = list.size();
    for (int i=0; i<size; i++) {
      renderer =  (JournalLogRenderer)list.get(i);
      String syncName = renderer.getSyncName();
      JournalEntry[] entryList = JournalManager.getManager().getAllJournalEntryForService(syncName);
      if (entryList.length > 0) {
        renderer.render(templatelist, entryList);
      }
    }
    templateData.put("serviceLogList", templatelist);
    return templatePath;
  }
	
	public String getNavigationBarLabel(HttpServletRequest req)	{
		return "Journal de Synchro";
	}
	
	public boolean displayFormExitMessage() {
		return false;
	}
	
	public boolean displayRecursive() {
		return false;
	}

  /* (non-Javadoc)
   * @see org.openmobileis.services.Service#getName()
   */
  public String getName() {
    // TODO Auto-generated method stub
    return "/synchro/journal";
  }
}