/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.protocol;


/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Represents Add, Delete, Replace commands
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class Command extends RequestCommand implements Element {
  /**
   * Set version of the serialized data to make it compatible with
   * new class versions.
  */
  final static long serialVersionUID = 1L;



    // target : define the receiver service in charge of processing the command
  protected String target;
  protected long sessionID=-1;
  // source : identify the source which sent the command
  protected String source;

  protected Command (int commandType) {
    super(commandType);
  }

  public Command(int commandType, String source, String target) {
    super(commandType);
    this.source = source;
    this.target = target;
  }


 public String getTarget () {
    return target;
    }

  public String getSource() {
    return source;
    }

  /**
   * return the optional sessionID of source element. If not present return -1.
   *
   */
  public long getSourceSessionID()  {
    return sessionID;
  }

 public void setSource(String source) {
  this.source = source;
 }

 public void setTarget (String target) {
  this.target = target;
 }

 public void setSourceSessionID(long sessionid) {
  this.sessionID = sessionid;
 }


 public String toString() {
  return super.toString() + " source : " + source + " target : " + target+" sessionID "+sessionID;
 }

  /*** Implements Element interface **/
 public void writeContent(StringBuffer buffer) {
   super.writeContent(buffer);
    if (source != null) {
      MessageFactory.writeSource(buffer, source, sessionID);
      buffer.append("\n");
    }
     if (target != null) {
      MessageFactory.writeTarget(buffer, target);
      buffer.append("\n");
     }
 }


}