/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.protocol;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Item used for Request commands and Result command.
 * If an item contains data, use DataItem
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class DataItem  extends Item implements Element {
  /**
   * Set version of the serialized data to make it compatible with
   * new class versions.
  */
  final static long serialVersionUID = 1L;


  protected String meta;
  protected String data;

  public DataItem(int itemType, String meta, String data, String source, String target) {
    super(itemType, source, target);
    this.data = data;
    this.meta = meta;
  }

  protected DataItem (int itemType) {
     super(itemType);
  }


  public String getMetaInformation() {
    return meta;
    }

  public String getData() {
    return data;
    }

  public String toString() {
    return super.toString() +  " meta : " + meta + " data : " + data;
  }

  public void setMetaInformation(String meta) {
    this.meta = meta;
  }

  public void setData(String data) {
    this.data = data;
  }


 public void writeContent(StringBuffer buffer) {
     super.writeContent(buffer);
     if (meta != null) {
        MessageFactory.writeElement(buffer, MessageFactory.TAG_META, meta);
        buffer.append("\n");
     }
     if (data != null) {
        String toDisplay;
        if ( (data.indexOf("<xml>")) != -1)
          toDisplay = "<![CDATA[" + data + "]]>";
        else
          toDisplay = data;
        MessageFactory.writeElement(buffer, MessageFactory.TAG_DATA, toDisplay);
        buffer.append("\n");
     }
  }


}