/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.protocol;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.synchro.openmsp.OpenMSPException;
/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Default handler to parse OpenMSP message
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public class DefaultHandler extends  Handler {

  private String parsingStep = "";
  private Message message = null;
  private Array pile = new Array(10);
  private int indexPile = -1;
  private boolean isCred = false;
  private StringBuffer tagContent;


  public DefaultHandler() {
  }

  // override Handler methods in order to create Message from OpenMSP string
  public void startElement (String name) throws OpenMSPException {
    if (name.equals(MessageFactory.TAG_ROOT)) {
      ContainerMessage root = new ContainerMessage();
      message = new Message(root, false);
      addParsedElement(root);
    }  else if (message == null)
      throw new OpenMSPException ("Don't find root tag OpenMSP");
    else if (name.equals(MessageFactory.TAG_HEADER)) {
      ((ContainerMessage)pile.get(indexPile)).content = new Header();
    } else if (name.equals(MessageFactory.TAG_CRED))
      isCred = true;
    else if (name.equals(MessageFactory.TAG_SYNC))
      addParsedElement(new ContainerMessage(new Command(Element.SYNC)));
    else if (name.equals(MessageFactory.TAG_ADD))
      addParsedElement(new ContainerMessage(new RequestCommand(Element.ADD)));
    else if (name.equals(MessageFactory.TAG_DELETE))
      addParsedElement(new ContainerMessage(new RequestCommand(Element.DELETE)));
    else if (name.equals(MessageFactory.TAG_REPLACE))
      addParsedElement(new ContainerMessage(new RequestCommand(Element.REPLACE)));
    else if (name.equals(MessageFactory.TAG_MAP))
      addParsedElement(new ContainerMessage(new Command(Element.MAP)));
    else if (name.equals(MessageFactory.TAG_GET))
      addParsedElement(new ContainerMessage(new RequestCommand(Element.GET)));
    else if (name.equals(MessageFactory.TAG_SEQUENCE))
      addParsedElement(new ContainerMessage(new Sequence()));
    else if (name.equals(MessageFactory.TAG_RESULT))
      addParsedElement(new ContainerMessage(new Result()));
    else if (name.equals(MessageFactory.TAG_STATUS))
      addParsedElement(new AtomicMessage(new Status()));
    else if (name.equals(MessageFactory.TAG_MAP_ITEM))
      addParsedElement(new AtomicMessage(new Item(Element.MAP_ITEM)));
    else if (name.equals(MessageFactory.TAG_ITEM))
      addParsedElement(new AtomicMessage(new DataItem(Element.ITEM)));
    else if (name.equals(MessageFactory.TAG_FINAL))
      message.setFinal(true);

    if (name.equals(MessageFactory.TAG_META))  {
      tagContent = new StringBuffer();
    } else if (!name.equals(MessageFactory.TAG_URI)) {
      parsingStep = name;
      if (tagContent!=null) {
        tagContent.append("<").append(name).append(">");
      }
    }
 }


 public void endElement (String name) throws OpenMSPException {
    if ( (name.equals(MessageFactory.TAG_ROOT)) || (name.equals(MessageFactory.TAG_SYNC)) || (name.equals(MessageFactory.TAG_ADD)) ||
         (name.equals(MessageFactory.TAG_DELETE)) || (name.equals(MessageFactory.TAG_REPLACE)) || (name.equals(MessageFactory.TAG_MAP)) ||
         (name.equals(MessageFactory.TAG_GET)) || (name.equals(MessageFactory.TAG_SEQUENCE)) || (name.equals(MessageFactory.TAG_RESULT)) ||
         (name.equals(MessageFactory.TAG_STATUS)) || (name.equals(MessageFactory.TAG_ITEM)) || (name.equals(MessageFactory.TAG_MAP_ITEM)))
      removeParsedElement();
    else if (name.equals(MessageFactory.TAG_CRED))
      isCred = false;
    else if ((name.equals(MessageFactory.TAG_META)) && isCred)  {
      getElement().setCredentialMeta(tagContent.toString());
      tagContent = null;
    } else if (name.equals(MessageFactory.TAG_META))  {
      getElement().setMetaInformation(tagContent.toString());
    }else if (tagContent != null)  {
      tagContent.append("</").append(name).append(">");
    }
    parsingStep = "";
  }


 public void getValue (String value) throws OpenMSPException {
   try {
      if (parsingStep.equals(MessageFactory.TAG_SESSION)) {
        Element element = getElement();
        if (element.getElementType() == Element.HEADER)  {
          ((Header)element).sessionID = Long.parseLong(value);
        } else  {
          element.setSourceSessionID(Long.parseLong(value));
        }
      }
      else if (parsingStep.equals(MessageFactory.TAG_TARGET))
        getElement().setTarget(value);
      else if (parsingStep.equals(MessageFactory.TAG_SOURCE))
        getElement().setSource(value);
      else  if ((parsingStep.equals(MessageFactory.TAG_DATA)) && isCred)
        getElement().setCredentialData(value);
      else if (parsingStep.equals(MessageFactory.TAG_CMD_REF))
        getElement().setCmdRef(Integer.parseInt(value));
      else if (parsingStep.equals(MessageFactory.TAG_SOURCE_REF))
        getElement().setSourceRef(value);
      else if (parsingStep.equals(MessageFactory.TAG_TARGET_REF))
        getElement().setTargetRef(value);
      else if (parsingStep.equals(MessageFactory.TAG_CMDID)) {
        int cmdId = Integer.parseInt(value);
        if (message.cmdIdCounter < cmdId)
            message.cmdIdCounter = cmdId;
        getElement().setCmdId(cmdId);
      } else if ((parsingStep.equals(MessageFactory.TAG_META)) && (!isCred) )
        getElement().setMetaInformation(value);
      else  if ((parsingStep.equals(MessageFactory.TAG_DATA)) && (!isCred) )
         getElement().setData(value);
      else if (tagContent != null)  {
        tagContent.append(value);
      }
   } catch (java.lang.NullPointerException e) { // means the getElement() return null
       throw new OpenMSPException ("Malformed cyberML document, for " + parsingStep + " tag with value : " + value);
   }
 }

 private void addParsedElement (ContainerMessage message) {
    indexPile++;
    pile.add(message);
 }

 private void removeParsedElement () throws OpenMSPException {
    if (indexPile < 0)
      throw new OpenMSPException ("Invalid XML message : the end tag " + parsingStep + "does not correspond to the begin tag");
    // else get current message
    ContainerMessage message = (ContainerMessage)pile.get(indexPile);
    pile.remove(indexPile);
    // add to its parent if it is not the root
    indexPile--;
    if (indexPile >= 0)
      ((ContainerMessage)pile.get(indexPile)).add(message);
 }

 private Element getElement() {
    return ((ContainerMessage)pile.get(indexPile)).getElement();
 }

 public Message getMessage() {
  return message;
 }
}