/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.protocol;

import org.openmobileis.common.util.OpenMISSerializable;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Interface for encoding / decoding OpenMSP elements
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public interface Element extends OpenMISSerializable {
  /**
   * Set version of the serialized data to make it compatible with
   * new class versions.
  */
  final static long serialVersionUID = 1L;


 // list of known element types
  public static int HEADER = 0;
  public static int SYNC = 1;
  public static int GET = 2;
  public static int ADD = 3;
  public static int REPLACE = 4;
  public static int DELETE = 5;
  public static int MAP = 6;
  public static int RESULT = 7;
  public static int STATUS = 8;
  public static int ITEM = 9;
  public static int MAP_ITEM = 10;
  public static int SEQUENCE = 11;
  public static int FINAL = 12;

  /*
  * Return the element type whose the value is one the element type defined as constant
  * @return: int
  */
  public int getElementType();

  /*
  * Return the command id of the element, return 0 if it is not initialized
  * @return: int
  */
  public int getCmdId();

  public boolean hasCommmandId();

 //**** Methods used to encode OpenMSP messages

  public void writeBeginTag(StringBuffer buffer);

  public void writeContent(StringBuffer buffer);

  public void writeEndTag(StringBuffer buffer);

 //***** Methods used to decode OpenMSP messages

 public void setCmdId (int cmdId);

 public void setElementType(int elementType);

 public void setCredentialData (String data);

 public void setCredentialMeta (String meta);

 public void setSource(String source);

 public void setSourceSessionID(long sessionID);

 public void setTarget (String target);

 public void setMetaInformation (String meta);

 public void setCmdRef (int cmdRef);

 public void setSourceRef(String source);

 public void setTargetRef(String target);

 public void setData (String data);





}