/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.protocol;

import org.openmobileis.synchro.openmsp.OpenMSPException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public class Parser {

  private static int BEGIN_TAG = 0;
  private static int END_TAG = 1;
  private static int SINGLE_TAG = 2;

  private Handler handler = null;
  // Manage elements tree - maximum deep is 50
  private String[] elements = new String[50];
  private int index = 0;

  public Parser(Handler handler) {
    this.handler = handler;
  }

  public void parse (String inputCyberML) throws OpenMSPException {
    int c;
    boolean inside = false;
    boolean exclamation = false;
    boolean isPCData = false;
    StringBuffer element = new StringBuffer();
    StringBuffer pcdata = new StringBuffer();
    java.io.ByteArrayInputStream inputXML = new java.io.ByteArrayInputStream(inputCyberML.getBytes());
    while ((c = inputXML.read())> -1) {
        switch ((char) c) {
          case '!' :
              if (inside)  // manage CDATA and comments
                exclamation = true;
              break;
          case '<' :
              if (inside && !exclamation)
                throw new OpenMSPException ("Malformed document, expected '>' and found '<' during OpenMSP parsing");
              else if (inside && exclamation) { // suppose it is a CDATA
                isPCData = true;
                element.append((char)c);
              } else {
                inside = true;
                element = new StringBuffer();
                parsePcData(pcdata);
                }
              break;
          case '>' :
              if (isPCData) { // test if it is the end of PC data ... could be better !! (can not use stringBuffer.substring() in jdk1.1
                  String tempStr = element.toString().substring(element.length()-2, element.length());
                  if (tempStr.equals("]]")) {// suppose it is inside a CDATA
                      isPCData = false;
                      exclamation = false;
                  }
              }
              if ((inside) && (!isPCData)) {
                  inside = false;
                  parseElement(element);
                  // ready to parse the possible PCDATA for the current element
                  pcdata = new StringBuffer();
              } else if (isPCData) {
                  element.append((char)c);
              } else
                throw new OpenMSPException ("Malformed document, expected '>' and found '<' during OpenMSP parsing");
              break;
         default :
            if (inside)
              element.append((char)c);
            else
              pcdata.append((char)c);
            break;
        } // end switch
      } // end while
   } //end parse


// Analyse elements (header, comments, begin or end tags
 private void parseElement (StringBuffer element) throws OpenMSPException {
    String parsedElement = element.toString();
    char c = parsedElement.charAt(0);
    switch (c) {
       case '!' :
          // skip xml COMMENT
          index = parsedElement.indexOf("--");
          int lastIndex = parsedElement.indexOf("--", parsedElement.length()-2);
          if ((index != -1) && (lastIndex != -1)) // find comment
            break;
        case '[' :
          if ((parsedElement.startsWith("[CDATA[")) && (parsedElement.endsWith("]]")))   { // manage CDATA
            String data = parsedElement.substring("[CDATA[".length() , parsedElement.length()-2);
            parsePcData(new StringBuffer(data));
          } else // Error, malformed tag
            throw new OpenMSPException ("Found illegal character '!' in element : " + parsedElement);
          break;
      case '/' : // end element
          addElement (parsedElement.substring(1), Parser.END_TAG);
          break;
      default :
          // case single tag : <xxx/>
            if (parsedElement.endsWith("/"))
                addElement (parsedElement.substring(0, parsedElement.length() -1), Parser.SINGLE_TAG);
            else
                addElement (parsedElement, Parser.BEGIN_TAG);
        } // end switch
  }

private void parsePcData (StringBuffer data) throws OpenMSPException {
    if ((data != null) && (data.toString().trim().length() > 0))
      handler.getValue (data.toString());
  }


private void addElement (String elementName, int elementStatus) throws OpenMSPException {
    if (elementStatus == Parser.SINGLE_TAG) {
      handler.startElement(elementName);
      handler.endElement(elementName);
    } else if (elementStatus == Parser.BEGIN_TAG) {
      elements[index] = elementName;
      index++;
      handler.startElement(elementName);
    } else if (elementStatus == Parser.END_TAG) {
        if (!elements[index-1].equals(elementName))
          throw new OpenMSPException ("Malformed document, found " + elementName + "expected : " + elements[index]);
        else {
           elements[index] = null;
           index--;
           handler.endElement(elementName);
        }
    }
  }

 }