/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.protocol;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Result command. Contains a list of data items.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 * @See Item
 * @See DataItem
 */

public class Result extends ResponseCommand  implements Element {
  /**
   * Set version of the serialized data to make it compatible with
   * new class versions.
  */
  final static long serialVersionUID = 1L;



  protected String meta;
  protected String targetRef;
  protected String sourceRef;



  protected Result () {
    super(AbstractCommand.RESULT);
  }

  public Result(int cmdRef, String sourceRef, String targetRef) {
    super (AbstractCommand.RESULT ,cmdRef);
    this.sourceRef = sourceRef;
    this.targetRef = targetRef;
  }


  public String getMetaInformation() {
    return meta;
    }

  public String getTargetRef() {
    return targetRef;
    }

  public String getSourceRef() {
    return sourceRef;
    }

  public String toString() {
    return super.toString() + "meta : " + meta + " sourceRef " + sourceRef + " targetRef " + targetRef;
 }

   /*** Implements Element interface **/
 public void setSourceRef(String source){
   sourceRef = source;
   }

  public void setTargetRef(String target){
   targetRef = target;
   }

  public void setMetaInformation (String meta) {
    this.meta = meta;
    }

 public void writeContent(StringBuffer buffer) {
      super.writeContent(buffer);
      if (meta != null) {
      MessageFactory.writeElement(buffer, MessageFactory.TAG_META, meta);
      buffer.append("\n");
      }
     if (sourceRef != null) {
      MessageFactory.writeElement(buffer, MessageFactory.TAG_SOURCE_REF, sourceRef);
      buffer.append("\n");
     }
     if (targetRef != null) {
      MessageFactory.writeElement(buffer, MessageFactory.TAG_TARGET_REF, targetRef);
      buffer.append("\n");
     }
 }

}