/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.synchro.openmsp.protocol.test;

import java.util.Properties;

import org.openmobileis.common.user.UserManager;
import org.openmobileis.common.user.impl.MonoUserUserManagerFactory;
import org.openmobileis.common.user.profil.ProfilDataManager;
import org.openmobileis.common.user.profil.impl.XmlFileProfilDataFactory;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.embedded.webserver.WebServer;
import org.openmobileis.synchro.client.SynchroDescriptor;
import org.openmobileis.synchro.client.SynchroManager;
import org.openmobileis.synchro.openmsp.client.OpenMSPSynchroManager;
import org.openmobileis.synchro.openmsp.client.conduit.HttpOpenMSPSynchroConduit;
import org.openmobileis.synchro.openmsp.server.synctarget.SynchroTargerManager;
import org.openmobileis.synchro.openmsp.server.synctarget.proxy.GenericProxySynchroTarget;
import org.openmobileis.synchro.security.auth.Credential;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class TestOpenMSPSynchro {

  /**
   * 
   */
  public TestOpenMSPSynchro() {
    super();
  }

  /**
   * @param args
   */
  public static void main(String[] args) {
    //init 
    LogManager.registerLogManager(null);
    //init Terminal side.
    //load synchro properties
    try {
      PropertiesManager.getManager().addProperty("org.openmobileis.synchro.client.openmsp.numsyncpropsfile",System.getProperty("user.dir") + "/WEB-INF/conf/numsync.txt");
      OpenMSPSynchroManager.getManager().addListener(new TestOpenMSSyncListener(), null);
      SynchroManager.getManager().registerSynchroProcessor(OpenMSPSynchroManager.getManager());
  
      //init Synchro Server side.
      PropertiesManager.getManager().addProperty("org.openmobileis.common.user.profil.xmlprofilfile.path",System.getProperty("user.dir") + "/WEB-INF/conf/defaultprofils.xml");
      SynchroTargerManager.getManager().addSynchroTargetListener(new TestOpenMSPSynchroTargetListener(), null, null);
      UserManager.getManager().registerUserManagerFactoryForGroup("default", new MonoUserUserManagerFactory());
      ProfilDataManager.getManager().registerProfilDataFactoryForGroup("default", new XmlFileProfilDataFactory());
      SynchroTargerManager.getManager().addSynchroTargetListener(new TestOpenMSPSynchroTargetListener(), null, null);
      SynchroTargerManager.getManager().registerProxyTargetForTerminal(new GenericProxySynchroTarget(), null);
      
      //start server
      Runnable startWebServer = new Runnable() {
  
        public void run() {
          try {
  
            System.out.println("start WebServer");
  
            Properties props = new Properties();
            props.put("LOGFILE", System.getProperty("user.dir") + "/WEB-INF/log/server/log");
            LogManager.registerLogManager(props); //log to a file
  
            WebServer server = null;
            server = new WebServer();
  
            server.serve();
          } catch (Throwable ex) {
            ex.printStackTrace();
          }
        }
  
      };
      Thread th = new Thread(startWebServer);
      th.start();
      //do synchro
      //wait 1 second until the webserver is started.
      Thread.currentThread().sleep(1000);
      Credential cred = new Credential("test", "test");
      SynchroDescriptor descriptor = new SynchroDescriptor();
      descriptor.setSynchroConduit(new HttpOpenMSPSynchroConduit(System.getProperty("user.dir") + "/synchro","OpenMSPPlug-WI-V1.0-WINCE[DR-FR-WINCE/04-02-00-00]"));
      descriptor.setSynchroGroup("default");
      descriptor.setServerURL("http://localhost:9091/services/openmspservice");
      
      SynchroManager.getManager().doSynchro(cred, descriptor);
      
    } catch (Throwable ex)  {
      LogManager.traceError(0, ex);
    }
  }

}
