/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.synchro.openmsp.protocol.test;

import org.openmobileis.common.util.codec.GeneralCoder;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.protocol.Command;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.DataItem;
import org.openmobileis.synchro.openmsp.protocol.Element;
import org.openmobileis.synchro.openmsp.protocol.Item;
import org.openmobileis.synchro.openmsp.protocol.RequestCommand;
import org.openmobileis.synchro.openmsp.protocol.Result;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.openmsp.server.synctarget.DefaultOpenMSPSynchroTargetListener;
import org.openmobileis.synchro.security.auth.Credential;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class TestOpenMSPSynchroTargetListener extends DefaultOpenMSPSynchroTargetListener {

  /**
   * 
   */
  public TestOpenMSPSynchroTargetListener() {
    super();
  }
  
  public String getTargetName() {
    return "testtarget";
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener#processMapCommand(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
   */
  protected ContainerMessage processMapCommand(Credential cred, ContainerMessage mapContainer) {
    return null;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener#notifySyncAction(long, org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.Command)
   */
  protected void notifySyncAction(long sessionId,Credential cred, Command syncCommand)  throws OpenMSPException  {
    LogManager.traceInfo(0, "TestOpenMSPSynchroTargetListener notifySyncAction META DATA :"+syncCommand.getMetaInformation());
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener#processResultCommand(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.Result)
   */
  protected void processResultCommand(Credential cred, Result resultCommande) throws OpenMSPException {
    // TODO Auto-generated method stub

  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener#processStatusCommand(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.Status)
   */
  protected void processStatusCommand(Credential cred, Status statusCommande) throws OpenMSPException {
    LogManager.traceInfo(0, "TestOpenMSPSynchroTargetListener processStatusCommand  status :"+statusCommande.getStatus()+" commandRef:"+statusCommande.getCmdRef());
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener#processAddCommand(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
   */
  protected ContainerMessage[] processGetCommand(Credential cred, ContainerMessage getContainer)  throws OpenMSPException {
    RequestCommand getCommand = (RequestCommand)getContainer.getElement();
    LogManager.traceInfo(0, "TestOpenMSPSynchroTargetListener processGetCommand META DATA :"+getCommand.getMetaInformation());
    DataItem item = (DataItem) getContainer.nextMessage().getElement();
    // get meta info
    // meta info are of type : AOUID:<PDAUID>contact
    try {
      String meta = new String(GeneralCoder.decodeBase64(item.getMetaInformation().getBytes()));
      String data = new String(GeneralCoder.decodeBase64(item.getData().getBytes()));
      LogManager.traceInfo(0, "TestOpenMSPSynchroTargetListener processGetCommand ITEM META DATA :"+meta);
      LogManager.traceInfo(0, "TestOpenMSPSynchroTargetListener processGetCommand ITEM DATA :"+data);     
    } catch (Throwable ex)  {
      LogManager.traceError(0, ex);
      Status status = new Status(getCommand.getCmdId(), Status.STATUS_FAILED);
      ContainerMessage[] containerms = new ContainerMessage[1];
      containerms[0] = new ContainerMessage(status);
      return containerms;
    }
    Status status = new Status(getCommand.getCmdId(), Status.STATUS_OK);
    ContainerMessage[] containerms = new ContainerMessage[2];
    containerms[0] = new ContainerMessage(status);
    Result result = new Result(getCommand.getCmdId(), "testtarget", "testlistener");
    result.setMetaInformation("RESULT META INFO");
    ContainerMessage newContainer = new ContainerMessage(result);
    Item newItem = new DataItem(
                Element.ITEM
                , new String("RESULT ITEM META")
                , new String("RESULT ITEM CONTENT")
                , null
                , null
            );
    newContainer.add(newItem);
    containerms[1] = newContainer;
    return containerms;
 }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener#processAddCommand(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
   */
  protected Status processAddCommand(Credential cred, ContainerMessage addContainer) throws OpenMSPException {
    RequestCommand addCommand = (RequestCommand)addContainer.getElement();
    LogManager.traceInfo(0, "TestOpenMSPSynchroTargetListener processAddCommand META DATA :"+addCommand.getMetaInformation());
    DataItem item = (DataItem) addContainer.nextMessage().getElement();
    // get meta info
    // meta info are of type : AOUID:<PDAUID>contact
    try {
      String meta = new String(GeneralCoder.decodeBase64(item.getMetaInformation().getBytes()));
      String data = new String(GeneralCoder.decodeBase64(item.getData().getBytes()));
      LogManager.traceInfo(0, "TestOpenMSPSynchroTargetListener processAddCommand ITEM META DATA :"+meta);
      LogManager.traceInfo(0, "TestOpenMSPSynchroTargetListener processAddCommand ITEM DATA :"+data);     
    } catch (Throwable ex)  {
      LogManager.traceError(0, ex);
      return new Status(addCommand.getCmdId(), Status.STATUS_FAILED);
    }
    return new Status(addCommand.getCmdId(), Status.STATUS_OK);
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener#processSyncItem(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.Item)
   */
  protected void processSyncItem(Credential cred, Item item) throws OpenMSPException {
    LogManager.traceInfo(0, "TestOpenMSPSynchroTargetListener processSyncItem ITEM :"+item.getCmdId());
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener#processDeleteCommand(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
   */
  protected Status processDeleteCommand(Credential cred, ContainerMessage deleteContainer) throws OpenMSPException {
    RequestCommand deleteCommand = (RequestCommand)deleteContainer.getElement();
    LogManager.traceInfo(0, "TestOpenMSPSynchroTargetListener processDeleteCommand META DATA :"+deleteCommand.getMetaInformation());
    DataItem item = (DataItem) deleteContainer.nextMessage().getElement();
    // get meta info
    // meta info are of type : AOUID:<PDAUID>contact
    try {
      String meta = new String(GeneralCoder.decodeBase64(item.getMetaInformation().getBytes()));
      LogManager.traceInfo(0, "TestOpenMSPSynchroTargetListener processDeleteCommand ITEM META DATA :"+meta);
    } catch (Throwable ex)  {
      LogManager.traceError(0, ex);
      return new Status(deleteCommand.getCmdId(), Status.STATUS_FAILED);
    }
    return new Status(deleteCommand.getCmdId(), Status.STATUS_OK);  
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener#processReplaceCommand(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
   */
  protected Status processReplaceCommand(Credential cred, ContainerMessage replaceContainer) throws OpenMSPException {
    RequestCommand replaceCommand = (RequestCommand)replaceContainer.getElement();
    LogManager.traceInfo(0, "TestOpenMSPSynchroTargetListener processReplaceCommand META DATA :"+replaceCommand.getMetaInformation());
    DataItem item = (DataItem) replaceContainer.nextMessage().getElement();
    // get meta info
    // meta info are of type : AOUID:<PDAUID>contact
    try {
      String meta = new String(GeneralCoder.decodeBase64(item.getMetaInformation().getBytes()));
      String data = new String(GeneralCoder.decodeBase64(item.getData().getBytes()));
      LogManager.traceInfo(0, "TestOpenMSPSynchroTargetListener processReplaceCommand ITEM META DATA :"+meta);
      LogManager.traceInfo(0, "TestOpenMSPSynchroTargetListener processReplaceCommand ITEM DATA :"+data);     
    } catch (Throwable ex)  {
      LogManager.traceError(0, ex);
      return new Status(replaceCommand.getCmdId(), Status.STATUS_FAILED);
    }
    return new Status(replaceCommand.getCmdId(), Status.STATUS_OK);  
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener#beginProcessSyncContainerMessage(org.openmobileis.synchro.security.auth.Credential)
   */
  protected void beginProcessOpenMSpCommand(Credential cred)  throws OpenMSPException {
    LogManager.traceInfo(0, "TestOpenMSPSynchroTargetListener beginProcessSyncContainerMessage.");
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener#endProcessSyncContainerMessage()
   */
  protected ContainerMessage[] endProcessOpenMSpCommand()  throws OpenMSPException  {
    LogManager.traceInfo(0, "TestOpenMSPSynchroTargetListener endProcessSyncContainerMessage.");
    ContainerMessage[] containerms = new ContainerMessage[1];
    Command getCommand = new Command(Element.GET, "testtarget", "testlistener");
    containerms[0] = new ContainerMessage(getCommand);
    return null;
  }

}
