/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.synchro.openmsp.protocol.test;

import org.openmobileis.common.util.codec.GeneralCoder;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.synchro.client.SynchroDescriptor;
import org.openmobileis.synchro.journal.JournalLogRenderer;
import org.openmobileis.synchro.journal.SimpleLogRenderer;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.client.DefaultOpenMSPSyncListener;
import org.openmobileis.synchro.openmsp.client.core.NumSyncManagerDB;
import org.openmobileis.synchro.openmsp.protocol.Command;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.DataItem;
import org.openmobileis.synchro.openmsp.protocol.Element;
import org.openmobileis.synchro.openmsp.protocol.Item;
import org.openmobileis.synchro.openmsp.protocol.Message;
import org.openmobileis.synchro.openmsp.protocol.RequestCommand;
import org.openmobileis.synchro.openmsp.protocol.Result;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.security.auth.Credential;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class TestOpenMSSyncListener extends DefaultOpenMSPSyncListener {
  private boolean syncGoOn = false;
  private Status resultStatus = null;

  /**
   * 
   */
  public TestOpenMSSyncListener() {
    super();
  }
  
  public void startSync(Credential cred, SynchroDescriptor synchrodescriptor) throws OpenMSPException   {
    super.startSync(cred, synchrodescriptor);
    syncGoOn = true;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#sendData(org.openmobileis.synchro.openmsp.protocol.Message)
   */
  public void sendData(Message message) throws OpenMSPException {
    if (resultStatus != null) {
      message.add(new ContainerMessage(resultStatus));
      resultStatus = null;
    }
    if (!syncGoOn) { // synchronize data only for the first call.
      return;
    }
    try {
      Command syncCommand = new Command(Element.SYNC, this.getSyncName(), "testtarget");
      long ns = NumSyncManagerDB.getManager().getSyncNumberForService(this.getSyncName());
      syncCommand.setSourceSessionID(ns);
      String commandMetadata = "testmetedata";
      if (commandMetadata != null)  {
        syncCommand.setMetaInformation(commandMetadata);
      }
      Credential cred = new Credential("test", "test");
      DefaultOpenMSPSyncListener.addCredentialToCommand(cred, syncCommand);
      ContainerMessage syncContainer = new ContainerMessage(syncCommand);

      // add ADD COMMANDE
      RequestCommand newCommand = new RequestCommand(Element.ADD);
      newCommand.setMetaInformation("ADD META INFO");
      ContainerMessage newContainer = new ContainerMessage(newCommand);
      Item newItem = new DataItem(
                  Element.ITEM
                  , new String(GeneralCoder.encodeBase64("AOUID:111".getBytes()))
                  , new String(GeneralCoder.encodeBase64("ADD COMMANDE CONTENT".getBytes()))
                  , null
                  , null
              );
      newContainer.add(newItem);
      syncContainer.add(newContainer);
      
      //add REPLACE COMMANDE
      newCommand = new RequestCommand(Element.REPLACE);
      newCommand.setMetaInformation("REPLACE META INFO");
      newContainer = new ContainerMessage(newCommand);
      newItem = new DataItem(
                  Element.ITEM
                  , new String(GeneralCoder.encodeBase64(("AOUID:222").getBytes()))
                  , new String(GeneralCoder.encodeBase64("REPLACE COMMANDE CONTENT".getBytes()))
                  , null
                  , null
              );
      newContainer.add(newItem);
      syncContainer.add(newContainer);

      //add DELETE COMMANDE
      newCommand = new RequestCommand(Element.DELETE);
      newContainer = new ContainerMessage(newCommand);
      newCommand.setMetaInformation("DELETE META INFO");
      newItem = new DataItem(
                  Element.ITEM
                  , new String(GeneralCoder.encodeBase64(("AOUID:3333").getBytes()))
                  , ""
                  , null
                  , null
              );
      newContainer.add(newItem);
      syncContainer.add(newContainer);
     
      //add GET COMMANDE
      newCommand = new RequestCommand(Element.GET);
      newCommand.setMetaInformation("GET META INFO");
      newContainer = new ContainerMessage(newCommand);
      newItem = new DataItem(
                  Element.ITEM
                  , new String("AOUID:GET")
                  , new String("GET COMMANDE CONTENT")
                  , null
                  , null
              );
      newContainer.add(newItem);
      syncContainer.add(newContainer);
      
      // add sync ITEM 
      newItem = new Item(
                Element.ITEM
                , null
                , null
              );
      syncContainer.add(newItem);
      message.add(syncContainer);
      
    } catch (Throwable ex)  {
      this.setSynchroStatus(Status.STATUS_FAILED);
      LogManager.trace(new SynchroException("DefaultDbSyncListener "+this.getSyncName()+" Send data exception ", ex));
    }
    syncGoOn = false;

  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#getSyncName()
   */
  public String getSyncName() {
    return "testlistener";
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#receiveMapCommand(org.openmobileis.synchro.openmsp.protocol.ContainerMessage, long)
   */
  public void receiveMapCommand(ContainerMessage mapContainer) throws OpenMSPException {
     LogManager.traceInfo(0, "Receive Map command :");
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#receiveResultCommand(org.openmobileis.synchro.openmsp.protocol.Result, long)
   */
  public void receiveResultCommand(ContainerMessage resultContainer, ContainerMessage initialCommand)  throws OpenMSPException  {
    Result result = (Result)resultContainer.getElement();
    LogManager.traceInfo(0, "TestOpenMSSyncListener receiveResultCommand META DATA :"+result.getMetaInformation());
    DataItem item = (DataItem) resultContainer.nextMessage().getElement();
    // get meta info
    // meta info are of type : AOUID:<PDAUID>contact
    try {
      String meta = item.getMetaInformation();
      String data = item.getData();
      LogManager.traceInfo(0, "TestOpenMSSyncListener receiveResultCommand ITEM META DATA :"+meta);
      LogManager.traceInfo(0, "TestOpenMSSyncListener receiveResultCommand ITEM DATA :"+data);     
      resultStatus =  new Status(result.getCmdId(), Status.STATUS_OK);
    } catch (Throwable ex)  {
      LogManager.traceError(0, ex);
      resultStatus = new Status(result.getCmdId(), Status.STATUS_FAILED);
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#receiveStatusCommand(org.openmobileis.synchro.openmsp.protocol.Status, long)
   */
  public void receiveStatusCommand( Status statusCommande, ContainerMessage initialCommand)  throws OpenMSPException  {
    LogManager.traceInfo(0, "TestOpenMSSyncListener receiveStatusCommand status :"+statusCommande.getStatus()+" commandRef:"+statusCommande.getCmdRef());
  }
  
  protected JournalLogRenderer getJournalLogRenderer()  {
    return new SimpleLogRenderer("test synchro", "service de test de synchro");
  }

}
