/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.synchro.openmsp.server.synctarget;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;

import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.codec.GeneralCoder;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.synchro.algo.replication.AtomicObjectArrayReplication;
import org.openmobileis.synchro.algo.replication.ReplicationReturnData;
import org.openmobileis.synchro.algo.replication.ReplicationSAOEvent;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;
import org.openmobileis.synchro.algo.replication.SynchroConflicResolver;
import org.openmobileis.synchro.algo.syncnumber.SyncNumberManager;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.client.db.DBImportFileCoder;
import org.openmobileis.synchro.openmsp.protocol.AbstractCommand;
import org.openmobileis.synchro.openmsp.protocol.Command;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.DataItem;
import org.openmobileis.synchro.openmsp.protocol.Element;
import org.openmobileis.synchro.openmsp.protocol.Result;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.openmsp.server.util.MemoryFile;
import org.openmobileis.synchro.openmsp.server.util.OpenMISFile;
import org.openmobileis.synchro.openmsp.server.util.ZipEntryMemoryFile;
import org.openmobileis.synchro.security.auth.Credential;

/**
 * Title: OpenMobileIS project source <BR>
 * Description:
 * 
 * @author Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public abstract class OpenMSPDBSynchroTargetListener implements OpenMSPSynchroTargetListener {

  /**
   * 
   */
  public OpenMSPDBSynchroTargetListener() {
    super();
  }

  public SyncTargetAnswer processCommand(Credential cred, ContainerMessage containerMessage) throws OpenMSPException {
    AbstractCommand syncCommand = (AbstractCommand) containerMessage.getElement();
    int commandType = syncCommand.getElementType();
    if (commandType != Element.SYNC) {
      SyncTargetAnswer answer = new SyncTargetAnswer();
      Status status = new Status(syncCommand.getCmdId(), Status.STATUS_WRONG_FORMAT);
      status.setCmdRef(syncCommand.getCmdId());
      ContainerMessage[] returnContainer = new ContainerMessage[1];
      returnContainer[0] = new ContainerMessage(status);
      answer.containerMessage = returnContainer;
      return answer;
    }

    // manage server connection
    try {
      this.connect(cred);
      try {

        Array returnList = new Array(5);
        boolean doClientUpdate = true; // if an error occurs during server update stop client update.
        // process incomming data
        Command sync = (Command) containerMessage.getElement();
        this.processSyncActionMetaData(sync.getMetaInformation());
        long sessionID = sync.getSourceSessionID();
        Array clientsaoList = new Array(10);
        Element getCommand = null;
        while (containerMessage.hasMoreMessage()) {
          ContainerMessage commandContainer = containerMessage.nextMessage();
          Element command = commandContainer.getElement();
          try {
            short actiontype = -1;
            DataItem item = (DataItem) commandContainer.nextMessage().getElement();
            String meta = new String(GeneralCoder.decodeBase64(item.getMetaInformation().getBytes()));
            String dataUID = "";
            if (meta.length() >0) dataUID =  meta.substring(6, meta.length());
            Object data = null;

            if (command.getElementType() == Element.ADD) {
              actiontype = SynchroAtomicObject.ADD;
              // data return [0] objID, [1] object serialized.
              data = DBImportFileCoder.getCoder().decodeData(new String(GeneralCoder.decodeBase64(item.getData().getBytes())));
            } else if (command.getElementType() == Element.DELETE) {
              actiontype = SynchroAtomicObject.DELETE;
              data = dataUID;
            } else if (command.getElementType() == Element.REPLACE) {
              // data return [0] objID, [1] object serialized.
              actiontype = SynchroAtomicObject.REPLACE;
              data = DBImportFileCoder.getCoder().decodeData(new String(GeneralCoder.decodeBase64(item.getData().getBytes())));
            } else if (command.getElementType() == Element.GET) {
              getCommand = command;
            }
            if (actiontype != -1) {
              SyncDBSynchroAtomicObject SAO = new SyncDBSynchroAtomicObject(dataUID, "");
              SAO.setModificationType(actiontype);
              SAO.setModifSyncNumber(sessionID);
              SAO.setSyncdata(data);
              SAO.setOpenMSPcmdId(command.getCmdId());
              clientsaoList.add(SAO);
            }
          } catch (Throwable ex) {
            LogManager.traceError(0, ex);
            Status status = new Status(command.getCmdId(), Status.STATUS_FAILED);
            status.setCmdRef(command.getCmdId());
            ContainerMessage statusContainer = new ContainerMessage(status);
            returnList.add(statusContainer);
            doClientUpdate = false;
          }
        }

        // manage synchro algo.
        AtomicObjectArrayReplication replication = new AtomicObjectArrayReplication(this.getSynchroConflicResolver());
        long newSessionId = SyncNumberManager.getManager().getNextSynchroNumber().getSynchroNumber();
        SynchroAtomicObject[] clientmodificationsoa = this.getAllModifiedAtomicObjectSince(sessionID);
        SynchroAtomicObject[] servermodificationsoa = new SynchroAtomicObject[clientsaoList.size()];
        clientsaoList.toArray(servermodificationsoa);
        ReplicationReturnData replicationData = replication.replicates(clientmodificationsoa, servermodificationsoa);

        // update client.
        ReplicationSAOEvent[] serverSideUpdates = replicationData.getServerSideModificationList();
        int clsize = serverSideUpdates.length;
        for (int i = 0; i < clsize; i++) {
          SyncDBSynchroAtomicObject syncObject = (SyncDBSynchroAtomicObject) serverSideUpdates[i].getAtomicObject();
          try {
            if ((serverSideUpdates[i].getTypeModif() == ReplicationSAOEvent.CREATE) || (serverSideUpdates[i].getTypeModif() == ReplicationSAOEvent.UPDATE)) {
              String[] updata = (String[]) syncObject.getSyncdata();
              Object obj = DBImportFileCoder.getCoder().unserializeDBObject(updata);
              this.updateTargetWithSynchroObject(obj);
            } else {
              this.deleteTargetForSynchroObjectId((String) syncObject.getSyncdata());
            }
            Status status = new Status(syncObject.getOpenMSPcmdId(), Status.STATUS_OK);
            status.setCmdRef(syncObject.getOpenMSPcmdId());
            ContainerMessage statusContainer = new ContainerMessage(status);
            returnList.add(statusContainer);
          } catch (Throwable ex) {
            LogManager.traceError(0, ex);
            Status status = new Status(syncObject.getOpenMSPcmdId(), Status.STATUS_FAILED);
            status.setCmdRef(syncObject.getOpenMSPcmdId());
            ContainerMessage statusContainer = new ContainerMessage(status);
            returnList.add(statusContainer);
            doClientUpdate = false;
          }
        }
        
        if (sessionID == 0) doClientUpdate = true; //force synchro complete.

        SyncTargetAnswer answer = new SyncTargetAnswer();
        // manager return sync. (answer to get commande)
        if ((getCommand != null) && (doClientUpdate)) {
          try {
            Array addFileRepository = new Array(15);
            int maxRow = this.getUpdateMaxNbRow();
            ReplicationSAOEvent[] clientSideUpdates = replicationData.getClientSideModificationList();
            if ((maxRow == -1) || (maxRow >= clientSideUpdates.length)) { // incremental
              // synchro
              // construct the result command with data file informations.
              // process PDA update
              byte[] encodedData = this.encodeSynchroReturnDataList(clientSideUpdates);

              String syncFileName = "/importdb/"+this.getTargetName();
              OpenMISFile file = new MemoryFile(syncFileName, encodedData);
              addFileRepository.add(file);
              DataItem newItem = new DataItem(Element.ITEM, "ImportDataFile", syncFileName, null, null);

              Result resultCommand = new Result(getCommand.getCmdId(), this.getTargetName(), sync.getSource());
              resultCommand.setMetaInformation(Long.toString(newSessionId));
              ContainerMessage resultContainer = new ContainerMessage(resultCommand);
              resultContainer.add(newItem);
              returnList.add(resultContainer);
            } else { // complete
              // construct the result command with db file informations.
              OpenMISFile[] files = this.getDatabaseImportFiles();
              if ((files != null) && (files.length > 0)) {
                Result resultCommand = new Result(getCommand.getCmdId(), this.getTargetName(), sync.getSource());
                resultCommand.setMetaInformation(Long.toString(newSessionId));
                ContainerMessage resultContainer = new ContainerMessage(resultCommand);
                DataItem newItem = null;
                for (int i = 0; i < files.length; i++) {
                  addFileRepository.add(files[i]);
                  String filename =  files[i].getFileCompleteName();
                  if (files[i] instanceof ZipEntryMemoryFile) {
                    filename = ((ZipEntryMemoryFile)files[i]).getZipFileName();
                  }
                  newItem = new DataItem(Element.ITEM, "DatabaseFile",filename, null, null);
                  resultContainer.add(newItem);
                }
                returnList.add(resultContainer);
              }

            }
            if (addFileRepository.size() > 0) {
              OpenMISFile[] addCyberFiles = new OpenMISFile[addFileRepository.size()];
              addFileRepository.toArray(addCyberFiles);
              answer.answerfiles = addCyberFiles;
            }
          } catch (Throwable ex) {
            LogManager.traceError(0, ex);
            Status status = new Status(getCommand.getCmdId(), Status.STATUS_FAILED);
            status.setCmdRef(getCommand.getCmdId());
            ContainerMessage statusContainer = new ContainerMessage(status);
            returnList.add(statusContainer);
          }
        }

        if (returnList.size() != 0) {
          ContainerMessage[] returnContainer = new ContainerMessage[returnList.size()];
          returnList.toArray(returnContainer);
          answer.containerMessage = returnContainer;
        }
        return answer;
      } finally {
        this.disconnect();
      }
    } catch (Throwable ex) {
      throw new OpenMSPException(ex);
    }
  }

  /**
   * Return the max number of row that update file can contains If more than max
   * row must be updated on the PDA, the database file will be send. If return
   * -1 (default implementation) max row in not taken into account (ie always
   * update file) If return 0 the database file is always send
   */
  protected int getUpdateMaxNbRow() {
    return -1;
  }

  /**
   * add to the specified collection all data (String[]) from uidList that must
   * be added to the PDA update datafile.
   */
  private byte[] encodeSynchroReturnDataList(ReplicationSAOEvent[] cserverUpdates) throws OpenMSPException, IOException {
    int size = cserverUpdates.length;
    ByteArrayOutputStream stream = new ByteArrayOutputStream();
    Writer writer = new OutputStreamWriter(stream);
    try {
      for (int i = 0; i < size; i++) {
        char[] encoded = null;
        if ((cserverUpdates[i].getTypeModif() == ReplicationSAOEvent.CREATE) ||(cserverUpdates[i].getTypeModif() == ReplicationSAOEvent.UPDATE))  {
          Object updateobject = this.getTargetObjectWithId(cserverUpdates[i].getAtomicObjectID()); 
          if (updateobject != null) {
            String[] toencode = DBImportFileCoder.getCoder().serializeDBObject(cserverUpdates[i].getAtomicObjectID(), updateobject);
            if (cserverUpdates[i].getTypeModif() == ReplicationSAOEvent.CREATE) {
              encoded = DBImportFileCoder.getCoder().encodeData(DBImportFileCoder.ADDAction, toencode);
            } else if (cserverUpdates[i].getTypeModif() == ReplicationSAOEvent.UPDATE) {
              encoded = DBImportFileCoder.getCoder().encodeData(DBImportFileCoder.REPLACEAction, toencode);
            }
          }
        } else  {
          String[] datadel = {cserverUpdates[i].getAtomicObjectID()};
          encoded = DBImportFileCoder.getCoder().encodeData(DBImportFileCoder.DELETEAction, datadel);
        }
        if (encoded != null) {
          writer.write(encoded);
        }
      }
    } finally {
      writer.flush();
      writer.close();
    }
    return stream.toByteArray();
  }

  /**
   * call to process sync command meta data Extends this method to manage sync
   * command meta data.
   */
  protected void processSyncActionMetaData(String metadata) throws ServiceException {

  }

  /**
   * return the target listener name
   */
  public abstract String getTargetName();

  /**
   * validate the credential and connect if needed
   */
  protected abstract void connect(Credential cred) throws UserNotFoundException, ServiceException;

  protected abstract void disconnect();

  protected abstract SynchroConflicResolver getSynchroConflicResolver();

  /**
   * return all atomic object uid modified since specified date for service
   */
  protected abstract SynchroAtomicObject[] getAllModifiedAtomicObjectSince(long syncNumber) throws OpenMSPException;

  protected abstract void updateTargetWithSynchroObject(Object syncObject) throws OpenMSPException;

  protected abstract void deleteTargetForSynchroObjectId(String uid) throws OpenMSPException;

  protected abstract Object getTargetObjectWithId(String uid) throws OpenMSPException;

  /**
   * call for complete DB synchro. return all the file needed to dump the
   * terminal db for this service.
   * 
   * @return
   * @throws ServiceException
   */
  protected abstract OpenMISFile[] getDatabaseImportFiles() throws ServiceException;

}
