/*
 *        OpenMobileIS - a free Java Framework for mobile applications
 *
 *   Copyright (C) 2002  Philippe Delrieu.
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Philippe Delrieu kept the rigth to distribute all code Copyrighted by philippe Delrieu
 *  under other licence term even commercial one.
 *  
 *  Modifications :
 *  2002 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.server.synctarget.proxy;

import org.openmobileis.common.user.profil.Profil;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.log.*;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.protocol.AbstractCommand;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.Message;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener;
import org.openmobileis.synchro.openmsp.server.synctarget.SyncTargetAnswer;
import org.openmobileis.synchro.openmsp.server.synctarget.SynchroTargerManager;
import org.openmobileis.synchro.openmsp.server.util.FileSystem;
import org.openmobileis.synchro.security.auth.Credential;
import org.openmobileis.common.context.*;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * 
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class GenericProxySynchroTarget implements ProxySyncroTarget {
  
  public GenericProxySynchroTarget() {
  }
  
  
  public void executeOpenMSPCommande(Credential cred, String listenerName, ContainerMessage container, Message answer, UserTerminal terminal, String version, Profil profil, FileSystem fileSystem) throws OpenMSPException  {
    // validate profil access
    if (!this.validateProfilAccess(listenerName, profil))  {
      Status status = new  Status (((AbstractCommand)(container.getElement())).getCmdId(), Status.STATUS_UNAUTHORIZED) ;
      answer.add(status);
      LogManager.traceInfo(0, "Error rubric not found in profil validation for synchro listener :"+listenerName);
      return;
    }
    OpenMSPSynchroTargetListener listener = SynchroTargerManager.getManager().getSynchroTargetListener(listenerName, version, terminal); //no version
    SyncTargetAnswer listenerAnswer = listener.processCommand(cred, container);   
    
    // process send files if any
    if (listenerAnswer.answerfiles != null)  {
        int size = listenerAnswer.answerfiles.length;
        for (int i=0; i<size; i++)  {
          try {
            fileSystem.addFile(listenerAnswer.answerfiles[i]);
          } catch (Exception ex) {
            LogManager.trace(new SynchroException("file not synchronized continue", ex));
            Status status = new  Status (((AbstractCommand)(container.getElement())).getCmdId(), Status.STATUS_FAILED) ;
            answer.add(status);
            return;
          }
        }
      }
      if (listenerAnswer.containerMessage != null)  {
        int size = listenerAnswer.containerMessage.length;
        for (int i=0; i<size; i++)  {
          answer.add(listenerAnswer.containerMessage[i]);
        }
     }
  }

  protected boolean validateProfilAccess(String listenerName, Profil profil) {
    int size = profil.profilRubric.length;
    for (int i=0; i<size; i++)  {
      if (listenerName.equals(profil.profilRubric[i].rubricName)) {
        return true;
      }
    }
    return false;
  }
}