/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.server.util;

import org.openmobileis.common.context.UserTerminal;
import org.openmobileis.common.util.exception.DefaultException;
import org.openmobileis.common.util.log.LogManager;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: manage a pool of virtual file system.
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class FileSystemObjectPoolSelecter {
  private static FileSystemObjectPoolSelecter manager;
  private FileSystemObjectPool memoryPool;

  private FileSystemObjectPoolSelecter()  throws ClassNotFoundException, IllegalAccessException, InstantiationException  {
    memoryPool = new FileSystemObjectPool("org.openmobileis.synchro.openmsp.server.util.MemoryFileSystem");
  }

  public final static FileSystemObjectPool getPool(UserTerminal terminal) {
    if (manager == null)  {
      synchronized (FileSystemObjectPool.class)  {
        if (manager == null)  {
          try {
            manager = new FileSystemObjectPoolSelecter();
          } catch (Exception e) {
            LogManager.trace(new DefaultException("Can't initialize FileSystemObject pool", e));

          }
        }
      }
    }
    FileSystemObjectPool pool = manager.memoryPool;
    return pool;
  }
}
