/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.synchro.sync4j.client;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.security.Principal;
import java.util.Date;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;
import org.openmobileis.database.fastobjectdb.synchro.client.SynchroFODBReturnListener;
import org.openmobileis.database.fastobjectdb.synchro.client.SynchroFastObjectDB;
import org.openmobileis.synchro.openmsp.client.db.ActionDB;
import org.openmobileis.synchro.openmsp.client.db.SyncActionDBManager;

import sync4j.framework.tools.Base64;
import sync4j.syncclient.spds.SyncException;
import sync4j.syncclient.spds.engine.SyncItem;
import sync4j.syncclient.spds.engine.SyncItemImpl;
import sync4j.syncclient.spds.engine.SyncItemProperty;
import sync4j.syncclient.spds.engine.SyncItemState;
import sync4j.syncclient.spds.engine.SyncSource;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Default Sync4j Source
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class Sync4jFODBSyncSource implements SyncSource {
  private static final int UNKNOWN=-1;
  
	private SynchroFODBReturnListener synchroListener;
  private String name;
  private String type;
  private String sourceURI;

  /**
   * 
   */
  public Sync4jFODBSyncSource() {
    super();
    // TODO Auto-generated constructor stub
  }
  
  public void registerSynchroListener(SynchroFODBReturnListener listener)	{
  	this.synchroListener = listener;
  }

  public String getName() {
      return name;
  }
  public void setName(String name) {
      this.name = name;
  }


  /** Getter for property uri.
   * @return Value of property uri.
   */
  public String getSourceURI() {
      return sourceURI;
  }

  /** Setter for property uri.
   * @param sourceURI New value of property uri.
   */
  public void setSourceURI(String sourceURI) {
      this.sourceURI = sourceURI;
  }

  public String getType() {
      return this.type;
  }
  
  public void setType(String type) {
      this.type = type;
  }

  /* (non-Javadoc)
   * @see sync4j.syncclient.spds.engine.SyncSource#getAllSyncItems(java.security.Principal)
   */
  public SyncItem[] getAllSyncItems(Principal arg0) throws SyncException {
    return this.getSyncItemForState(UNKNOWN);
  }

  /* (non-Javadoc)
   * @see sync4j.syncclient.spds.engine.SyncSource#getDeletedSyncItems(java.security.Principal, java.util.Date)
   */
  public SyncItem[] getDeletedSyncItems(Principal arg0, Date arg1) throws SyncException {
    return this.getSyncItemForState(ActionDB.DELETE_ACTION);
  }

  /* (non-Javadoc)
   * @see sync4j.syncclient.spds.engine.SyncSource#getNewSyncItems(java.security.Principal, java.util.Date)
   */
  public SyncItem[] getNewSyncItems(Principal arg0, Date arg1) throws SyncException {
    return this.getSyncItemForState(ActionDB.ADD_ACTION);
  }

  /* (non-Javadoc)
   * @see sync4j.syncclient.spds.engine.SyncSource#getUpdatedSyncItems(java.security.Principal, java.util.Date)
   */
  public SyncItem[] getUpdatedSyncItems(Principal arg0, Date arg1) throws SyncException {
    return this.getSyncItemForState(ActionDB.UPDATE_ACTION);
  }

  /* (non-Javadoc)
   * @see sync4j.syncclient.spds.engine.SyncSource#removeSyncItem(java.security.Principal, sync4j.syncclient.spds.engine.SyncItem)
   */
  public void removeSyncItem(Principal arg0, SyncItem syncItem) throws SyncException {
    try	{
	    String colid = syncItem.getKey().getKeyAsString();
	    String colname = null;
	    int index = colid.indexOf('/');
	    if (index != -1)	{
	      colname = colid.substring(0, index);
	      colid = colid.substring(index+1);
	    } else	{
	      throw new SyncException("Bad UID encoding for removeSyncItem.");
	    }
			SynchroFastObjectDB db = (SynchroFastObjectDB)FastObjectDBManager.getCurrentFODB();
			db.deleteWithIdFODB(colname, colid);
		} catch (Throwable ex)	{
			LogManager.traceError(0,ex);
			throw new SyncException(ex.getMessage());
		}	
 }

  /* (non-Javadoc)
   * @see sync4j.syncclient.spds.engine.SyncSource#setSyncItem(java.security.Principal, sync4j.syncclient.spds.engine.SyncItem)
   */
  public SyncItem setSyncItem(Principal arg0, SyncItem syncItem) throws SyncException {
    try	{
	    String colid = syncItem.getKey().getKeyAsString();
	    String colname = null;
	    int index = colid.indexOf('/');
	    if (index != -1)	{
	      colname = colid.substring(0, index);
	      colid = colid.substring(index+1);
	    } else	{
	      throw new SyncException("Bad UID encoding for setSyncItem.");
	    }
	    byte[] fileContent =
	        (byte[])syncItem.getPropertyValue(SyncItem.PROPERTY_BINARY_CONTENT);
	    fileContent = Base64.decode(fileContent);
	    Object newObj = convertSyncDataToObject(fileContent);
			if (this.synchroListener !=null)	{
			 	Object newOne = this.synchroListener.notifyBeginDBUpdate(newObj);
			 	if (newOne != null)	{
			 	 newObj = newOne;
			 	}
			}
			SynchroFastObjectDB db = (SynchroFastObjectDB)FastObjectDBManager.getCurrentFODB();
			boolean exist = db.isObjectInCollection(colname, newObj);
			if (exist)	{
				db.replaceFODB(colname, newObj);
			} else	{
				db.addFODB(colname, newObj);
			}
      SyncItem newSyncItem = new SyncItemImpl(this, colid, SyncItemState.NEW);
      newSyncItem.setProperties(syncItem.getProperties());

      return newSyncItem;		
    } catch (Throwable ex)	{
			LogManager.traceError(0,ex);
			throw new SyncException(ex.getMessage());
		}	
  }

  /* (non-Javadoc)
   * @see sync4j.syncclient.spds.engine.SyncSource#beginSync(int)
   */
  public void beginSync(int arg0) throws SyncException {

  }

  /* (non-Javadoc)
   * @see sync4j.syncclient.spds.engine.SyncSource#commitSync()
   */
  public void commitSync() throws SyncException {

  }
  
  protected char convertActionStateToSyncItem(ActionDB action)	{
    switch (action.getActionType())	{
  	case ActionDB.ADD_ACTION :
  	  return SyncItemState.NEW;
  	case ActionDB.UPDATE_ACTION :
  	  return SyncItemState.UPDATED;
  	case ActionDB.DELETE_ACTION :
  	  return SyncItemState.DELETED;
  	default :
  	  return SyncItemState.UNKNOWN;
    }
  }
  
  protected SyncItem encodeActionInSyncItem(ActionDB action) throws SyncException	{
    String objectUID = action.getActionObjectUID();
    SyncItemImpl syncItem = new SyncItemImpl(this, action.getActionServiceName()+"/"+objectUID, this.convertActionStateToSyncItem(action));
  	try	{
  	  if (action.getActionType() != ActionDB.DELETE_ACTION)	{
	  		Object obj = ((SynchroFastObjectDB)FastObjectDBManager.getCurrentFODB()).getObjectFromCollectionWithId(action.getActionServiceName(), objectUID);
	 			ByteArrayOutputStream bstr = new ByteArrayOutputStream();
				ObjectOutputStream ostr = new ObjectOutputStream(bstr);
				ostr.writeObject(obj);
				byte[] b = bstr.toByteArray();
	      syncItem.setProperty(
	          new SyncItemProperty(SyncItem.PROPERTY_BINARY_CONTENT         ,
	                               Base64.encode(b))
	      );
  	  }
		} catch (Throwable ex)	{
			LogManager.traceError(0,ex);
			throw new SyncException(ex.getMessage());
		}	
		return syncItem;
  }
  
  protected SyncItem[] getSyncItemForState(int state)	throws SyncException	{
    ActionDB[] actionArray = SyncActionDBManager.getManager().getActionsForService(this.getName());
    Array syncItemList = new Array(actionArray.length);
    for (int i=0; i<actionArray.length; i++)	{
      if ((state == UNKNOWN) || (actionArray[i].getActionType() == state)) {
        syncItemList.add(encodeActionInSyncItem(actionArray[i]));
      }
    }
    SyncItem[] ret = new SyncItem[syncItemList.size()];
    syncItemList.toArray(ret);
    return ret;
  }
  
	private Object convertSyncDataToObject(byte[] data) throws IOException , ClassNotFoundException	{
		ByteArrayInputStream bstr = new ByteArrayInputStream(data);
		ObjectInputStream ostr = new ObjectInputStream(bstr);
		return ostr.readObject();
	}

}
