/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.synchro.sync4j.server;

import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;

import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.fastobjectdb.db.test.synchro.sync4j.TestFODBSync4JSource;

import sync4j.framework.engine.source.ContentType;
import sync4j.framework.engine.source.SyncSourceInfo;
import sync4j.syncadmin.AdminException;
import sync4j.syncadmin.ui.SourceManagementPanel;

/**
 * @author Administrateur
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class FODBSync4JSourcePanel extends SourceManagementPanel implements Serializable {

  /**
   * Allowed characters for name and uri
   */
  public static final String NAME_ALLOWED_CHARS
  = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_.";

  // ------------------------------------------------------------ Private data
  /** label for the panel's name */
  private JLabel panelName = new JLabel();

  /** border to evidence the title of the panel */
  private TitledBorder  titledBorder1;

  private JLabel           nameLabel          = new JLabel()     ;
  private JTextField       nameValue          = new JTextField() ;
  private JLabel           typeLabel          = new JLabel()     ;
  private JTextField       typeValue          = new JTextField() ;
  private JLabel           sourceUriLabel     = new JLabel()     ;
  private JTextField       sourceUriValue     = new JTextField() ;

  private JButton          confirmButton      = new JButton()    ;

  private FODBSync4JSource  syncSource         = null             ;
  /**
   * 
   */
  public FODBSync4JSourcePanel() {
    super();
    System.out.println("Start FODBSync4JSourcePanel");
    notifyError(new AdminException("init client panel"));

    Properties props = new Properties();
    props.setProperty("LOGFILE", "c:\\logs4j.txt");
    LogManager.registerLogManager(props);
    this.init();
  }
  /**
   * Create the panel
   * @throws Exception if error occures during creation of the panel
   */
  private void init(){
    // set layout
    this.setLayout(null);

    // set properties of label, position and border
    //  referred to the title of the panel
    titledBorder1 = new TitledBorder("");

    panelName.setFont(titlePanelFont);
    panelName.setText("Edit Dummy SyncSource");
    panelName.setBounds(new Rectangle(14, 5, 316, 28));
    panelName.setAlignmentX(SwingConstants.CENTER);
    panelName.setBorder(titledBorder1);

    sourceUriLabel.setText("Source URI: ");
    sourceUriLabel.setFont(defaultFont);
    sourceUriLabel.setBounds(new Rectangle(14, 60, 150, 18));
    sourceUriValue.setFont(new java.awt.Font("Arial", 0, 12));
    sourceUriValue.setBounds(new Rectangle(170, 60, 350, 18));

    nameLabel.setText("Name: ");
    nameLabel.setFont(defaultFont);
    nameLabel.setBounds(new Rectangle(14, 90, 150, 18));
    nameValue.setFont(new java.awt.Font("Arial", 0, 12));
    nameValue.setBounds(new Rectangle(170, 90, 350, 18));

    typeLabel.setText("Type: ");
    typeLabel.setFont(defaultFont);
    typeLabel.setBounds(new Rectangle(14, 120, 150, 18));
    typeValue.setFont(new java.awt.Font("Arial", 0, 12));
    typeValue.setBounds(new Rectangle(170, 120, 350, 18));

    confirmButton.setFont(defaultFont);
    confirmButton.setText("Add");
    confirmButton.setBounds(170, 200, 70, 25);

    confirmButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent event ) {
            try {
                validateValues();
                getValues();
                if (getState() == STATE_INSERT) {
                  FODBSync4JSourcePanel.this.actionPerformed(new ActionEvent(FODBSync4JSourcePanel.this, ACTION_EVENT_INSERT, event.getActionCommand()));
                } else {
                  FODBSync4JSourcePanel.this.actionPerformed(new ActionEvent(FODBSync4JSourcePanel.this, ACTION_EVENT_UPDATE, event.getActionCommand()));
                }
            } catch (Exception e) {
                notifyError(new AdminException(e.getMessage()));
            }
        }
    });

    // add all components to the panel
    this.add(panelName        , null);
    this.add(nameLabel        , null);
    this.add(nameValue        , null);
    this.add(typeLabel        , null);
    this.add(typeValue        , null);
    this.add(sourceUriLabel   , null);
    this.add(sourceUriValue   , null);
    this.add(confirmButton    , null);

}

/**
 * Loads the given syncSource showing the name, uri and type in the panel's
 * fields.
 *
 * @param syncSource the SyncSource instance
 */
public void updateForm() {
     if (!(getSyncSource() instanceof TestFODBSync4JSource)) {
      notifyError(
          new AdminException(
              "This is not an DummySyncSource! Unable to process SyncSource values."
          )
      );
      return;
    }
    if (getState() == STATE_INSERT) {
      confirmButton.setText("Add");
    } else if (getState() == STATE_UPDATE) {
      confirmButton.setText("Save");
    }

    this.syncSource = (TestFODBSync4JSource) getSyncSource();

    sourceUriValue.setText(syncSource.getSourceURI() );
    nameValue.setText     (syncSource.getName()      );

    if (this.syncSource.getSourceURI() != null) {
        sourceUriValue.setEditable(false);
    }
}

// ----------------------------------------------------------- Private methods
/**
 * Checks if the values provided by the user are all valid. In caso of errors,
 * a IllegalArgumentException is thrown.
 *
 * @throws IllegalArgumentException if:
 *         <ul>
 *         <li>name, uri, type or directory are empty (null or zero-length)
 *         <li>the types list length does not match the versions list length
 *         </ul>
 */
private void validateValues() throws IllegalArgumentException {
    String value = null;

    value = nameValue.getText();


    value = typeValue.getText();

    value = sourceUriValue.getText();
}

/**
 * Set syncSource properties with the values provided by the user.
 */
private void getValues() {
    syncSource.setSourceURI      (sourceUriValue.getText().trim());
    syncSource.setName           (nameValue.getText().trim()     );
    syncSource.setType           (typeValue.getText().trim()     );

    ContentType[] contentTypes = new ContentType[] {
        new ContentType("text/plain", "1.0")
    };


    syncSource.setInfo(new SyncSourceInfo(contentTypes, 0));
}
}
